<?php
/**
 * Register settings
 *
 * @package     RCP\AWeberPro\Admin\Settings\Register
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Register the AWeber Pro settings
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_aweber_pro_register_settings() {
	register_setting( 'rcp_aweber_pro_settings_group', 'rcp_aweber_pro_settings', 'rcp_aweber_pro_sanitize_settings' );
}
add_action( 'admin_init', 'rcp_aweber_pro_register_settings', 100 );


/**
 * Sanitize settings
 *
 * @since       1.0.0
 * @param       array $settings The settings to save
 * @return      array $settings The settings to save
 */
function rcp_aweber_pro_sanitize_settings( $settings ) {
	$saved_settings = get_option( 'rcp_aweber_pro_settings' );

	// Delete transient on key changes
	if( isset( $saved_settings['auth_code'] ) ) {
		if( trim( $saved_settings['auth_code'] ) !== trim( $settings['auth_code'] ) ) {
			delete_transient( 'rcp_aweber_pro_lists');
			delete_option( 'rcp_aweber_pro_secrets' );
			delete_option( 'rcp_aweber_pro_status' );
		}
	}

	return $settings;
}


/**
 * Add the AWeber Pro menu item
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_aweber_pro_admin_menu() {
	add_submenu_page(
		'rcp-members',
		__( 'AWeber Pro Settings', 'rcp-aweber-pro' ),
		__( 'AWeber Pro', 'rcp-aweber-pro' ),
		'manage_options',
		'rcp-aweber-pro',
		'rcp_aweber_pro_render_settings_page'
	);
}
add_action( 'admin_menu', 'rcp_aweber_pro_admin_menu', 100 );


/**
 * Render the settings page
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_aweber_pro_render_settings_page() {
	$settings   = get_option( 'rcp_aweber_pro_settings' );
	$saved_list = isset( $settings['saved_list'] ) ? $settings['saved_list'] : false;

	if ( empty( $settings['default_checkbox_state'] ) ) {
		$settings['default_checkbox_state'] = 'checked';
	}

	echo '<div class="wrap">';
	echo '<h2>' . esc_html( get_admin_page_title() ) . '</h2>';

	if( isset( $_REQUEST['updated'] ) && $_REQUEST['updated'] !== false ) {
		echo '<div class="updated fade"><p><strong>' . __( 'Options saved', 'rcp-aweber-pro' ) . '</strong></p></div>';
	}
	?>

	<form method="post" action="options.php" class="rcp_options_form">
		<?php settings_fields( 'rcp_aweber_pro_settings_group' ); ?>
		<?php
			if( ! empty( $settings['auth_code'] ) ) {
				$lists = rcp_aweber_pro_get_lists();
			} else {
				$lists = false;
			}
		?>

		<table class="form-table">
			<tr>
				<th colspan="2">
					<h3><?php _e( 'Authentication', 'rcp-aweber-pro' ); ?></h3>
				</th>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[auth_code]"><?php _e( 'Authorization Code', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<input class="regular-text" type="text" id="rcp_aweber_pro_settings[auth_code]" name="rcp_aweber_pro_settings[auth_code]" value="<?php echo ( isset( $settings['auth_code'] ) ? $settings['auth_code'] : '' ); ?>" />
					<div class="description"><?php printf( __( 'Enter your %s. Link opens in new window.', 'rcp-aweber-pro' ), sprintf( '<a href="%s" target="_blank">%s</a>', 'https://auth.aweber.com/1.0/oauth/authorize_app/32affb42', __( 'AWeber Authorization Code', 'rcp-aweber-pro' ) ) ); ?></div>
				</td>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[api_status]"><?php _e( 'API Status', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<?php
						$api_status = get_option( 'rcp_aweber_pro_status' );

						if( $api_status ) {
							echo $api_status;
						} else {
							_e( 'Not Connected', 'rcp-aweber-pro' );
						}
					?>
				</td>
			</tr>
			<tr>
				<th colspan="2">
					<h3><?php _e( 'Configuration', 'rcp-aweber-pro' ); ?></h3>
				</th>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[saved_list]"><?php _e( 'Default List', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<?php
					if( $lists ) {
						if( ! is_array( $lists ) ) {
							echo '<strong>' . __( 'Error:', 'rcp-aweber-pro' ) . '</strong> ' . $lists;
						} else { ?>
							<select id="rcp_aweber_pro_settings[saved_list]" name="rcp_aweber_pro_settings[saved_list]">
								<?php
								foreach( $lists as $list_id => $list_name ) {
									echo '<option value="' . esc_attr( $list_id ) . '"' . selected( $saved_list, $list_id, false ) . '>' . esc_html( $list_name ) . '</option>';
								} ?>
							</select>
							<div class="description"><?php _e( 'Choose the list to subscribe users to if no per-level list is selected.', 'rcp-aweber-pro' ); ?></div>
						<?php }
					} else {
						echo __( 'Enter your authorization code to select a default list.', 'rcp-aweber-pro' );
					} ?>
				</td>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[double_subscribe]"><?php _e( 'Double Subscribe', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<input type="checkbox" id="rcp_aweber_pro_settings[double_subscribe]" name="rcp_aweber_pro_settings[double_subscribe]" value="1" <?php echo( isset( $settings['bypass_optin'] ) ? checked( $settings['bypass_optin'], 1, false ) : '' ); ?>/>
					<span class="description"><?php _e( 'Check to subscribe to BOTH the default and per-level lists when possible.', 'rcp-aweber-pro' ); ?></span>
				</td>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[signup_label]"><?php _e( 'Form Label', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<input class="regular-text" type="text" id="rcp_aweber_pro_settings[signup_label]" name="rcp_aweber_pro_settings[signup_label]" value="<?php echo ( isset( $settings['signup_label'] ) && ! empty( $settings['signup_label'] ) ? $settings['signup_label'] : __( 'Signup for Newsletter', 'rcp-aweber-pro' ) ); ?>" />
					<div class="description"><?php _e( 'Enter the label to be used for the "Signup for Newsletter" checkbox.', 'rcp-aweber-pro' ); ?></div>
				</td>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[auto_subscribe]"><?php _e( 'Auto Subscribe', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<input type="checkbox" id="rcp_aweber_pro_settings[auto_subscribe]" name="rcp_aweber_pro_settings[auto_subscribe]" value="1" <?php echo ( isset( $settings['auto_subscribe'] ) ? checked( $settings['auto_subscribe'], 1, false ) : '' ); ?> />
					<span class="description"><?php _e( 'Check to hide the subscribe checkbox and automatically subscribe users.', 'rcp-aweber-pro' ); ?></span>
				</td>
			</tr>
			<tr>
				<th>
					<label for="rcp_aweber_pro_settings[default_checkbox_state]"><?php _e( 'Opt-In Checkbox Default State', 'rcp-aweber-pro' ); ?></label>
				</th>
				<td>
					<select id="rcp_aweber_pro_settings[default_checkbox_state]" name="rcp_aweber_pro_settings[default_checkbox_state]">
						<option value="checked" <?php selected( 'checked' == $settings['default_checkbox_state'] ); ?>><?php _e( 'Checked', 'rcp-aweber-pro' ); ?></option>
						<option value="unchecked" <?php selected( $settings['default_checkbox_state'], 'unchecked' ); ?>><?php _e( 'Unchecked', 'rcp-aweber-pro' ); ?></option>
					</select>
					<div class="description"><?php _e( 'Choose whether you want the opt-in on the registration form checked or unchecked by default.', 'rcp-aweber-pro' ); ?></div>
				</td>
			</tr>
		</table>

		<p class="submit">
			<input type="submit" class="button-primary" value="<?php _e( 'Save Options', 'rcp-aweber-pro' ); ?>" />
		</p>
	</form>
	<?php
	echo '</div>';
}


/**
 * Add per-level setting fields
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_aweber_pro_add_subscription_settings() {
	?>
		<tr class="form-field">
			<th scope="row" valign="top">
				<label for="rcp-aweber-pro-list"><?php _e( 'AWeber List', 'rcp-aweber-pro' ); ?></label>
			</th>
			<td>
				<?php
				$lists = rcp_aweber_pro_get_lists();

				if( isset( $_GET['edit_subscription'] ) ) {
					$subscription_lists = get_option( 'rcp_aweber_pro_subscription_lists' );

					if( is_array( $subscription_lists ) && array_key_exists( $_GET['edit_subscription'], $subscription_lists ) ) {
						$saved_list = $subscription_lists[$_GET['edit_subscription']];
					} else {
						$saved_list = '';
					}
				} else {
					$saved_list = '';
				}

				if( $lists ) {
					if( ! is_array( $lists ) ) {
						echo '<strong>' . __( 'Error: ', 'rcp-aweber-pro' ) . '</strong> ' . $lists;
					} else { ?>
						<select name="aweber-list" id="rcp-aweber-pro-list">
							<option value="inherit"<?php echo selected( $saved_list, 'inherit', false ); ?>><?php _e( 'Use System Default', 'rcp-aweber-pro' ); ?></option>
							<?php
							foreach( $lists as $list_id => $list_name ) {
								echo '<option value="' . esc_attr( $list_id ) . '"' . selected( $saved_list, $list_id, false ) . '>' . esc_html( $list_name ) . '</option>';
							} ?>
						</select>
						<p class="description"><?php _e( 'The AWeber list to subscribe users to at this level.', 'rcp-aweber-pro' ); ?></p>
					<?php }
				} else {
					echo __( 'Enter an API key to select a default list.', 'rcp-aweber-pro' );
				}
				?>
			</td>
		</tr>
	<?php
}
add_action( 'rcp_add_subscription_form', 'rcp_aweber_pro_add_subscription_settings' );
add_action( 'rcp_edit_subscription_form', 'rcp_aweber_pro_add_subscription_settings' );


/**
 * Store the AWeber list in subscription meta
 *
 * @since       1.0.0
 * @param       int $level_id The subscription ID
 * @param       array $args Arguements passed to the action
 */
function rcp_aweber_pro_save_subscription( $level_id = 0, $args ) {
	if( ! empty( $_POST['aweber-list'] ) ) {
		$subscription = get_option( 'rcp_aweber_pro_subscription_lists' );
		$subscription[$level_id] = $_POST['aweber-list'];

		update_option( 'rcp_aweber_pro_subscription_lists', $subscription );
	}
}
add_action( 'rcp_add_subscription', 'rcp_aweber_pro_save_subscription', 10, 2 );
add_action( 'rcp_edit_subscription_level', 'rcp_aweber_pro_save_subscription', 10, 2 );