<?php
/**
 * AWeber API Handler
 *
 * @package     RCP\AWeberPro\API
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * AWeber API handler class
 *
 * @since       1.0.0
 */
class RCP_AWeber_Pro_API {


	/**
	 * @var         object $api The AWeber API object
	 * @since       1.0.0
	 */
	public $api;


	/**
	 * @var         array $settings AWeber Pro settings
	 * @since       1.0.0
	 */
	public $settings;


	/**
	 * @var         array $lists Per-level list settings
	 * @since       1.0.0
	 */
	public $lists;


	/**
	 * Get things started
	 *
	 * @access      public
	 * @since       1.0.0
	 * @return      void
	 */
	public function __construct() {
		$this->setup_api();

		// Maybe add a signup flag.
		add_action( 'rcp_form_processing', array( $this, 'maybe_signup' ), 10, 8 );

		if ( function_exists( 'rcp_get_membership' ) ) {
			// Maybe subscribe to list.
			add_action( 'rcp_successful_registration', array( $this, 'maybe_subscribe' ), 10, 3 );
		} else {
			add_action( 'rcp_set_status', array( $this, 'deprecated_maybe_subscribe' ), 10, 4 );
		}
	}


	/**
	 * Setup the API object
	 *
	 * @access      private
	 * @since       1.0.0
	 * @return      void
	 */
	private function setup_api() {
		if( ! $this->api ) {
			$this->settings = get_option( 'rcp_aweber_pro_settings' );
			$this->lists    = get_option( 'rcp_aweber_pro_subscription_lists' );

			if( ! empty( $this->settings['auth_code'] ) ) {
				$return     = '';
				$error_code = '';
				$secrets    = get_option( 'rcp_aweber_pro_secrets' );

				if( $secrets !== false ) {
					try {
						$this->api = new AWeberAPI( $secrets['consumer_key'], $secrets['consumer_secret'] );

						$return = __( 'Connected', 'rcp-aweber-pro' );
					} catch( AWeberAPIException $e ) {
						$this->api = false;
					}
				} else {
					try {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = AWeberAPI::getDataFromAweberID( $this->settings['auth_code'] );
					} catch( AWeberAPIException $e ) {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = null;

						$desc = $e->getMessage();
						$desc = preg_replace( '/http.*$/i', '', $desc );
						$desc = preg_replace( '/[\.\!:]+.*$/i', '', $desc );

						$error_code = ' (' . $desc . ')';
					} catch( AWeberOAuthDataMissing $e ) {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = null;
					} catch( AWeberException $e ) {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = null;
					}

					if( ! $access_secret ) {
						$return = sprintf( __( 'Unable to connect to your AWeber account%s<br />', 'rcp-aweber-pro' ), $error_code );

						# Show oauth_id if it failed and an api exception was not raised
						if( empty( $error_code ) ) {
							$return .= sprintf( __( 'The provided authorization code was: %s<br />', 'rcp-aweber-pro' ), $this->settings['auth_code'] );
						}

						$return .= __( 'Please verify your authorization code and try again.', 'rcp-aweber-pro' );
					} else {
						$secrets = array(
							'consumer_key'    => $consumer_key,
							'consumer_secret' => $consumer_secret,
							'access_key'      => $access_key,
							'access_secret'   => $access_secret
						);

						update_option( 'rcp_aweber_pro_secrets', $secrets );

						$this->api = new AWeberAPI( $consumer_key, $consumer_secret );

						$return = __( 'Connected', 'rcp-aweber-pro' );
					}
				}
			} else {
				delete_option( 'rcp_aweber_pro_secrets' );
			}

			$return = ! empty( $return ) ? $return : __( 'Not Connected', 'rcp-aweber-pro' );

			update_option( 'rcp_aweber_pro_status', $return );
		}
	}


	/**
	 * Retrieve the list to signup for
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       int $level_id The ID of the subscription level to lookup
	 * @return      string $list The AWeber list to signup for
	 */
	public function get_list( $level_id ) {
		if( is_array( $this->lists ) && array_key_exists( $level_id, $this->lists ) && $this->lists[$level_id] !== 'inherit' ) {
			$list = $this->lists[$level_id];
		} else {
			$list = $this->settings['saved_list'];
		}

		return apply_filters( 'rcp_aweber_pro_get_list', $list, $level_id );
	}


	/**
	 * Check if a given member is subscribed on AWeber
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       string $email The email address to lookup
	 * @param       string $list The list to check
	 * @return      bool $member_exists True if subscribed, false otherwise
	 */
	public function member_exists( $email, $list ) {
		$secrets = get_option( 'rcp_aweber_pro_secrets' );

		try {
			$account     = $this->api->getAccount( $secrets['access_key'], $secrets['access_secret'] );
			$list_url    = '/accounts/' . $account->id . '/lists/4156144';
			$list        = $account->loadFromUrl( $list_url );
			$subscribers = $list->subscribers;

			if( $subscribers->data['total_size'] > 0 ) {
				foreach( $subscribers->data['entries'] as $subscriber ) {
					if( $subscriber['email'] == $email ) {
						$member_exists = true;
						break;
					}
				}
			}
		} catch( Exception $e ) {
			$member_exists = false;
		}

		return $member_exists;
	}


	/**
	 * Subscribe an email to AWeber
	 *
	 * @access      public
	 * @since       1.0.0
	 *
	 * @param       string         $email      The email address to subscribe
	 * @param       int            $user_id    The ID of the user to subscribe
	 * @param RCP_Membership|false $membership Membership object.
	 *
	 * @return      bool True if added successfully, false otherwise
	 */
	public function subscribe( $email, $user_id, $membership = false ) {

		$return              = false;
		$secrets             = get_option( 'rcp_aweber_pro_secrets' );
		$saved_list          = $this->settings['saved_list'];
		$membership_level_id = is_a( $membership, 'RCP_Membership' ) ? $membership->get_object_id() : rcp_get_subscription_id( $user_id );
		$level_list          = $this->get_list( $membership_level_id );
		$fname               = isset( $_POST['rcp_user_first'] ) ? sanitize_text_field( $_POST['rcp_user_first'] ) : '';
		$lname               = isset( $_POST['rcp_user_last'] ) ? sanitize_text_field( $_POST['rcp_user_last'] ) : '';

		$name = trim( sprintf( '%s %s', $fname, $lname ) );

		$params = array( 'email' => $email, 'ip_address' => rcp_get_ip() );

		if ( ! empty( $name ) ) {
			$params['name'] = $name;
		}

		try {
			$account  = $this->api->getAccount( $secrets['access_key'], $secrets['access_secret'] );
			$list_url = '/accounts/' . $account->id . '/lists/' . $level_list;
			$list     = $account->loadFromUrl( $list_url );

			$subscribers    = $list->subscribers;
			$new_subscriber = $subscribers->create( $params );
		} catch( Exception $e ) {
			$return = false;
		}

		if( $return === false ) {
			return false;
		} else {
			if( isset( $this->settings['double_subscribe'] ) && $saved_list != $level_list ) {
				try {
					$account  = $this->api->getAccount( $secrets['access_key'], $secrets['access_secret'] );
					$list_url = '/accounts/' . $account->id . '/lists/' . $level_list;
					$list     = $account->loadFromUrl( $list_url );

					$subscribers    = $list->subscribers;
					$new_subscriber = $subscribers->create( $params );
				} catch( Exception $e ) {
					$return = false;
				}
			}

			if( $return === false ) {
				return false;
			} else {
				return true;
			}
		}
	}


	/**
	 * Add a "pending signup" flag if the user has opted in to the mailing list.
	 *
	 * @access      public
	 * @since       1.0.0
	 *
	 * @param array                $posted              The fields posted by the submission form
	 * @param int                  $user_id             The ID of this user
	 * @param float                $price               Price of the membership.
	 * @param RCP_Customer|false   $customer            Customer object.
	 * @param int                  $membership_id       ID of the membership for this registration.
	 * @param RCP_Membership|false $previous_membership Previous membership, if this is an upgrade/downgrade.
	 * @param string               $registration_type   Type of registration (new, upgrade, downgrade, renewal).
	 *
	 * @return      void
	 */
	public function maybe_signup( $posted, $user_id, $price = 0.00, $payment_id = 0, $customer = false, $membership_id = 0, $previous_membership = false, $registration_type = 'new' ) {

		if ( function_exists( 'rcp_add_membership_meta' ) && ! empty( $membership_id ) ) {
			if ( isset( $posted['rcp_aweber_pro_signup'] ) ) {
				rcp_add_membership_meta( $membership_id, 'aweber_pending_signup', true );
			} else {
				rcp_delete_membership_meta( $membership_id, 'aweber_pending_signup' );
			}
		} else {
			if ( isset( $posted['rcp_aweber_pro_signup'] ) ) {
				// Set a flag so we know to add them to the list after account activation.
				update_user_meta( $user_id, 'rcp_aweber_pending_signup', true );
			} else {
				delete_user_meta( $user_id, 'rcp_aweber_pending_signup' );
			}
		}

	}

	/**
	 * Add user to mailing list if they have the pending flag.
	 *
	 * @param RCP_Member           $user
	 * @param RCP_Customer|false   $customer
	 * @param RCP_Membership|false $membership
	 *
	 * @since 1.1.4
	 * @return void
	 */
	public function maybe_subscribe( $user, $customer = false, $membership = false ) {

		if ( ! is_a( $membership, 'RCP_Membership' ) ) {
			return;
		}

		if ( ! rcp_get_membership_meta( $membership->get_id(), 'aweber_pending_signup', true ) ) {
			return;
		}

		$user = get_userdata( $membership->get_customer()->get_user_id() );

		if ( empty( $user ) ) {
			return;
		}

		$subscribed = $this->subscribe( $user->user_email, $user->ID, $membership );

		if ( $subscribed ) {
			update_user_meta( $user->ID, 'rcp_subscribed_to_aweber', 'yes' );
			rcp_delete_membership_meta( $membership->get_id(), 'aweber_pending_signup' );
		}

	}

	/**
	 * Add member to the Aweber list when their account is activated.
	 *
	 * Used for RCP 2.9 and lower only.
	 *
	 * @param string     $status     New status.
	 * @param int        $user_id    ID of the user.
	 * @param string     $old_status Previous status.
	 * @param RCP_Member $member     Member object.
	 *
	 * @since  1.1.4
	 * @return void
	 */
	public function deprecated_maybe_subscribe( $status, $user_id, $old_status, $member ) {

		// Handles version 2.9 signups only.
		if ( function_exists( 'rcp_get_membership' ) ) {
			return;
		}

		if ( ! in_array( $status, array( 'active', 'free' ) ) ) {
			return;
		}

		if ( ! get_user_meta( $user_id, 'rcp_aweber_pending_signup', true ) ) {
			return;
		}

		$subscribed = $this->subscribe( $member->user_email, $user_id );

		if ( $subscribed ) {
			update_user_meta( $user_id, 'rcp_subscribed_to_aweber', 'yes' );
			delete_user_meta( $user_id, 'rcp_aweber_pending_signup' );
		}

	}
}
