<?php
/**
 * ConvertKit API Handler
 *
 * @package     RCP\ConvertKit\API
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
    exit;
}


/**
 * ConvertKit API handler class
 *
 * @since       1.0.0
 */
class RCP_ConvertKit_API {


	/**
	 * @var         string $api_key The ConvertKit API key
	 * @since       1.0.0
	 */
	public $api_key;


	/**
	 * @var         string $api_secret The ConvertKit API secret
	 * @since       1.0.0
	 */
	public $api_secret;


	/**
	 * @var         string $api_url The ConvertKit API URL
	 * @since       1.0.0
	 */
	public $api_url = 'https://api.convertkit.com/v3/';


	/**
	 * @var         array $settings MailChimp Pro settings
	 * @since       1.0.0
	 */
	public $settings;


	/**
	 * @var         array $lists Available lists
	 * @since       1.0.0
	 */
	public $lists;


	/**
	 * @var         array $tags Available tags
	 * @since       1.0.0
	 */
	public $tags;


	/**
	 * Get things started
	 *
	 * @access      public
	 * @since       1.0.0
	 * @return      void
	 */
	public function __construct() {
		$this->setup_api();

		// Maybe set flag to subscribe user
		add_action( 'rcp_form_processing', array( $this, 'maybe_signup' ), 10, 8 );

		if ( function_exists( 'rcp_get_membership') ) {
			// Subscribe to list.
			add_action( 'rcp_transition_membership_status_active', array( $this, 'maybe_subscribe' ), 10, 2 );

			// Apply or remove expiration tag.
			add_action( 'rcp_transition_membership_status', array( $this, 'maybe_add_remove_expiration_tag' ), 10, 3 );
		} else {
			// Maybe subscribe user.
			add_action( 'rcp_set_status', array( $this, 'maybe_add_to_list' ), 10, 4 );

			// Apply or remove expiration tag.
			add_action( 'rcp_set_status', array( $this, 'add_remove_expiration_tag' ), 10, 4 );
		}

		// Update ConvertKit when user changes email
		add_action( 'profile_update', array( $this, 'update_subscription' ), 10, 2 );

		// Payments API.
		add_action( 'rcp_update_payment_status_complete', array( $this, 'insert_payment' ) );
	}


	/**
	 * Setup the API object
	 *
	 * @access      private
	 * @since       1.0.0
	 * @return      void
	 */
	private function setup_api() {
		if( ! $this->api_key || ! $this->api_secret ) {
			$this->settings = get_option( 'rcp_convertkit_settings' );

			if( ! empty( $this->settings['api_key'] ) && ! empty( $this->settings['api_secret'] ) ) {
				$this->api_key    = $this->settings['api_key'];
				$this->api_secret = $this->settings['api_secret'];
			}
		}
	}


	/**
	 * Retrieve the available lists
	 *
	 * @param bool $cache Whether or not to get cached results.
	 *
	 * @access      public
	 * @since       1.0.0
	 * @return      array
	 */
	public function get_lists( $cache = true ) {
		if( $this->api_key ) {
			$lists = $cache ? get_transient( 'rcp_convertkit_list_data' ) : false;
			$url   = $this->api_url . 'forms?api_key=' . $this->api_key;

			if( $lists === false ) {
				$request = wp_remote_get( $url );

				if( ! is_wp_error( $request ) && wp_remote_retrieve_response_code( $request ) == 200 ) {
					$lists = json_decode( wp_remote_retrieve_body( $request ) );

					set_transient( 'rcp_convertkit_list_data', $lists, 24*24*24 );
				} elseif ( function_exists( 'rcp_log' ) ) {
					rcp_log( 'Error fetching ConvertKit lists.' );
				}
			}

			if( ! empty( $lists ) && ! empty( $lists->forms ) ) {
				foreach( $lists->forms as $key => $form ) {
					$this->lists[$form->id] = $form->name;
				}
			}
		}

		return (array) $this->lists;
	}


	/**
	 * Retrieve the available tags
	 *
	 * @access      public
	 * @since       1.0.0
	 * @return      array
	 */
	public function get_tags() {
		if( $this->api_key ) {
			delete_transient( 'rcp_convertkit_tag_data' );
			$tags = get_transient( 'rcp_convertkit_tag_data' );
			$url   = $this->api_url . 'tags?api_key=' . $this->api_key;

			if( $tags === false ) {
				$request = wp_remote_get( $url );

				if( ! is_wp_error( $request ) && wp_remote_retrieve_response_code( $request ) == 200 ) {
					$tags = json_decode( wp_remote_retrieve_body( $request ) );

					set_transient( 'rcp_convertkit_tag_data', $tags, 24*24*24 );
				} elseif ( function_exists( 'rcp_log' ) ) {
					rcp_log( 'Error fetching ConvertKit tags.' );
				}
			}

			if( ! empty( $tags ) && ! empty( $tags->tags ) ) {
				foreach( $tags->tags as $key => $tag ) {
					$this->tags[$tag->id] = $tag->name;
				}
			}
		}

		return (array) $this->tags;
	}


	/**
	 * Retrieve the list to signup for
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       int $level_id The ID of the subscription level to lookup
	 * @return      string $list The MailChimp list to signup for
	 */
	public function get_list( $level_id ) {
		$lists = get_option( 'rcp_convertkit_subscription_lists' );

		if( is_array( $lists ) && array_key_exists( $level_id, $lists ) && $lists[$level_id] !== 'inherit' ) {
			$list = $lists[$level_id];
		} else {
			$list = $this->settings['saved_list'];
		}

		return apply_filters( 'rcp_convertkit_get_list', $list, $level_id );
	}


	/**
	 * Subscribe an email to MailChimp
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       string $email The email address to subscribe
	 * @param       string $name The name of the user to subscribe
	 * @param       int $subscription_level_id ID of the user's subscription level. Used for getting the list ID.
	 * @return      bool True if added successfully, false otherwise
	 */
	public function subscribe( $email, $name, $subscription_level_id = 0 ) {
		$return = false;
		$list   = $this->get_list( $subscription_level_id );
		$tags   = apply_filters( 'rcp_convertkit_subscribe_tags', get_option( 'rcp_convertkit_subscription_tags' ), $email, $subscription_level_id );

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Subscribing user to ConvertKit list %s with tags %s.', $list, var_export( $tags, true ) ) );
		}

		$args = apply_filters( 'rcp_convertkit_subscribe_vars', array(
			'email'      => $email,
			'first_name' => $name
		) );

		$request = wp_remote_post(
			$this->api_url . 'forms/' . $list . '/subscribe?api_key=' . $this->api_key,
			array(
				'body'    => $args,
				'timeout' => 30
			)
		);

		if( ! is_wp_error( $request ) && wp_remote_retrieve_response_code( $request ) == 200 ) {
			if( is_array( $tags ) && array_key_exists( $subscription_level_id, $tags ) ) {
				foreach( $tags[ $subscription_level_id ] as $tag ) {
					$request = wp_remote_post(
						$this->api_url . 'tags/' . $tag . '/subscribe?api_key=' . $this->api_key,
						array(
							'body'    => $args,
							'timeout' => 30
						)
					);
				}
			}

			return true;
		}

		if ( function_exists( 'rcp_log' ) ) {
			$message = is_wp_error( $request ) ? $request->get_error_message() : 'Unknown';
			rcp_log( sprintf( 'Error subscribing user to ConvertKit list %s. Response code: %d; Error message: %s.', $list, wp_remote_retrieve_response_code( $request ), $message ) );
		}

		return false;
    }


    /**
     * Set a flag for ConverKit signup if they've opted in during registration.
     * Actual signup occurs when membership is activated.
     *
     * @see RCP_ConvertKit_API::maybe_add_to_list()
     *
     * @access      public
     * @since       1.0.0
     * @param       array                $posted              The fields posted by the submission form
     * @param       int                  $user_id             The ID of this user
     * @param       float                $price               Price of the membership.
     * @param       RCP_Customer|false   $customer            Customer object.
     * @param       int                  $membership_id       ID of the membership for this registration.
     * @param       RCP_Membership|false $previous_membership Previous membership, if this is an upgrade/downgrade.
     * @param       string               $registration_type   Type of registration (new, upgrade, downgrade, renewal).
     * @return      void
     */
    public function maybe_signup( $posted, $user_id, $price = 0.00, $payment_id = 0, $customer = false, $membership_id = 0, $previous_membership = false, $registration_type = 'new' ) {

	    if ( function_exists( 'rcp_add_membership_meta' ) && ! empty( $membership_id ) ) {

	    	/**
		     * RCP 3.0+
		     */
		    if ( isset( $posted['rcp_convertkit_signup'] ) ) {
			    rcp_add_membership_meta( $membership_id, 'convertkit_pending_signup', true );
		    } else {
			    rcp_delete_membership_meta( $membership_id, 'convertkit_pending_signup' );
		    }

	    } else {

		    /**
		     * RCP 2.9 and lower
		     */
		    if ( isset( $posted['rcp_convertkit_signup'] ) ) {
			    if ( function_exists( 'rcp_log' ) ) {
				    rcp_log( sprintf( 'Setting ConvertKit opt-in flag for user #%d.', $user_id ) );
			    }

			    update_user_meta( $user_id, 'rcp_pending_convertkit_signup', true );
		    } else {
			    delete_user_meta( $user_id, 'rcp_pending_convertkit_signup' );

			    if ( function_exists( 'rcp_log' ) ) {
				    rcp_log( sprintf( 'User #%d did not opt-in to ConvertKit list.', $user_id ) );
			    }
		    }

	    }
	}

	/**
	 * When a membership is activated, add the user to the ConvertKit list if the flag has been set.
	 *
	 * @param string $old_status    Previous status before this change.
	 * @param int    $membership_id ID of the membership.
	 *
	 * @access public
	 * @since 1.1.2
	 * @return void
	 */
	public function maybe_subscribe( $old_status, $membership_id ) {

		if ( ! rcp_get_membership_meta( $membership_id, 'convertkit_pending_signup', true ) ) {
			return;
		}

		$membership = rcp_get_membership( $membership_id );

		if ( empty( $membership ) ) {
			return;
		}

		$user = get_userdata( $membership->get_customer()->get_user_id() );

		if ( empty( $user ) ) {
			return;
		}

		$name = trim( $user->first_name );

		if ( empty( $name ) ) {
			$name = $user->user_login;
		}

		$subscribed = $this->subscribe( $user->user_email, $name, $membership->get_object_id() );

		if ( $subscribed !== false ) {
			update_user_meta( $user->ID, 'rcp_subscribed_to_convertkit', $subscribed );
			delete_user_meta( $user->ID, 'rcp_pending_convertkit_signup' );
			rcp_delete_membership_meta( $membership_id, 'convertkit_pending_signup' );
		}

	}

	/**
	 * Maybe add member to the ConvertKit list when their account is activated
	 *
	 * @deprecated 1.1.2 In favour of `maybe_subscribe()`
	 * @see RCP_ConvertKit_API::maybe_subscribe()
	 *
	 * @param string     $status     New status being set.
	 * @param int        $user_id    ID of the user.
	 * @param string     $old_status Previous status.
	 * @param RCP_Member $member     Member object.
	 *
	 * @access public
	 * @since  1.1
	 * @return void
	 */
	public function maybe_add_to_list( $status, $user_id, $old_status, $member ) {

		// These are not the statuses we're looking for!
		if ( ! in_array( $status, array( 'active', 'free' ) ) ) {
			return;
		}

		// Not flagged for signup - bail
		if ( ! get_user_meta( $user_id, 'rcp_pending_convertkit_signup', true ) ) {
			return;
		}

		// API key not set - bail
		if ( ! $this->api_key ) {
			return;
		}

		$name = trim( $member->first_name );

		if ( empty( $name ) ) {
			$name = $member->user_login;
		}

		$subscribed = $this->subscribe( $member->user_email, $name, $member->get_subscription_id() );

		if ( $subscribed !== false ) {
			update_user_meta( $user_id, 'rcp_subscribed_to_convertkit', $subscribed );
			delete_user_meta( $user_id, 'rcp_pending_convertkit_signup' );
		}

	}

	/**
	 * Add/remove expiration tag
	 *
	 * When a membership expires, the expiration tag is applied.
	 * When an expired membership is reactivated, the expiration tag is removed.
	 *
	 * @param string $old_status    Previous status, before this update.
	 * @param string $new_status    New membership status.
	 * @param int    $membership_id ID of the membership.
	 *
	 * @since 1.1.2
	 * @return void
	 */
	public function maybe_add_remove_expiration_tag( $old_status, $new_status, $membership_id ) {

		if ( empty( $this->api_key ) ) {
			return;
		}

		$expiration_tag = ! empty( $this->settings['expired_tag'] ) ? $this->settings['expired_tag'] : '';

		// Bail if no tag is set.
		if ( empty( $expiration_tag ) ) {
			return;
		}

		$membership = rcp_get_membership( $membership_id );

		if ( empty( $membership ) ) {
			return;
		}

		// If this membership is disabled, we're not going to track it.
		if ( $membership->is_disabled() ) {
			return;
		}

		$user_id = $membership->get_customer()->get_user_id();

		if ( empty( $user_id ) ) {
			return;
		}

		$user          = get_userdata( $user_id );
		$subscriber_id = get_user_meta( $user_id, 'rcp_subscribed_to_convertkit', true );

		if ( ! is_numeric( $subscriber_id ) || '1' == $subscriber_id ) {
			$subscriber_id = $this->get_convertkit_id_from_user_id( $user_id );
		}

		// Bail if not in ConvertKit.
		if ( empty( $subscriber_id ) ) {
			return;
		}

		if ( 'expired' != $new_status ) {

			/**
			 * REMOVE TAG
			 *
			 * When a user moves off the "expired" status, remove the tag.
			 */

			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Removing expiration tag (ID %s) from user #%d.', $expiration_tag, $user_id ) );
			}

			$response = wp_remote_request(
				$this->api_url . 'subscribers/' . urlencode( $subscriber_id ) . '/tags/' . urlencode( $expiration_tag ) .'?api_secret=' . $this->api_secret,
				array(
					'timeout' => 30,
					'method'  => 'DELETE'
				)
			);

			if ( is_wp_error( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Error removing expiration tag (ID: %s) from user #%d. Error message: %s.', $expiration_tag, $user_id, $response->get_error_message() ) );
			} elseif( 200 != wp_remote_retrieve_response_code( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Invalid response code while removing expiration tag (ID: %s) from user #%d. Code: %d.', $expiration_tag, $user_id, wp_remote_retrieve_response_code( $response ) ) );
			}

		} elseif ( 'expired' == $new_status ) {

			/**
			 * ADD TAG
			 *
			 * When a user expires, apply the expiration tag.
			 */

			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Applying expiration tag (ID %s) to user #%d.', $expiration_tag, $user_id ) );
			}

			// Bail if user still has at least one expired membership.
			if ( $membership->get_customer()->has_active_membership() ) {
				if ( function_exists( 'rcp_log' ) ) {
					rcp_log( sprintf( 'ConvertKit: Not applying expired tag, as user #%d still has at lesat one active membership.', $user_id ) );
				}

				return;
			}

			$response = wp_remote_post(
				$this->api_url . 'tags/' . urlencode( $expiration_tag ) . '/subscribe?api_key=' . $this->api_key,
				array(
					'body'    => array(
						'email'      => sanitize_text_field( $user->user_email ),
						'first_name' => sanitize_text_field( $user->first_name )
					),
					'timeout' => 30
				)
			);

			if ( is_wp_error( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Error applying expiration tag (ID: %s) to user #%d. Error message: %s.', $expiration_tag, $user_id, $response->get_error_message() ) );
			} elseif( 200 != wp_remote_retrieve_response_code( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Invalid response code while applying expiration tag (ID: %s) to user #%d. Code: %d.', $expiration_tag, $user_id, wp_remote_retrieve_response_code( $response ) ) );
			} elseif ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Successfully applied expiration tag (ID: %s) to user #%d.', $expiration_tag, $user_id ) );
			}

		}

	}


	/**
	 * Add/remove expiration tag
	 *
	 * When a user expires, the expiration tag is applied.
	 * When an expired user is reactivated, the expiration tag is removed.
	 *
	 * @deprecated 1.1.2 In favour of maybe_add_remove_expiration_tag()
	 * @see RCP_ConvertKit_API::maybe_add_remove_expiration_tag()
	 *
	 * @param string     $new_status User's new status.
	 * @param int        $user_id    ID of the user.
	 * @param string     $old_status Previous status before the update.
	 * @param RCP_Member $member     Member object.
	 *
	 * @access public
	 * @since  1.1
	 * @return void
	 */
	public function add_remove_expiration_tag( $new_status, $user_id, $old_status, $member ) {

		if ( empty( $this->api_key ) ) {
			return;
		}

		$expiration_tag = ! empty( $this->settings['expired_tag'] ) ? $this->settings['expired_tag'] : '';

		// Bail if no tag is set.
		if ( empty( $expiration_tag ) ) {
			return;
		}

		$subscriber_id = get_user_meta( $user_id, 'rcp_subscribed_to_convertkit', true );

		if ( ! is_numeric( $subscriber_id ) || '1' == $subscriber_id ) {
			$subscriber_id = $this->get_convertkit_id_from_user_id( $user_id );
		}

		// Bail if not in ConvertKit.
		if ( empty( $subscriber_id ) ) {
			return;
		}

		if ( 'expired' != $new_status && 'expired' == $old_status ) {

			/**
			 * REMOVE TAG
			 *
			 * When a user moves off the "expired" status, remove the tag.
			 */

			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Removing expiration tag (ID %s) from user #%d.', $expiration_tag, $user_id ) );
			}

			$response = wp_remote_request(
				$this->api_url . 'subscribers/' . urlencode( $subscriber_id ) . '/tags/' . urlencode( $expiration_tag ) .'?api_secret=' . $this->api_secret,
				array(
					'timeout' => 30,
					'method'  => 'DELETE'
				)
			);

			if ( is_wp_error( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Error removing expiration tag (ID: %s) from user #%d. Error message: %s.', $expiration_tag, $user_id, $response->get_error_message() ) );
			} elseif( 200 != wp_remote_retrieve_response_code( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Invalid response code while removing expiration tag (ID: %s) from user #%d. Code: %d.', $expiration_tag, $user_id, wp_remote_retrieve_response_code( $response ) ) );
			}

		} elseif ( 'expired' == $new_status ) {

			/**
			 * ADD TAG
			 *
			 * When a user expires, apply the expiration tag.
			 */

			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Applying expiration tag (ID %s) to user #%d.', $expiration_tag, $user_id ) );
			}

			$response = wp_remote_post(
				$this->api_url . 'tags/' . urlencode( $expiration_tag ) . '/subscribe?api_key=' . $this->api_key,
				array(
					'body'    => array(
						'email'      => sanitize_text_field( $member->user_email ),
						'first_name' => sanitize_text_field( $member->first_name )
					),
					'timeout' => 30
				)
			);

			if ( is_wp_error( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Error applying expiration tag (ID: %s) to user #%d. Error message: %s.', $expiration_tag, $user_id, $response->get_error_message() ) );
			} elseif( 200 != wp_remote_retrieve_response_code( $response ) && function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'ConvertKit: Invalid response code while applying expiration tag (ID: %s) to user #%d. Code: %d.', $expiration_tag, $user_id, wp_remote_retrieve_response_code( $response ) ) );
			}

		}

	}


	/**
	 * Update subscription when user changes their email and/or first name
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       int $user_id The ID of the user
	 * @param       WP_User $old_user_data The old data for the user
	 * @return      void
	 */
	public function update_subscription( $user_id, $old_user_data ) {

		if ( empty( $this->api_key ) ) {
			return;
		}

		$user_data = get_userdata( $user_id );

		$new_email = $user_data->user_email;
		$old_email = $old_user_data->user_email;

		$subscriber = get_user_meta( $user_id, 'rcp_subscribed_to_convertkit', true );

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Maybe updating ConvertKit details for user #%d.', $user_id ) );
		}

		// Bail if not in ConvertKit.
		if ( empty( $subscriber ) ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'User #%d not signed up to ConvertKit - exiting.', $user_id ) );
			}

			return;
		}

		// Bail if old and new email match.
		if ( $new_email == $old_email ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'Email for user #%d was not changed - exiting.', $user_id ) );
			}

			return;
		}

		if ( ! is_numeric( $subscriber ) || '1' == $subscriber ) {

			$subscriber = $this->get_convertkit_id_from_user_id( $user_id, $old_email );

			if ( empty( $subscriber ) ) {
				if ( function_exists( 'rcp_log' ) ) {
					rcp_log( sprintf( 'Unable to find a ConvertKit subscriber ID for user #%d - exiting.', $user_id ) );
				}

				return;
			}

		}

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Updating ConvertKit data for user #%d.', $user_id ) );
		}

		// Now update the subscriber's profile with the new data.
		$args = apply_filters( 'rcp_convertkit_subscribe_vars', array(
			'email_address' => $new_email,
			'first_name'    => $user_data->first_name
		) );

		$request = wp_remote_request(
			$this->api_url . 'subscribers/' . urlencode( $subscriber ) . '?api_secret=' . $this->api_secret,
			array(
				'method'  => 'PUT',
				'body'    => $args,
				'timeout' => 30
			)
		);

		if ( is_wp_error( $request ) || 200 != wp_remote_retrieve_response_code( $request ) ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'Error updating user #%d in ConvertKit: invalid response when updating subscriber.', $user_id ) );

				return;
			}
		}

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Successfully updated ConvertKit subscriber #%d after member #%d profile edit.', $subscriber, $user_id ) );
		}

	}

	/**
	 * Add payment details to ConvertKit when a new payment is completed.
	 *
	 * @param int $payment_id
	 *
	 * @access public
	 * @since 1.1
	 * @return void
	 */
	public function insert_payment( $payment_id ) {

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Attempting to add payment #%d to ConvertKit.', $payment_id ) );
		}

		$payments         = new RCP_Payments();
		$payment          = $payments->get_payment( $payment_id );
		$user             = get_userdata( $payment->user_id );
		$membership_level = rcp_get_subscription_details( $payment->object_id );

		if ( empty( $payment->transaction_id ) ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'Not adding payment #%d to ConvertKit - transaction ID is blank.', $payment_id ) );
			}

			return;
		}

		$args = array(
			'api_secret'      => $this->api_secret,
			'integration_key' => '6rU1haZnJBocLR57XiinAw',
			'purchase'        => array(
				'integration'      => 'Restrict Content Pro',
				'transaction_id'   => $payment->transaction_id,
				'email_address'    => $user->user_email,
				'currency'         => rcp_get_currency(),
				'transaction_time' => $payment->date,
				'subtotal'         => $payment->subtotal,
				'discount'         => $payment->discount_amount,
				'total'            => $payment->amount,
				'status'           => 'paid',
				'products'         => array(
					array(
						'name'       => $membership_level->name,
						'pid'        => 'rcp-' . $membership_level->id,
						'lid'        => 1,
						'unit_price' => $membership_level->price + $membership_level->fee,
						'quantity'   => 1
					)
				)
			)
		);

		$request = wp_remote_post(
			$this->api_url . 'purchases',
			array(
				'body'    => json_encode( $args ),
				'timeout' => 30,
				'headers' => array(
					'Content-Type' => 'application/json'
				)
			)
		);

		if ( is_wp_error( $request ) || 201 != wp_remote_retrieve_response_code( $request ) ) {
			if ( function_exists( 'rcp_log' ) ) {
				$error = is_wp_error( $request ) ? $request->get_error_message() : 'unknown';
				rcp_log( sprintf( 'Error adding payment #%d in ConvertKit: invalid response. Response code: %s; message: %s.', $payment_id, wp_remote_retrieve_response_code( $request ), $error ) );

				return;
			}
		}

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Successfully added payment #%d to ConvertKit.', $payment_id ) );
		}

	}

	/**
	 * Searches ConvertKit for a subscriber with a given email address and returns their subscriber ID number.
	 *
	 * @param int    $user_id ID of the user.
	 * @param string $email   Email address of the user, if not the one currently saved to their account.
	 *
	 * @access public
	 * @since  1.1
	 * @return int|false
	 */
	public function get_convertkit_id_from_user_id( $user_id, $email = '' ) {

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'Getting ConvertKit ID number for user #%d.', $user_id ) );
		}

		if ( empty( $email ) ) {
			$user_data = get_userdata( $user_id );
			$email     = $user_data->user_email;
		}

		// First search ConvertKit subscribers to get this subscriber's ID. Once we've gotten it we'll save it so we can skip this step next time.
		$request = wp_remote_get( add_query_arg( array(
			'api_secret'    => $this->api_secret,
			'email_address' => urlencode( $email )
		), $this->api_url . 'subscribers' ) );

		if ( is_wp_error( $request ) || 200 != wp_remote_retrieve_response_code( $request ) ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'Invalid ConvertKit response when searching for subscriber for user #%d.', $user_id ) );
			}

			return false;
		}

		$body = json_decode( wp_remote_retrieve_body( $request ), true );

		// Unexpected result or no subscribers found.
		if ( empty( $body['subscribers'] ) || ! is_array( $body['subscribers'] ) ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'No subscriber found in ConvertKit with this email address (user ID #%d).', $user_id ) );
			}

			return false;
		}

		$subscriber_id = $body['subscribers'][0]['id'];

		// Save the subscriber ID so we can use it next time.
		update_user_meta( $user_id, 'rcp_subscribed_to_convertkit', sanitize_text_field( $subscriber_id ) );

		return $subscriber_id;

	}
}
