<?php

namespace RCP_Custom_Renew;

class Init {

	/**
	 * @var
	 */
	protected static $_instance;

	/**
	 * Only make one instance of \RCP_Custom_Renew\Init
	 *
	 * @return Init
	 */
	public static function get_instance() {
		if ( ! self::$_instance instanceof Init ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Add Hooks and Actions
	 */
	protected function __construct() {
		add_action( 'plugins_loaded', array( $this, 'maybe_setup' ), - 9999 );
	}

	/**
	 * Setup the plugin
	 */
	public function maybe_setup() {
		if ( ! $this->check_required_plugins() ) {
			return;
		}

		$this->includes();
		$this->actions();
	}

	/**
	 * Includes
	 */
	protected function includes() {
		Admin\Init::get_instance();
	}

	/**
	 * Actions and Filters
	 */
	protected function actions() {
		add_action( 'wp_enqueue_scripts', array( $this, 'scripts' ) );
		add_action( 'wp_footer', array( $this, 'print_scripts' ), 11 );
		add_filter( 'rcp_registration_is_recurring', array( $this, 'custom_auto_renew' ) );
	}

	/** Actions **************************************/

	/**
	 * Required Plugins notice
	 */
	public function required_plugins() {
		printf( '<div class="error"><p>%s</p></div>', __( 'Restrict Content Pro is required for the Restrict Content Pro - Custom Renew add-on to function.', 'rcp-custom-renew' ) );
	}

	/**
	 * Get custom level auto renew
	 *
	 * @return bool
	 */
	public function custom_auto_renew( $auto_renew ) {

		$subscription = rcp_get_registration()->get_membership_level_id();

		// get the custom auto renew meta
		if ( ! $custom_renew = self::meta_get( $subscription ) ) {
			return $auto_renew;
		}

		// this level does not support auto renew
		if ( 2 == $custom_renew ) {
			return false;
		}

		// if we pass the first two conditionals, then this level is set to auto renew
		$auto_renew = true;

		// make sure this gateway supports recurring payments
		if ( $auto_renew && ! empty( $_POST['rcp_gateway'] ) ) {
			$auto_renew = rcp_gateway_supports( sanitize_text_field( $_POST['rcp_gateway'] ), 'recurring' );
		}

		if ( $auto_renew && ! empty( $_POST['rcp_level'] ) ) {
			$details = rcp_get_subscription_details( $_POST['rcp_level'] );

			// check if this is an unlimited or free subscription
			if ( empty( $details->duration ) || empty( $details->price ) ) {
				$auto_renew = false;
			}
		}

		if ( ! rcp_get_registration_recurring_total() > 0 ) {
			$auto_renew = false;
		}

		return $auto_renew;
	}

	/**
	 * Register scripts
	 */
	public function scripts() {
		$suffix = ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ? '' : '.min';
		wp_register_script( 'rcp-custom-renew', RCP_CUSTOM_RENEW_PLUGIN_URL . "assets/js/public{$suffix}.js", array( 'jquery', 'rcp-register' ), RCP_CUSTOM_RENEW_PLUGIN_VERSION, true );
	}

	/**
	 * Print scripts
	 *
	 * Print scripts the way RCP does to make sure that we print our scripts after RCP
	 */
	public function print_scripts() {
		global $rcp_load_scripts;

		// this variable is set to TRUE if the short code is used on a page/post
		if ( ! $rcp_load_scripts ) {
			return;
		} // this means that neither short code is present, so we get out of here


		$levels = rcp_get_subscription_levels( 'active' );
		$custom_renew = array();
		foreach( $levels as $level ) {
			$renew = self::meta_get( $level->id );

			// disable the auto renew field if we have a custom override meta
			if ( '1' == $renew || '2' == $renew ) {
				$custom_renew[] = $level->id;
			}
		}

		wp_localize_script( 'rcp-custom-renew', 'rcpcr', $custom_renew );
		wp_print_scripts( 'rcp-custom-renew' );
	}

	/** Helper Methods **************************************/

	/**
	 * Make sure RCP is active
	 * @return bool
	 */
	protected function check_required_plugins() {

		if ( ! function_exists( 'is_plugin_active' ) ) {
			require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		}

		if ( is_plugin_active( 'restrict-content-pro/restrict-content-pro.php' ) ) {
			return true;
		}

		add_action( 'admin_notices', array( $this, 'required_plugins' ) );

		return false;
	}

	/**
	 * Gets the plugin support URL
	 *
	 * @since 1.0.0
	 * @return string
	 */
	public function get_support_url() {

		return 'https://skillfulplugins.com/support';
	}

	/**
	 * Returns the plugin name, localized
	 *
	 * @since 1.0.0
	 * @return string the plugin name
	 */
	public function get_plugin_name() {
		return __( 'Restrict Content Pro - Custom Renew', 'rcp-custom-renew' );
	}

	public function get_id() {
		return 'rcp-custom-renew';
	}

	/** Subscription Meta Helpers ********************/

	/**
	 * Save subscription meta
	 *
	 * @param $subscription_id
	 * @param $values
	 */
	public static function meta_save( $subscription_id, $values ) {
		global $rcp_levels_db;

		$rcp_levels_db->update_meta( $subscription_id, 'custom_renewal', $values );
	}

	/**
	 * Get subscription meta
	 *
	 * @param $subscription_id
	 *
	 * @return mixed
	 */
	public static function meta_get( $subscription_id ) {
		global $rcp_levels_db;

		$meta = $rcp_levels_db->get_meta( $subscription_id, 'custom_renewal', true );

		return apply_filters( 'rcp_custom_renew_get', $meta, $subscription_id );
	}

}