<?php
namespace RCP\Addon\DripContent\Shortcodes;

defined( 'WPINC' ) or die;

/**
 * Renders the current member's drip content schedule.
 *
 * @param array $atts {
 * 	Shortcode attributes. Optional.
 *
 * 	@type bool   $show_future Whether or not to show future content not yet available.
 * 	@type string $orderby     The sort order of the schedule. Default 'id'. Options: 'id', 'name', 'date'.
 * }
 * @since 1.0.1 Added orderby attribute.
 * @since 1.0
 * @return string
 * @throws \Exception
 */
function member_content_shortcode( $atts ) {

	$atts = shortcode_atts( array(
		'show_future' => true,
		'orderby'     => 'id',
	), $atts, 'rcp_drip_content' );

	if ( 'false' === $atts['show_future'] ) {
		$atts['show_future'] = false;
	}

	$output = '';

	if ( ! is_user_logged_in() ) {
		return $output;
	}

	$user_id  = wp_get_current_user()->ID;
	$customer = false;

	if ( function_exists( 'rcp_get_customer_membership_level_ids' ) ) {
		$membership_level_ids = array();
		$customer             = rcp_get_customer_by_user_id( $user_id );

		if ( ! empty( $customer ) ) {
			$membership_level_ids = $customer->get_memberships( array(
				'status__in' => array( 'active', 'cancelled' ),
				'fields'     => 'object_id'
			) );
		}
	} else {
		$sub_id               = (int) rcp_get_subscription_id( $user_id );
		$membership_level_ids = ! empty( $sub_id ) ? array( $sub_id ) : array();
	}

	if ( empty( $membership_level_ids ) ) {
		return $output;
	}

	foreach ( $membership_level_ids as $sub_id ) {
		$schedules = rcp_drip_get_level_schedules( $sub_id );

		if ( ! $schedules || empty( $schedules[0] ) ) {
			continue;
		}

		// Sort the schedules
		switch ( $atts['orderby'] ) {

			case 'name':

				foreach ( $schedules as $key => $schedule ) {
					if ( 'post' === $schedule['type'] ) {
						$schedules[ $key ]['name'] = get_post_field( 'post_title', $schedule['post_id'], 'raw' );
					}

					if ( 'taxonomy' === $schedule['type'] ) {
						$schedules[ $key ]['name'] = get_term_by( 'term_taxonomy_id', $schedule['taxonomy_term'] )->name;
					}
				}

				usort( $schedules, function ( $a, $b ) {
					return strcasecmp( $a['name'], $b['name'] );
				} );

				break;

			case 'date':
				foreach ( $schedules as $key => $schedule ) {
					$schedules[ $key ]['time'] = strtotime( $schedule['unit'] . ' ' . $schedule['period'] );
				}

				usort( $schedules, function ( $a, $b ) {
					return $a['time'] - $b['time'];
				} );

				break;

			case 'id':
			default:
				break;

		}

		$timezone   = get_option( 'timezone_string' );
		$timezone   = ! empty( $timezone ) ? $timezone : 'UTC';
		$now        = new \DateTime( null, new \DateTimeZone( $timezone ) );
		$start_date = false;

		if ( ! empty( $customer ) ) {
			$memberships = $customer->get_memberships( array(
				'status__in' => array( 'active', 'cancelled' ),
				'object_id'  => $sub_id,
				'orderby'    => 'created_date',
				'order'      => 'ASC'
			) );

			if ( ! empty( $memberships ) && ! empty( $memberships[0] ) ) {
				/**
				 * @var \RCP_Membership $membership
				 */
				$membership = $memberships[0];
				$start_date = $membership->get_created_date( false );
			}

			if ( empty( $start_date ) ) {
				$start_date = $customer->get_date_registered( false );
			}
		}

		if ( empty( $start_date ) ) {
			$rcp_member = new \RCP_Member( $user_id );
			$start_date = $rcp_member->get_joined_date( $sub_id );
		}

		// If we still don't have a start date, bail.
		if ( empty( $start_date ) ) {
			continue;
		}

		$start_date = new \DateTime( $start_date, new \DateTimeZone( $timezone ) );
		$start_date->modify( 'midnight' );

		$output .= '<p class="rcp-drip-available-content-level-name">' . esc_html( rcp_get_subscription_name( $sub_id ) ) . '</p>';

		$output .= '<ul class="rcp-drip-available-content">';

		foreach ( $schedules as $key => $schedule ) {

			if ( 'post' === $schedule['type'] ) {

				$post_available_date = clone $start_date;

				$post_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

				$post_available_date->modify( 'midnight' );

				$post = get_post( $schedule['post_id'] );

				if ( empty( $post ) ) {
					continue;
				}

				if ( $now >= $post_available_date ) {

					$output .= '<li><a href="' . esc_url( get_permalink( $post ) ) . '">' . get_the_title( $post ) . '</a></li>';

				} else if ( $atts['show_future'] ) {

					$output .= '<li>' . $post->post_title . ' ' . sprintf( __( '(Available on %s)', 'rcp-drip-content' ), $post_available_date->format( get_option( 'date_format' ) ) ) . '</li>';
				}
			}

			if ( 'taxonomy' === $schedule['type'] ) {

				$taxonomy_available_date = clone $start_date;

				$taxonomy_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

				$term = get_term( $schedule['taxonomy_term'] );

				if ( empty( $term ) || is_wp_error( $term ) ) {
					continue;
				}

				if ( $now >= $taxonomy_available_date ) {

					$output .= '<li><a href="' . esc_url( get_term_link( $term ) ) . '">' . ucfirst( $term->taxonomy ) . ': ' . $term->name . '</a></li>';

				} else if ( $atts['show_future'] ) {

					$output .= '<li>' . ucfirst( $term->taxonomy ) . ': ' . $term->name . ' ' . sprintf( __( '(Available on %s)', 'rcp-drip-content' ), $taxonomy_available_date->format( get_option( 'date_format' ) ) ) . '</li>';

				}
			}

		}

		$output .= '</ul>';
	}

	/**
	 * Filters the shortcode content.
	 *
	 * @param string $output
	 *
	 * @since 1.0.6
	 */
	return apply_filters( 'rcp_drip_content_shortcode', $output );
}
add_shortcode( 'rcp_drip_member_content', '\RCP\Addon\DripContent\Shortcodes\member_content_shortcode' );