<?php
namespace RCP\Addon\DripContent\Visibility;

use RCP_Membership;
use Exception;
use DateTime;
use DateTimeZone;

defined( 'WPINC' ) or die;

/**
 * Overrides a membership's access to a post if it doesn't meet the drip schedule requirements.
 *
 * @param bool           $can_access    Whether or not this membership grants access to the post.
 * @param int            $membership_id ID of the membership.
 * @param int            $post_id       ID of the post being checked.
 * @param RCP_Membership $membership    Membership object.
 *
 * @since 1.0.5
 * @return bool
 * @throws Exception
 *
 */
function membership_can_access( $can_access, $membership_id, $post_id, $membership ) {

	$schedules = rcp_drip_get_level_schedules( $membership->get_object_id() );

	if ( ! $schedules || empty( $schedules[0] ) ) {
		return $can_access;
	}

	$user_id    = $membership->get_customer()->get_user_id();
	$timezone   = get_option( 'timezone_string' );
	$timezone   = ! empty( $timezone ) ? $timezone : 'UTC';
	$now        = new DateTime( "now", new DateTimeZone( $timezone ) );
	$start_base = method_exists( $membership, 'get_activated_date' ) ? $membership->get_activated_date() : $membership->get_created_date( false );
	$start_date = new DateTime( $start_base, new DateTimeZone( $timezone ) );
	$start_date->modify( 'midnight' );

	foreach ( $schedules as $key => $schedule ) {

		if ( 'taxonomy' === $schedule['type'] && is_object_in_term( $post_id, $schedule['taxonomy'], $schedule['taxonomy_term'] ) ) {

			// Return early if the user already can't access the content via taxonomy restrictions
			if ( true === rcp_is_post_taxonomy_restricted( $post_id, $schedule['taxonomy'], $user_id ) ) {
				return false;
			}

			$taxonomy_available_date = clone $start_date;

			$taxonomy_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

			if ( $now >= $taxonomy_available_date ) {
				return true; // return early to avoid additional queries
			}

			$can_access = false;
		}

		if ( 'post' === $schedule['type'] && (int) $schedule['post_id'] === $post_id ) {

			// If the post isn't restricted, no point in checking it.
			if ( ! rcp_is_restricted_content( $post_id ) ) {
				return $can_access;
			}

			$post_available_date = clone $start_date;

			$post_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

			if ( $now >= $post_available_date ) {
				return true; // return early to avoid additional queries
			}

			$can_access = false;
		}
	}

	return $can_access;

}

/**
 * Determines if the member can access the requested content
 * based on the drip schedule, if defined.
 *
 * @since      1.0
 * @return bool True if the content is visible to the member, false if not.
 * @deprecated 1.0.5 In favour of `membership_can_access()`
 *
 */
function can_access( $ret, $user_id, $post_id, $user ) {

	if ( function_exists( 'rcp_get_membership' ) ) {
		return $ret;
	}

	if ( ! $ret ) {
		return false;
	}

	if ( ! is_user_logged_in() ) {
		return $ret;
	}

	// Return early for editors to avoid additional queries.
	if ( user_can( $user, 'edit_others_posts' ) ) {
		return true;
	}

	$sub_id = (int) rcp_get_subscription_id( $user_id );

	if ( ! $sub_id ) {
		return $ret;
	}

	$schedules = rcp_drip_get_level_schedules( $sub_id );

	if ( ! $schedules || empty( $schedules[0] ) ) {
		return $ret;
	}

	$rcp_member = new \RCP_Member( wp_get_current_user()->ID );

	$timezone = get_option( 'timezone_string' );

	$timezone = ! empty( $timezone ) ? $timezone : 'UTC';

	$now = new \DateTime( "now", new \DateTimeZone( $timezone ) );

	$join_date = new \DateTime( $rcp_member->get_joined_date( $sub_id ), new \DateTimeZone( $timezone ) );

	$join_date->modify( 'midnight' );

	foreach( $schedules as $key => $schedule ) {

		if ( 'taxonomy' === $schedule['type'] && is_object_in_term( $post_id, $schedule['taxonomy'], $schedule['taxonomy_term'] ) ) {

			// Return early if the user already can't access the content via taxonomy restrictions
			if ( true === rcp_is_post_taxonomy_restricted( $post_id, $schedule['taxonomy'], $user_id ) ) {
				return false;
			}

			$taxonomy_available_date = clone $join_date;

			$taxonomy_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

			if ( $now >= $taxonomy_available_date ) {
				return true; // return early to avoid additional queries
			}

			$ret = false;
		}

		if ( 'post' === $schedule['type'] && (int) $schedule['post_id'] === $post_id ) {

			// If the post isn't restricted, no point in checking it.
			if ( ! rcp_is_restricted_content( $post_id ) ) {
				return $ret;
			}

			$post_available_date = clone $join_date;

			$post_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

			if ( $now >= $post_available_date ) {
				return true; // return early to avoid additional queries
			}

			$ret = false;
		}
	}

	return $ret;
}