<?php
/**
 * Membership Level Settings
 *
 * @package   rcp-group-accounts
 * @copyright Copyright (c) 2019, Restrict Content Pro team
 * @license   GPL2+
 * @since     2.0
 */

namespace RCPGA\Admin\Settings;

/**
 * Add a new membership level table column for "Seats".
 *
 * @param array $columns
 *
 * @since 2.0
 * @return array
 */
function seat_count_column( $columns ) {

	// We specifically want to add our column BEFORE "order".
	if ( array_key_exists( 'order', $columns ) ) {
		$offset  = array_search( 'order', array_keys( $columns ) );
		$columns = array_merge(
			array_slice( $columns, 0, $offset ),
			array( 'group_seats' => __( 'Seats', 'rcp-group-accounts' ) ),
			array_slice( $columns, $offset, null )
		);
	} else {
		$columns['group_seats'] = __( 'Seats', 'rcp-group-accounts' );
	}

	return $columns;

}

add_filter( 'rcp_membership_levels_list_table_columns', __NAMESPACE__ . '\seat_count_column' );

/**
 * Display the number of Group Accounts seats for the membership level.
 *
 * @param mixed  $value            Column value.
 * @param object $membership_level Membership level being displayed.
 *
 * @since 2.0
 * @return int
 */
function seat_count_value( $value, $membership_level ) {
	return rcpga_get_level_group_seats_allowed( $membership_level->id );
}

add_filter( 'rcp_membership_levels_list_table_column_group_seats', __NAMESPACE__ . '\seat_count_value', 10, 2 );

/**
 * Display group accounts fields on the Add/Edit Membership Level forms.
 *
 * @param object|null $level Membership level object from the database, or null if this is a new level.
 *
 * @since 2.0
 * @return void
 */
function display_membership_level_fields( $level = null ) {

	/**
	 * Filters whether or not group accounts is enabled by default.
	 *
	 * @param bool        $enabled
	 * @param object|null $level
	 */
	$enabled    = apply_filters( 'rcpga_default_level_group_accounts_status', false, $level );
	$enabled    = ( empty( $level->id ) ) ? $enabled : rcpga_is_level_group_accounts_enabled( $level->id );
	$seat_count = ( empty( $level->id ) ) ? 0 : rcpga_get_level_group_seats_allowed( $level->id ); ?>

	<tr class="form-field">
		<th scope="row" valign="top">
			<label for="rcpga-group-seats-allow"><?php _e( 'Allow Group Account', 'rcp-group-accounts' ); ?></label>
		</th>
		<td>
			<input id="rcpga-group-seats-allow" type="checkbox" name="rcpga-group-seats-allow" <?php checked( $enabled ); ?> />
			<p class="description"><?php _e( 'Check to enable group accounts for this membership level.', 'rcp-group-accounts' ); ?></p>
		</td>
	</tr>

	<tr class="form-field" <?php echo ( $enabled ) ? '' : 'style="display:none;"' ?>>
		<th scope="row" valign="top">
			<label for="rcpga-group-seats"><?php _e( 'Group Seats', 'rcp-group-accounts' ); ?></label>
		</th>
		<td>
			<input id="rcpga-group-seats" type="number" name="rcpga-group-seats" value="<?php echo absint( $seat_count ); ?>" min="0" style="width: 100px;"/>
			<p class="description"><?php _e( 'The number of group seats available to this level including the group owner.', 'rcp-group-accounts' ); ?></p>
		</td>
	</tr>

	<?php

}

add_action( 'rcp_add_subscription_form', __NAMESPACE__ . '\display_membership_level_fields' );
add_action( 'rcp_edit_subscription_form', __NAMESPACE__ . '\display_membership_level_fields' );

/**
 * Save the group accounts settings for the membership level.
 *
 * @param int   $membership_level_id ID of the membership level being added / edited.
 * @param array $args                Array of data.
 *
 * @since 2.0
 * @return void
 */
function save_membership_level_fields( $membership_level_id, $args ) {

	// Bail if using activate/deactivate row action.
	if ( ! empty( $_GET['rcp-action'] ) && in_array( $_GET['rcp-action'], array( 'activate_subscription', 'deactivate_subscription' ) ) ) {
		return;
	}

	// Bail if using activate/deactivate bulk action.
	if ( ! empty($_REQUEST['_wpnonce'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'bulk-membershiplevels' ) ) {
		return;
	}

	if ( isset( $_POST['rcpga-group-seats-allow'] ) ) {
		rcpga_enable_level_group_accounts( $membership_level_id );
	} else {
		rcpga_disable_level_group_accounts( $membership_level_id );
	}

	if ( ! empty( $_POST['rcpga-group-seats'] ) ) {
		rcpga_set_level_group_seats_allowed( $membership_level_id, absint( $_POST['rcpga-group-seats'] ) );
	} else {
		rcpga_remove_level_seat_count( $membership_level_id );
	}

}

add_action( 'rcp_add_subscription', __NAMESPACE__ . '\save_membership_level_fields', 10, 2 );
add_action( 'rcp_edit_subscription_level', __NAMESPACE__ . '\save_membership_level_fields', 10, 2 );