<?php
/**
 * Membership Level Functions
 *
 * @package   rcp-group-accounts
 * @copyright Copyright (c) 2019, Restrict Content Pro team
 * @license   GPL2+
 * @since     2.0
 */

/**
 * Removes the group seats metadata for the specified membership level.
 *
 * @param int $level_id The membership level ID.
 *
 * @return void
 */
function rcpga_remove_level_seat_count( $level_id ) {

	if ( function_exists( 'rcp_delete_membership_level_meta' ) ) {
		rcp_delete_membership_level_meta( $level_id, 'group_seats_allowed' );
	} else {
		$levels = new RCP_Levels;
		$levels->delete_meta( $level_id, 'group_seats_allowed' );
	}

	do_action( 'rcpga_remove_level_seat_count', $level_id );

}

add_action( 'rcp_remove_level', 'rcpga_remove_level_seat_count' );

/**
 * Gets the number of seats allowed for the specified membership level.
 *
 * @param int $level_id The membership level ID.
 *
 * @return integer The number of seats allowed for the specified membership level.
 */
function rcpga_get_level_group_seats_allowed( $level_id ) {

	if ( function_exists( 'rcp_get_membership_level_meta' ) ) {
		$count = rcp_get_membership_level_meta( $level_id, 'group_seats_allowed', true );
	} else {
		$levels = new RCP_Levels;
		$count  = $levels->get_meta( $level_id, 'group_seats_allowed', true );
	}

	return apply_filters( 'rcpga_get_level_group_seats_allowed', absint( $count ), $level_id );

}

/**
 * Check if group accounts are disabled for this level.
 * Note, this does not check the number of seats for this level, which could be 0.
 *
 * @param int $level_id Membership level ID.
 *
 * @return bool True if the level is group enabled, false if not.
 */
function rcpga_is_level_group_accounts_enabled( $level_id ) {

	if ( function_exists( 'rcp_get_membership_level_meta' ) ) {
		$enabled = (bool) rcp_get_membership_level_meta( $level_id, 'group_seats_enabled', true );
	} else {
		$levels  = new RCP_Levels;
		$enabled = (bool) $levels->get_meta( $level_id, 'group_seats_enabled', true );
	}

	return apply_filters( 'rcpga_is_level_group_accounts_enabled', $enabled, $level_id );
}

/**
 * Sets the number of group seats allowed for the specified membership level.
 *
 * @param int $level_id      The membership level ID.
 * @param int $seats_allowed The number of seats allowed.
 *
 * @return void
 */
function rcpga_set_level_group_seats_allowed( $level_id, $seats_allowed ) {

	if ( function_exists( 'rcp_update_membership_level_meta' ) ) {
		rcp_update_membership_level_meta( $level_id, 'group_seats_allowed', absint( $seats_allowed ) );
	} else {
		$levels = new RCP_Levels;
		$levels->update_meta( $level_id, 'group_seats_allowed', absint( $seats_allowed ) );
	}

	do_action( 'rcpga_set_level_group_seats_allowed', $level_id, absint( $seats_allowed ) );

}

/**
 * Disable group accounts for this level
 *
 * @param int $level_id Membership level ID.
 *
 * @return void
 */
function rcpga_disable_level_group_accounts( $level_id ) {

	if ( function_exists( 'rcp_delete_membership_level_meta' ) ) {
		rcp_delete_membership_level_meta( $level_id, 'group_seats_enabled' );
	} else {
		$levels = new RCP_Levels;
		$levels->delete_meta( $level_id, 'group_seats_enabled' );
	}

	do_action( 'rcpga_disable_level_group_accounts', $level_id );

}

/**
 * Disable group accounts for this level
 *
 * @param int $level_id Membership level ID.
 *
 * @return void
 */
function rcpga_enable_level_group_accounts( $level_id ) {

	if ( function_exists( 'rcp_update_membership_level_meta' ) ) {
		rcp_update_membership_level_meta( $level_id, 'group_seats_enabled', true );
	} else {
		$levels = new RCP_Levels;
		$levels->update_meta( $level_id, 'group_seats_enabled', true );
	}

	do_action( 'rcpga_enable_level_group_accounts', $level_id );

}

/**
 * Return all levels that are enabled for group accounts
 *
 * @return array
 */
function rcpga_get_group_enabled_levels() {

	global $wpdb;

	$enabled_levels = $wpdb->get_results( $wpdb->prepare( "SELECT level_id FROM {$wpdb->levelmeta} WHERE meta_key = %s", 'group_seats_enabled' ), ARRAY_A );

	return apply_filters( 'rcpga_get_group_enabled_levels', wp_list_pluck( $enabled_levels, 'level_id' ) );
}