<?php
/**
 * Registration Actions
 *
 * @package   rcp-group-accounts
 * @copyright Copyright (c) 2019, Restrict Content Pro team
 * @license   GPL2+
 * @since     2.0
 */

namespace RCPGA;

use RCP_Customer;
use RCP_Membership;

/**
 * Add group creation fields to registration form
 *
 * @since 2.0
 * @return void
 */
function add_registration_fields() {
	rcp_get_template_part( 'group', 'register' );
	wp_enqueue_script( 'rcp-group-accounts' );
}

add_action( 'rcp_after_register_form_fields', __NAMESPACE__ . '\add_registration_fields', 100 );

/**
 * Customers cannot change levels if the level in question has fewer seats available than the total
 * number in their group already.
 *
 * @param array $post Posted form data.
 *
 * @since 2.0
 * @return void
 */
function maybe_prevent_level_change( $post ) {

	$show_error = false;

	if ( method_exists( rcp_get_registration(), 'get_registration_type' ) ) {

		/**
		 * RCP 3.1+
		 */

		// Bail if this isn't an upgrade/downgrade.
		if ( ! in_array( rcp_get_registration()->get_registration_type(), array( 'upgrade', 'downgrade' ) ) ) {
			return;
		}

		$membership = rcp_get_registration()->get_membership();

		if ( empty( $membership ) ) {
			return;
		}

		// Bail if the membership in question doesn't have a group.
		$group = rcpga_get_group_by( 'membership_id', $membership->get_id() );

		if ( empty( $group ) ) {
			return;
		}

		$member_count = $group->get_member_count();

		// If member count is 1, set it to 0. This allows us to exclude the group owner if they're the only one in the group.
		if ( 1 === $member_count ) {
			$member_count = 0;
		}

		if ( rcpga_get_level_group_seats_allowed( absint( $post['rcp_level'] ) ) < $member_count ) {
			$show_error = true;
		}
	} else {

		/**
		 * RCP 3.0
		 */

		if ( empty( $post['rcp_level'] ) ) {
			return;
		}

		// Bail if this user isn't a group owner.
		if ( ! rcpga_user_is_group_member( get_current_user_id(), 'owner' ) ) {
			return;
		}

		$group = rcpga_get_group_by( 'owner_id', get_current_user_id() );

		if ( empty( $group ) ) {
			return;
		}

		$member_count = $group->get_member_count();

		// If member count is 1, set it to 0. This allows us to exclude the group owner if they're the only one in the group.
		if ( 1 === $member_count ) {
			$member_count = 0;
		}

		if ( rcpga_get_level_group_seats_allowed( absint( $post['rcp_level'] ) ) < $member_count ) {
			$show_error = true;
		}

	}

	if ( $show_error ) {
		rcp_errors()->add( 'remove_children', __( 'You have too many members in your group to change to this level.', 'rcp-group-accounts' ), 'register' );
	}

}

add_action( 'rcp_form_errors', __NAMESPACE__ . '\maybe_prevent_level_change' );

/**
 * Require group name for Group Accounts registrations.
 *
 * @param array $post Posted form data.
 *
 * @since 2.0
 * @return void
 */
function require_group_name( $post ) {

	$show_error = false;

	if ( method_exists( rcp_get_registration(), 'get_registration_type' ) ) {

		/**
		 * RCP 3.1+
		 */

		// Make sure we have a membership level.
		if ( ! $membership_level_id = rcp_get_registration()->get_membership_level_id() ) {
			return;
		}

		// Bail if this level doesn't support group accounts.
		if ( ! rcpga_is_level_group_accounts_enabled( $membership_level_id ) ) {
			return;
		}

		// Bail if this user is already a group owner and multiple memberships is not enabled.
		if ( rcpga_user_is_group_member( get_current_user_id(), 'owner' ) && ! rcp_multiple_memberships_enabled() ) {
			return;
		}

		// Not required on renewals.
		if ( 'renewal' == rcp_get_registration()->get_registration_type() ) {
			return;
		}

		// Not required if upgrading a membership that already has a group.
		$membership = rcp_get_registration()->get_membership();
		$is_change  = in_array( rcp_get_registration()->get_registration_type(), array( 'upgrade', 'downgrade' ) );
		if ( ! empty( $membership ) && $is_change && rcpga_get_group_by( 'membership_id', $membership->get_id() ) ) {
			return;
		}

		$name       = isset( $post['rcpga-group-name'] ) ? trim( $post['rcpga-group-name'] ) : '';
		$show_error = empty( $name );

	} else {

		/**
		 * RCP 3.0
		 */
		if ( ! $membership_id = rcp_get_registration()->get_membership_level_id() ) {
			return;
		}

		// Bail if this user is already a group owner.
		if ( rcpga_user_is_group_member( get_current_user_id(), 'owner' ) ) {
			return;
		}

		// Bail if this level doesn't support group accounts.
		if ( ! rcpga_is_level_group_accounts_enabled( $membership_id ) ) {
			return;
		}

		$name       = isset( $post['rcpga-group-name'] ) ? trim( $post['rcpga-group-name'] ) : '';
		$show_error = empty( $name );

	}

	if ( $show_error ) {
		rcp_errors()->add( 'group_name_required', __( 'Please enter a group name.', 'rcp-group-accounts' ), 'register' );
	}

}

add_action( 'rcp_form_errors', __NAMESPACE__ . '\require_group_name' );

/**
 * Create pending group during registration.
 *
 * Group is saved in membership meta until payment is confirmed.
 *
 * @see   activate_membership_group()
 *
 * @param array                $post                Posted data.
 * @param int                  $user_id             ID of the user who's registering.
 * @param float                $price               Price of the subscription.
 * @param int                  $payment_id          ID of the pending payment associated with this registration.
 * @param RCP_Customer|false   $customer            Customer object.
 * @param int                  $membership_id       ID of the new pending membership.
 * @param RCP_Membership|false $previous_membership Previous membership object, or false if none.
 * @param string               $registration_type   Type of registration: 'new', 'renewal', or 'upgrade'.
 *
 * @since 2.0
 * @return void
 */
function create_pending_group( $post, $user_id, $price, $payment_id = 0, $customer = false, $membership_id = 0, $previous_membership = false, $registration_type = 'new' ) {

	if ( ! $membership_level_id = rcp_get_registration()->get_membership_level_id() ) {
		return;
	}

	// If multiple memberships is disabled, then the member must not already be a group owner.
	if ( ! rcp_multiple_memberships_enabled() && rcpga_user_is_group_member( $user_id, 'owner' ) ) {
		return;
	}

	// Make sure this level supports group accounts.
	if ( ! $seats = rcpga_get_level_group_seats_allowed( $membership_level_id ) ) {
		return;
	}

	// Finally, make sure we have a group name.
	if ( empty( $post['rcpga-group-name'] ) ) {
		return;
	}

	$args = array(
		'owner_id'      => absint( $user_id ),
		'membership_id' => absint( $membership_id ),
		'seats'         => absint( $seats ),
		'name'          => wp_unslash( sanitize_text_field( $post['rcpga-group-name'] ) ),
		'description'   => ! empty( $post['rcpga-group-description'] ) ? wp_unslash( wp_filter_post_kses( $post['rcpga-group-description'] ) ) : '',
	);

	rcp_add_membership_meta( $membership_id, 'rcpga_pending_group', $args );

}

add_action( 'rcp_form_processing', __NAMESPACE__ . '\create_pending_group', 10, 8 );

/**
 * Add group information to Stripe payment intent meta data
 *
 * @param array                       $intent_args Payment intent arguments.
 * @param \RCP_Payment_Gateway_Stripe $stripe_gateway
 *
 * @since 2.0.1
 * @return array
 */
function add_stripe_payment_intent_args( $intent_args, $stripe_gateway ) {

	$group = rcpga_get_group_by( 'membership_id', $stripe_gateway->membership->get_id() );

	if ( $group ) {
		$group_name = $group->get_name();
	} else {
		// Check membership meta.
		$pending_group_args = rcp_get_membership_meta( $stripe_gateway->membership->get_id(), 'rcpga_pending_group', true );
		$group_name         = ! empty( $pending_group_args['name'] ) ? $pending_group_args['name'] : '';
	}

	if ( ! empty( $group_name ) ) {
		$intent_args['metadata']['rcp_group_name'] = esc_html( $group_name );
	}

	return $intent_args;

}
add_filter( 'rcp_stripe_create_payment_intent_args', __NAMESPACE__ . '\add_stripe_payment_intent_args', 10, 2 );