<?php
/**
 * Test Group Member Invites
 *
 * @package   rcp-group-accounts
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCPGA\Tests\Group_Members;

use RCPGA\Tests\UnitTestCase;

/**
 * Class Test_Member_Invites
 *
 * @package RCPGA\Tests\Group_Members
 */
class Test_Member_Invites extends UnitTestCase {

	/**
	 * @var \RCPGA_Group_Member
	 */
	protected static $group_member;

	/**
	 * Set up once before tests run
	 */
	public static function setUpBeforeClass() {
		parent::setUpBeforeClass();

		$level_id = self::rcpga()->level->create();

		// Enable Group Accounts on the level.
		rcpga_enable_level_group_accounts( $level_id );
		rcpga_set_level_group_seats_allowed( $level_id, 3 );

		// Create our group owner's customer and membership record.
		$customer      = self::rcpga()->customer->create_and_get();
		$membership_id = $customer->add_membership( array(
			'object_id' => $level_id,
			'status'    => 'active'
		) );

		$group = self::rcpga()->group->create_and_get( array(
			'owner_id'      => $customer->get_user_id(),
			'membership_id' => $membership_id,
			'seats'         => 3
		) );

		// Add a group member.
		self::$group_member = self::rcpga()->group_member->create_and_get( array(
			'group_id' => $group->get_group_id(),
			'role'     => 'invited'
		) );
	}

	/**
	 * Changes the invite link validity length to 1 hour
	 *
	 * @param int $duration
	 *
	 * @return int
	 */
	public function invite_link_duration_1_hour( $duration ) {
		return HOUR_IN_SECONDS;
	}

	/**
	 * Changes the invite link validity length to 3 hours
	 *
	 * @param int $duration
	 *
	 * @return int
	 */
	public function invite_link_duration_3_hours( $duration ) {
		return 3 * HOUR_IN_SECONDS;
	}

	/**
	 * @covers ::rcpga_generate_member_activation_key
	 */
	public function test_generating_activation_key_sets_db_column() {
		$this->assertSame( '', self::$group_member->get_activation_key() );

		$raw_key      = rcpga_generate_member_activation_key( self::$group_member );
		$group_member = rcpga_get_group_member_by_id( self::$group_member->get_id() );

		$this->assertIsString( $raw_key );
		$this->assertNotEmpty( $group_member->get_activation_key() );
	}

	/**
	 * @covers ::rcpga_validate_activation_key
	 */
	public function test_validating_valid_key_returns_true() {
		$raw_key      = rcpga_generate_member_activation_key( self::$group_member );
		$group_member = rcpga_get_group_member_by_id( self::$group_member->get_id() );
		$is_valid     = rcpga_validate_activation_key( $raw_key, $group_member, time() );

		$this->assertTrue( $is_valid );
	}

	/**
	 * Ensures a valid key is still valid after 1 day.
	 *
	 * @covers ::rcpga_validate_activation_key
	 */
	public function test_validating_valid_key_returns_true_after_1_day() {
		$raw_key      = rcpga_generate_member_activation_key( self::$group_member );
		$group_member = rcpga_get_group_member_by_id( self::$group_member->get_id() );
		$is_valid     = rcpga_validate_activation_key( $raw_key, $group_member, strtotime( '+1 day' ) );

		$this->assertTrue( $is_valid );
	}

	/**
	 * @covers ::rcpga_validate_activation_key
	 */
	public function test_validating_invalid_key_returns_wp_error() {
		$raw_key = rcpga_generate_member_activation_key( self::$group_member );

		// Add some random characters to the valid key to invalidate it.
		$raw_key = $raw_key . rand( 100, 500 );

		$group_member = rcpga_get_group_member_by_id( self::$group_member->get_id() );
		$is_valid     = rcpga_validate_activation_key( $raw_key, $group_member, time() );

		$this->assertInstanceOf( \WP_Error::class, $is_valid );
		$this->assertSame( 'invalid_key', $is_valid->get_error_code() );
	}

	/**
	 * Ensures a valid key is expired after 10 days
	 *
	 * @covers ::rcpga_validate_activation_key
	 */
	public function test_validating_expired_key_returns_wp_error() {
		$raw_key      = rcpga_generate_member_activation_key( self::$group_member );
		$group_member = rcpga_get_group_member_by_id( self::$group_member->get_id() );
		$is_valid     = rcpga_validate_activation_key( $raw_key, $group_member, strtotime( '+10 days' ) );

		$this->assertInstanceOf( \WP_Error::class, $is_valid );
		$this->assertSame( 'expired_key', $is_valid->get_error_code() );
	}

	/**
	 * The invite link duration must be at least 2 hours. Applying a 1 hour filter should still make it 2 hours.
	 *
	 * @covers ::rcpga_get_invite_link_validity_length
	 */
	public function test_validity_duration_cannot_be_less_than_2_hours() {
		add_filter( 'rcpga_invite_link_validity_length', array( $this, 'invite_link_duration_1_hour' ) );

		$this->assertSame( ( 2 * HOUR_IN_SECONDS ), rcpga_get_invite_link_validity_length() );

		remove_filter( 'rcpga_invite_link_validity_length', array( $this, 'invite_link_duration_1_hour' ) );
	}

	/**
	 * A valid key is expired after 4 hours if the duration has been filtered to 3 hours
	 *
	 * @covers ::rcpga_get_invite_link_validity_length
	 * @covers ::rcpga_validate_activation_key
	 */
	public function test_validating_expired_key_after_3_hours_returns_wp_error() {
		add_filter( 'rcpga_invite_link_validity_length', array( $this, 'invite_link_duration_3_hours' ) );

		$this->assertSame( ( 3 * HOUR_IN_SECONDS ), rcpga_get_invite_link_validity_length() );

		$raw_key      = rcpga_generate_member_activation_key( self::$group_member );
		$group_member = rcpga_get_group_member_by_id( self::$group_member->get_id() );
		$is_valid     = rcpga_validate_activation_key( $raw_key, $group_member, strtotime( '+4 hours' ) );

		$this->assertInstanceOf( \WP_Error::class, $is_valid );
		$this->assertSame( 'expired_key', $is_valid->get_error_code() );

		remove_filter( 'rcpga_invite_link_validity_length', array( $this, 'invite_link_duration_3_hours' ) );
	}

}