<?php
/**
 * Group Member Permissions
 *
 * @package   rcp-group-accounts
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCPGA\Tests\Group_Members;

use RCP\Tests\Factory\Customer_Factory;
use RCP\Tests\Factory\Level_Factory;
use RCPGA\Tests\UnitTestCase;

/**
 * Class Member_Permission_Tests
 *
 * @package RCPGA\Tests\Group_Members
 */
class Member_Permission_Tests extends UnitTestCase {

	/**
	 * @var object
	 */
	protected static $membership_level;

	/**
	 * @var object
	 */
	protected static $non_ga_membership_level;

	/**
	 * @var int Number of seats allowed for the membership level.
	 */
	protected static $seats_allowed = 5;

	/**
	 * @var \RCPGA_Group
	 */
	protected static $group;

	/**
	 * @var \RCPGA_Group_Member
	 */
	protected static $group_member;

	/**
	 * @var \WP_Post
	 */
	protected static $post_restricted_to_level;

	/**
	 * @var \WP_Post
	 */
	protected static $post_restricted_to_non_ga_level;

	/**
	 * Set up once before all tests are run
	 */
	public static function setUpBeforeClass() {

		parent::setUpBeforeClass();

		self::$membership_level        = self::rcpga()->level->create_and_get();
		self::$non_ga_membership_level = self::rcpga()->level->create_and_get();

		// Add level meta to enable Group Accounts.
		rcpga_enable_level_group_accounts( self::$membership_level->id );
		rcpga_set_level_group_seats_allowed( self::$membership_level->id, self::$seats_allowed );

		// Create the group owner.
		$customer = self::rcpga()->customer->create_and_get();

		// Grant them a membership.
		$membership_id = $customer->add_membership( array(
			'object_id' => self::$membership_level->id,
			'status'    => 'active'
		) );

		// Now create the group.
		self::$group = self::rcpga()->group->create_and_get( array(
			'owner_id'      => $customer->get_user_id(),
			'membership_id' => $membership_id,
			'seats'         => self::$seats_allowed
		) );

		// Add a group member.
		self::$group_member = self::rcpga()->group_member->create_and_get( array(
			'group_id' => self::$group->get_group_id()
		) );

		// Create a post and restrict it to this level.
		self::$post_restricted_to_level = self::factory()->post->create_and_get();
		update_post_meta( self::$post_restricted_to_level->ID, 'rcp_subscription_level', array( self::$membership_level->id ) );

		// Create a second post and restrict it to the non-Group Accounts level.
		self::$post_restricted_to_non_ga_level = self::factory()->post->create_and_get();
		update_post_meta( self::$post_restricted_to_non_ga_level->ID, 'rcp_subscription_level', array( self::$non_ga_membership_level->id ) );

	}

	/**
	 * @covers ::rcp_user_has_active_membership()
	 */
	public function test_user_has_active_membership_returns_true_for_group_member() {
		$this->assertTrue( rcp_user_has_active_membership( self::$group_member->get_user_id() ) );
	}

	/**
	 * @covers ::rcp_user_has_paid_membership()
	 */
	public function test_user_has_paid_membership_returns_true_for_group_member() {
		$this->assertTrue( rcp_user_has_paid_membership( self::$group_member->get_user_id() ) );
	}

	/**
	 * A user who is not in a group should not be able to access a restricted post.
	 *
	 * @covers ::rcp_user_can_access()
	 */
	public function test_user_without_group_membership_cannot_access_restricted_post() {

		// Create a new user. This will eventually be our group member.
		$user = self::factory()->user->create_and_get();

		$this->assertFalse( rcp_user_can_access( $user->ID, self::$post_restricted_to_level->ID ) );

	}

	/**
	 * Members of a group should be able to access the post restricted to the owner's membership level.
	 *
	 * @covers ::rcp_user_can_access()
	 */
	public function test_group_members_can_access_post_restricted_to_ga_level() {
		$this->assertTrue( rcp_user_can_access( self::$group->get_owner_id(), self::$post_restricted_to_level->ID ) );
		$this->assertTrue( rcp_user_can_access( self::$group_member->get_user_id(), self::$post_restricted_to_level->ID ) );
	}

	/**
	 * Members of a group should not be able to access a post restricted to a different membership level.
	 *
	 * @covers ::rcp_user_can_access()
	 */
	public function test_group_members_cannot_access_post_restricted_to_non_ga_level() {
		$this->assertFalse( rcp_user_can_access( self::$group->get_owner_id(), self::$post_restricted_to_non_ga_level->ID ) );
		$this->assertFalse( rcp_user_can_access( self::$group_member->get_user_id(), self::$post_restricted_to_non_ga_level->ID ) );
	}

	/**
	 * Members of a group should not be able to access a restricted post if the group's membership is expired.
	 *
	 * @covers ::rcp_user_can_access()
	 * @covers ::rcp_user_has_active_membership()
	 * @covers ::rcp_user_has_paid_membership()
	 * @covers ::rcp_user_has_expired_membership()
	 */
	public function test_group_members_cannot_access_restricted_post_if_membership_expired() {
		$membership = rcp_get_membership( self::$group->get_membership_id() );
		$membership->expire();

		$this->assertFalse( rcp_user_can_access( self::$group->get_owner_id(), self::$post_restricted_to_level->ID ) );
		$this->assertFalse( rcp_user_can_access( self::$group_member->get_user_id(), self::$post_restricted_to_level->ID ) );

		$this->assertFalse( rcp_user_has_active_membership( self::$group_member->get_user_id() ) );
		$this->assertFalse( rcp_user_has_paid_membership( self::$group_member->get_user_id() ) );
		$this->assertTrue( rcp_user_has_expired_membership( self::$group_member->get_user_id() ) );
	}

	/**
	 * Members of a group should be able to access a restricted post after the expired membership is renewed.
	 *
	 * @covers ::rcp_user_can_access()
	 */
	public function group_members_can_access_restricted_post_after_renewal() {
		$membership = rcp_get_membership( self::$group->get_membership_id() );
		$membership->renew();

		$this->assertTrue( rcp_user_can_access( self::$group->get_owner_id(), self::$post_restricted_to_level->ID ) );
		$this->assertTrue( rcp_user_can_access( self::$group_member->get_user_id(), self::$post_restricted_to_level->ID ) );
	}

}