<?php
/**
 * Group Tests
 *
 * @package   rcp-group-accounts
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCPGA\Tests\Groups;

use RCPGA\Tests\UnitTestCase;

/**
 * Class Group_Tests
 *
 * @package RCPGA\Tests\Groups
 */
class Group_Tests extends UnitTestCase {

	/**
	 * Group Accounts enabled membership level.
	 * Allows 3 seats.
	 *
	 * @var object|\WP_Error
	 */
	protected static $level_1;

	/**
	 * Regular membership level. Not Group Accounts enabled.
	 *
	 * @var object|\WP_Error
	 */
	protected static $level_2;

	/**
	 * @var \RCPGA_Group
	 */
	protected static $group;

	/**
	 * Set up once before all tests are run
	 */
	public static function setUpBeforeClass() {
		parent::setUpBeforeClass();

		self::$level_1 = self::rcpga()->level->create_and_get();
		self::$level_2 = self::rcpga()->level->create_and_get();

		// Enable Group Accounts on level 1.
		rcpga_enable_level_group_accounts( self::$level_1->id );
		rcpga_set_level_group_seats_allowed( self::$level_1->id, 3 );

		// Create our group owner's customer and membership record.
		$customer      = self::rcpga()->customer->create_and_get();
		$membership_id = $customer->add_membership( array(
			'object_id' => self::$level_1->id,
			'status'    => 'active'
		) );

		self::$group = self::rcpga()->group->create_and_get( array(
			'owner_id'      => $customer->get_user_id(),
			'membership_id' => $membership_id,
			'seats'         => 3
		) );
	}

	/**
	 * @covers RCPGA_Group::get_seats
	 */
	public function test_seat_count_is_3() {
		$this->assertEquals( 3, self::$group->get_seats() );
	}

	/**
	 * @covers RCPGA_Group::get_member_count
	 */
	public function test_member_count_is_1() {
		$this->assertEquals( 1, self::$group->get_member_count() );
	}

	/**
	 * @covers RCPGA_Group::add_member
	 */
	public function test_adding_member_increments_member_count() {
		self::rcpga()->group_member->create( array(
			'group_id' => self::$group->get_group_id()
		) );

		// Refresh group object to get new member count.
		self::$group = self::rcpga()->group->get_object_by_id( self::$group->get_group_id() );

		$this->assertEquals( 2, self::$group->get_member_count() );
	}

	/**
	 * @covers RCPGA_Group::can_add_members
	 */
	public function test_can_add_members_returns_true_if_seat_limit_not_reached() {
		$this->assertTrue( self::$group->can_add_members() );
	}

	/**
	 * A user with their own membership record cannot be added to a group.
	 *
	 * @covers RCPGA_Group::add_member
	 */
	public function test_user_with_membership_cannot_be_added_to_group() {

		$customer = self::rcpga()->customer->create_and_get();
		$customer->add_membership( array(
			'object_id' => self::$level_2->id,
			'status'    => 'active'
		) );

		$this->assertWPError( self::$group->add_member( array(
			'user_id' => $customer->get_user_id()
		) ) );

	}

	/**
	 * @covers RCPGA_Group::can_add_members
	 */
	public function test_can_add_members_returns_false_if_seat_limit_is_reached() {
		// We need to add two more members.
		self::rcpga()->group_member->create_many( 2, array(
			'group_id' => self::$group->get_group_id()
		) );

		// Refresh group object to get new member count.
		self::$group = self::rcpga()->group->get_object_by_id( self::$group->get_group_id() );

		$this->assertEquals( 3, self::$group->get_member_count() );

		$this->assertFalse( self::$group->can_add_members() );
	}

}