<?php
/**
 * Help Scout Plugin
 *
 * Handles the response to Help Scout queries
 *
 * @package   rcp-help-scout
 * @copyright Copyright (c) 2019, Restrict Content Pro team
 * @license   GPL2+
 * @since     1.0
 */

/**
 * Class PW_RCP_Help_Scout_Plugin_Handler
 *
 * @since 1.0
 */
class PW_RCP_Help_Scout_Plugin_Handler {

	private $input   = false;
	private $secret  = '';
	private $email   = '';

	/**
	 * PW_RCP_Help_Scout_Plugin_Handler constructor.
	 *
	 * @since 1.0
	 */
	public function __construct() {
		global $rcp_options;

		if( ! empty( $rcp_options['hs_secret_key'] ) ) {
			$this->secret = trim( $rcp_options[ 'hs_secret_key' ] );
		}

		if( ! empty( $rcp_options['hs_email'] ) ) {
			$this->email = trim( $rcp_options[ 'hs_email' ] );
		}
	}

	/**
	 * Returns the requested HTTP header.
	 *
	 * @param string $header
	 *
	 * @since 1.0
	 * @return bool|string
	 */
	private function getHeader( $header ) {
		if ( isset( $_SERVER[$header] ) ) {
			return $_SERVER[$header];
		}
		return false;
	}

	/**
	 * Retrieve the JSON input
	 *
	 * @since 1.0
	 * @return bool|string
	 */
	private function getJsonString() {
		if ( $this->input === false ) {
			$this->input = @file_get_contents( 'php://input' );
		}
		return $this->input;
	}

	/**
	 * Generate the signature based on the secret key, to compare in isSignatureValid
	 *
	 * @since 1.0
	 * @return bool|string
	 */
	private function generateSignature() {
		$str = $this->getJsonString();
		if ( $str ) {
			return base64_encode( hash_hmac( 'sha1', $str, $this->secret, true ) );
		}
		return false;
	}

	/**
	 * Returns true if the current request is a valid webhook issued from Help Scout, false otherwise.
	 *
	 * @since 1.0
	 * @return boolean
	 */
	private function isSignatureValid() {

		$signature = $this->generateSignature();

		if ( ! $signature || ! $this->getHeader( 'HTTP_X_HELPSCOUT_SIGNATURE' ) ) {
			return false;
		}

		return $signature == $this->getHeader( 'HTTP_X_HELPSCOUT_SIGNATURE' );
	}

	/**
	 * Create a response.
	 *
	 * @since 1.0
	 * @return array
	 */
	public function getResponse() {

		$ret = array( 'html' => '' );

		if ( ! $this->isSignatureValid() ) {
			return array( 'html' => 'Invalid signature' );
		}

		$data = json_decode( $this->input, true );

		// do some stuff
		$ret['html'] = $this->fetchHtml( $data );

		// Used for debugging
		// $ret['html'] = '<pre>'.print_r($data,1).'</pre>' . $ret['html'];

		return $ret;
	}

	/**
	 * Generate output for the response.
	 *
	 * @param array $data
	 *
	 * @since 1.0
	 * @return string
	 */
	private function fetchHtml( $data ) {

		$emails = array();

		if ( isset( $data['customer']['emails'] ) && is_array( $data['customer']['emails'] ) ) {

			if( ( $key = array_search( $this->email, $data['customer']['emails'] ) ) !== false ) {
			    unset( $data['customer']['emails'][ $key ] );
			}

			$emails = $data['customer']['emails'];

		} else {

			if ( $data['customer']['email'] == $this->email ) {
				return sprintf( __( 'Cannot query member account. E-mail from %s', 'rcp-help-scout' ), $this->email );
			}

			$emails[] = $data['customer']['email'];

		}

		$user  = false;
		$email = isset( $emails[0] ) ? $emails[0] : '';

		foreach( $emails as $email ) {

			$user = get_user_by( 'email', $email );
			if( $user ) {
				break;
			}

		}


		if( ! $user ) {

			return sprintf( __( 'No member found for %s', 'rcp-help-scout' ), $email );

		}

		$html = '';

		if ( version_compare( RCP_PLUGIN_VERSION, '3.0', '>=' ) ) {

			$customer = rcp_get_customer_by_user_id( $user->ID );

			if ( empty( $customer ) ) {
				return sprintf( __( 'No customer found for %s', 'rcp-help-scout' ), $user->display_name );
			}

			$memberships = $customer->get_memberships();

			if ( empty( $memberships ) ) {
				return $html;
			}

			$membership_info = array();

			foreach ( $memberships as $membership ) {

				/**
				 * @var RCP_Membership $membership
				 */

				$membership_level = rcp_get_subscription_details( $membership->get_object_id() );

				$membership_html = $membership_level->name . ' - ';

				if ( 'cancelled' == $membership->get_status() ) {
					$membership_html .= '<span class="orange">' . __( 'Cancelled', 'rcp-help-scout' ) . '</span>';
				} elseif ( $membership->is_active() ) {
					$membership_html .= '<span class="green">' . __( 'Active', 'rcp-help-scout' ) . '</span>';
				} else {
					$membership_html .= '<span class="red">' . __( 'Expired', 'rcp-help-scout' ) . '</span>';
				}

				$membership_html .= '<br>';
				$membership_html .= $membership->get_formatted_billing_cycle() . '<br>';

				// Times billed
				if ( 0 == $membership->get_maximum_renewals() && $membership_level->duration > 0 && $membership_level->price > 0 ) {
					$membership_html .= sprintf( __( 'Times billed: %d / until cancelled', 'rcp-help-scout' ), $membership->get_times_billed() );
				} else {
					$renewals = ( 0 == $membership_level->price ) ? 1 : $membership->get_maximum_renewals() + 1;

					$membership_html .= sprintf( __( 'Times billed: %d / %d', 'rcp' ), $membership->get_times_billed(), $renewals );
				}

				$membership_html .= '<br>';

				// Expiration date
				if ( $membership->is_trialing() ) {
					$membership_html .= sprintf( __( 'Trial end date: %s', 'rcp-help-scout' ), $membership->get_trial_end_date() );
				} else {
					$membership_html .= sprintf( __( 'Expiration date: %s', 'rcp-help-scout' ), $membership->get_expiration_date( false ) );
				}

				// Edit link
				$membership_html .= '<br><br><a href="' . esc_url( add_query_arg( 'membership_id', urlencode( $membership->get_id() ), admin_url( 'admin.php?page=rcp-members&view=edit' ) ) ) . '">' . __( 'Edit Membership Details', 'rcp-help-scout' ) . '</a>';

				$membership_info[] = $membership_html;

			}

			$last_login = $customer->get_last_login();
			if ( empty( $last_login ) ) {
				$last_login = __( 'Unknown', 'rcp-help-scout' );
			}

			ob_start();
			?>
			<div class="c-sb-section c-sb-section--toggle">
				<div class="c-sb-section__title js-sb-toggle">
					<i class="icon-person"></i> <?php echo ! empty( $user->display_name ) ? esc_html( $user->display_name ) : esc_html( $user->user_login ); ?>
					<i class="caret sb-caret"></i>
				</div>
				<div class="c-sb-section__body">
					<ul class="c-sb-list--two-line">
						<li class="c-sb-list-item">
							<span class="c-sb-list-item__label">
								<?php printf( __( 'Customer Since: %s', 'rcp-help-scout' ), '<span class="c-sb-list-item__text">' . $customer->get_date_registered() . '</span>' ); ?>
							</span>
						</li>
						<li class="c-sb-list-item">
							<span class="c-sb-list-item__label">
								<?php printf( __( 'Lifetime Value: %s', 'rcp-help-scout' ), '<span class="c-sb-list-item__text">' . rcp_currency_filter( $customer->get_lifetime_value() ) . '</span>' ); ?>
							</span>
						</li>
						<li class="c-sb-list-item">
							<span class="c-sb-list-item__label">
								<?php printf( __( 'Last Login: %s', 'rcp-help-scout' ), '<span class="c-sb-list-item__text">' .  $last_login . '</span>' ); ?>
							</span>
						</li>
						<li class="c-sb-list-item">
							<a href="<?php echo esc_url( add_query_arg( 'customer_id', urlencode( $customer->get_id() ), admin_url( 'admin.php?page=rcp-customers&view=edit' ) ) ); ?>"><?php _e( 'Edit Customer Details', 'rcp-help-scout' ); ?></a>
						</li>
					</ul>
				</div>
			</div>
			<div class="c-sb-section c-sb-section--toggle open">
				<div class="c-sb-section__title js-sb-toggle">
					<i class="icon-star icon-sb"></i> <?php printf( __( 'Memberships (%d)', 'rcp-help-scout' ), count( $memberships ) ); ?>
					<i class="caret sb-caret"></i>
				</div>
				<div class="c-sb-section__body">
					<ul>
						<li><?php echo implode( '</li><li>', $membership_info ); ?></li>
					</ul>
				</div>
			</div>
			<?php
			$html = ob_get_clean();

		} else {

			$member = new RCP_Member( $user->ID );

			if ( $member->get_subscription_id() ) {

				$rcpinfo = array();

				if ( $member->is_active() ) {
					$rcp = sprintf( '<strong style="color: green;">' . __( 'Valid Subscription', 'rcp-help-scout' ) . '</strong> %s', $subscription );
				} elseif ( 'cancelled' == $member->get_status() ) {
					$rcp = sprintf( '<strong style="color: gray;">' . __( 'Subscription Cancelled', 'rcp-help-scout' ) . '</strong> %s', $subscription );
				} elseif ( $member->is_expired() ) {
					$rcp = sprintf( '<strong style="color: red;">' . __( 'Subscription Expired', 'rcp-help-scout' ) . '</strong> %s', $subscription );
				}

				$rcpinfo[] = $rcp;
				$rcpinfo[] = sprintf( '<strong>' . __( 'Status', 'rcp-help-scout' ) . '</strong>: %s', $member->get_status() );
				$rcpinfo[] = sprintf( '<strong>' . __( 'Subscription Level', 'rcp-help-scout' ) . '</strong>: %s', $member->get_subscription_name() );
				$rcpinfo[] = sprintf( '<strong>' . __( 'Expiration', 'rcp-help-scout' ) . '</strong>: %s', $member->get_expiration_date() );
				$rcpinfo[] = sprintf( '<a href="%s">' . __( 'View Member Details', 'rcp-help-scout' ) . '</a>', admin_url( 'admin.php?page=rcp-members&edit_member=' . $member->ID ) );

				$html .= '<h4 class="toggleBtn"><i class="icon-gear"></i> ' . __( 'Subscription', 'rcp-help-scout' ) . '</h4>';
				$html .= '<ul><li>';
				$html .= implode( '</li><li>', $rcpinfo );
				$html .= '</li></ul>';

			}

		}

		return $html;
	}
}