<?php
/**
 * Filters
 *
 * @package     RCP\IP_Restriction\Filters
 * @since       1.0.0
 */


// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Override RCP can_access
 *
 * @since       1.0.0
 * @param       bool $can_access The current access condition
 * @param       int $user_id The ID of the current user
 * @param       int $post_id The ID of the current post
 * @param       object $member The RCP Member object
 * @return      bool $can_access True if whitelisted, false otherwise
 */
function rcp_ip_restriction_member_can_access( $can_access, $user_id = 0, $post_id = 0, $member = false ) {

	// Bail if the user already has access.
	if ( $can_access ) {
		return $can_access;
	}

	// Check global, membership level, and individual post whitelists.
	if ( rcp_ip_restriction_is_whitelisted( $post_id ) ) {
		$can_access = true;
	}

	return $can_access;
}
add_filter( 'rcp_member_can_access', 'rcp_ip_restriction_member_can_access', 10, 4 );


/**
 * Override restrict shortcode
 *
 * @since       1.0.1
 * @param       bool $has_access The current access condition
 * @param       int $user_id The ID of the current user
 * @param       array $atts Arguements passed to the shortcode
 * @return      bool $has_access True if whitelisted, false otherwise
 */
function rcp_ip_restriction_shortcode_can_access( $has_access, $user_id, $atts ) {

	// Bail early if they already have access.
	if ( $has_access ) {
		return $has_access;
	}

	// Bail early if pending email verification.
	if ( rcp_is_pending_verification() ) {
		return $has_access;
	}

	// Grant access if they're globally whitelisted.
	if ( rcp_ip_restriction_is_whitelisted( get_the_ID() ) ) {
		return true;
	}

	$spoofed_subscription_id = rcp_ip_restriction_get_subscription_level_for_ip();

	// Bail if this user doesn't have a subscription level whitelist.
	if ( empty( $spoofed_subscription_id ) ) {
		return $has_access;
	}

	// Bail if restricted by user role and user is not logged in.
	if ( ! empty( $atts['userlevel'] ) && 'none' != $atts['userlevel'] && ! is_user_logged_in() ) {
		return $has_access;
	}

	/*
	 * Bail if "paid", "level", and "subscription" attributes are not being used.
	 * This means the restriction isn't related to subscription levels.
	 */
	if ( empty( $atts['paid'] ) && empty( $atts['level'] ) && empty( $atts['subscription'] ) ) {
		return $has_access;
	}

	$required_subscription = array_map( 'trim', explode( ',', $atts['subscription'] ) );

	// Check "paid" status.
	if( $atts['paid'] && in_array( $spoofed_subscription_id, wp_list_pluck( rcp_get_paid_levels(), 'id' ) ) ) {
		$has_access = true;
	}

	// Check membership level requirements.
	if ( ! empty( $required_subscription ) && ! empty( $required_subscription[0] ) ) {
		if ( in_array( $spoofed_subscription_id, $required_subscription ) ) {
			$has_access = true;
		} else {
			$has_access = false;
		}
	}

	// Check access level.
	if ( ! empty( $atts['level'] ) ) {
		$access_level_needed = absint( $atts['level'] );
		$spoofed_access      = rcp_get_subscription_access_level( $spoofed_subscription_id );

		if ( ( $spoofed_access >= $access_level_needed ) || $access_level_needed == 0 ) {
			// The membership has the access level or higher.
			$has_access = true;
		} else {
			$has_access = false;
		}
	}

	return $has_access;

}
add_filter( 'rcp_restrict_shortcode_has_access', 'rcp_ip_restriction_shortcode_can_access', 10, 3 );

/**
 * Override the current user's subscription level if their IP is whitelisted for a subscription level.
 *
 * @deprecated
 *
 * @param int|false  $subscription_id Subscription level ID for this member.
 * @param int        $user_id         ID of the user.
 * @param RCP_Member $member          Member object.
 *
 * @return false|int
 */
function rcp_ip_restriction_filter_subscription_id( $subscription_id, $user_id, $member ) {

	// Bail if the user being checked isn't the current user.
	if ( $user_id != get_current_user_id() ) {
		return $subscription_id;
	}

	$spoofed_subscription_id = rcp_ip_restriction_get_subscription_level_for_ip();

	if ( ! empty( $spoofed_subscription_id ) ) {
		$subscription_id = $spoofed_subscription_id;
	}

	return $subscription_id;

}
add_filter( 'rcp_member_get_subscription_id', 'rcp_ip_restriction_filter_subscription_id', 10, 3 );

/**
 * Override the current user's expiration date if their IP is whitelisted for a subscription level.
 *
 * @deprecated
 *
 * @param string     $expiration Expiration date for this member.
 * @param int        $user_id    ID of the user.
 * @param RCP_Member $member     Member object.
 * @param bool       $formatted  Whether or not the returned value should be formatted.
 * @param bool       $pending    Whether or not to check the pending expiration date.
 *
 * @return string
 */
function rcp_ip_restriction_filter_expiration_date( $expiration, $user_id, $member, $formatted = true, $pending = true ) {

	// Bail if the user being checked isn't the current user.
	if ( $user_id != get_current_user_id() ) {
		return $expiration;
	}

	$spoofed_subscription_id = rcp_ip_restriction_get_subscription_level_for_ip();

	if ( ! empty( $spoofed_subscription_id ) ) {
		$expiration = 'none';
	}

	return $expiration;

}
add_filter( 'rcp_member_get_expiration_date', 'rcp_ip_restriction_filter_expiration_date', 10, 5 );

/**
 * Override the current user's status date if their IP is whitelisted for a subscription level.
 *
 * @deprecated
 *
 * @param string     $status  Status for this member.
 * @param int        $user_id ID of the user.
 * @param RCP_Member $member  Member object.
 *
 * @return string
 */
function rcp_ip_restriction_filter_status( $status, $user_id, $member ) {

	// Bail if the user being checked isn't the current user.
	if ( $user_id != get_current_user_id() ) {
		return $status;
	}

	$spoofed_subscription_id = rcp_ip_restriction_get_subscription_level_for_ip();

	if ( ! empty( $spoofed_subscription_id ) ) {
		/**
		 * @var RCP_Levels $rcp_levels_db
		 */
		global $rcp_levels_db;

		$subscription_level = $rcp_levels_db->get_level( $spoofed_subscription_id );

		$status = ( 0 == $subscription_level->price && 0 == $subscription_level->duration ) ? 'free' : 'active';
	}

	return $status;

}
add_filter( 'rcp_member_get_status', 'rcp_ip_restriction_filter_status', 10, 3 );

/**
 * Override the current user's activity if their IP is whitelisted for a paid subscription level.
 *
 * @deprecated
 *
 * @param bool       $is_active Whether or not the current user is active (paid).
 * @param int        $user_id   ID of the user.
 * @param RCP_Member $member    Member object.
 *
 * @return bool
 */
function rcp_ip_restriction_filter_active( $is_active, $user_id, $member ) {

	// Bail if the user being checked isn't the current user.
	if ( $user_id != get_current_user_id() ) {
		return $is_active;
	}

	$spoofed_subscription_id = rcp_ip_restriction_get_subscription_level_for_ip();

	if ( ! empty( $spoofed_subscription_id ) ) {
		/**
		 * @var RCP_Levels $rcp_levels_db
		 */
		global $rcp_levels_db;

		$subscription_level = $rcp_levels_db->get_level( $spoofed_subscription_id );

		$is_active = ( 0 == $subscription_level->price && 0 == $subscription_level->duration ) ? false : true;
	}

	return $is_active;

}
add_filter( 'rcp_is_active', 'rcp_ip_restriction_filter_active', 10, 3 );