<?php
/**
 * Helper functions
 *
 * @package     RCP\IP_Restriction\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Sanitize IP fields
 *
 * @since       1.0.0
 * @param       string $ip_addresses Addresses in string format
 * @return      array $ip_addresses Addresses in array format
 */
function rcp_ip_restriction_sanitize_ips( $ip_addresses = '' ) {
	if ( ! is_array( $ip_addresses ) ) {
		$ip_addresses = explode( "\n", $ip_addresses );
	}
	$ip_addresses = array_map( 'trim', $ip_addresses );
	$ip_addresses = array_unique( $ip_addresses );
	$ip_addresses = array_map( 'sanitize_text_field', $ip_addresses );

	return $ip_addresses;
}


/**
 * Check if a given IP address is whitelisted to view a specified post.
 *
 * @since       1.0.0
 * @param       int $post_id The ID of a given post
 * @return      bool $is_whitelisted True if whitelisted, false otherwise
 */
function rcp_ip_restriction_is_whitelisted( $post_id ) {
	$is_whitelisted = false;
	$ip_address     = rcp_get_ip();

	// Check global whitelist
	$settings         = get_option( 'rcp_ip_restriction_settings' );
	$global_whitelist = ( isset( $settings['whitelist'] ) ? $settings['whitelist'] : array() );

	foreach ( $global_whitelist as $whitelist_address ) {
		if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelist_address ) ) {
			$is_whitelisted = true;
			break;
		}
	}

	// Maybe check post whitelist
	if ( ! $is_whitelisted ) {
		$post_whitelist = get_post_meta( $post_id, '_whitelist', true );
		$post_whitelist = ( is_array( $post_whitelist ) ? $post_whitelist : array() );

		foreach ( $post_whitelist as $whitelist_address ) {
			if ( ! empty( $whitelist_address ) && rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelist_address ) ) {
				$is_whitelisted = true;
				break;
			}
		}
	}

	// Maybe check membership level whitelist
	if ( ! $is_whitelisted ) {
		// @todo consider checking access levels
		$post_type_restrictions = rcp_get_post_type_restrictions( get_post_type( $post_id ) );

		if ( empty( $post_type_restrictions ) ) {
			$post_levels = get_post_meta( $post_id, 'rcp_subscription_level', true );
		} else {
			$post_levels = array_key_exists( 'subscription_level', $post_type_restrictions ) ? $post_type_restrictions['subscription_level'] : false;
		}

		$is_whitelisted = rcp_ip_restriction_is_ip_whitelisted_for_level( $post_levels, $ip_address );
	}

	// Maybe check whitelists on this post's terms
	if ( ! $is_whitelisted ) {
		$terms = (array) rcp_get_connected_term_ids( $post_id );
		
		if ( ! empty( $terms ) ) {
			foreach ( $terms as $term_id ) {
				$restrictions = rcp_get_term_restrictions( $term_id );

				if ( ! empty( $restrictions['subscriptions'] ) ) {
					$is_whitelisted = rcp_ip_restriction_is_ip_whitelisted_for_level( $restrictions['subscriptions'], $ip_address );
				}
			}
		}
	}

	return $is_whitelisted;
}

/**
 * Check if a certain membership level or array of levels has a certain IP address whitelisted.
 *
 * @param array|int|string $level      Level(s) to check. Accepts an array of membership level IDs, a single membership ID as an integer, or a string `all` (all levels), `any-paid (any paid level).
 * @param string           $ip_address IP address to check in the whitelist.
 *
 * @return bool
 */
function rcp_ip_restriction_is_ip_whitelisted_for_level( $level, $ip_address = '' ) {

	$level_whitelists = get_option( 'rcp_ip_restriction_subscription_whitelists' );
	$is_whitelisted   = false;

	if ( empty( $level ) ) {
		return $is_whitelisted;
	}

	if ( empty( $ip_address ) ) {
		$ip_address = rcp_get_ip();
	}

	if ( 'all' == $level || 'any' == $level ) {
		$membership_level_ids = wp_list_pluck( rcp_get_subscription_levels(), 'id' );
	} elseif ( 'any-paid' == $level ) {
		$paid_levels = rcp_get_paid_levels();
		$membership_level_ids   = wp_list_pluck( $paid_levels, 'id' );
	} elseif ( ! is_array( $level ) ) {
		$membership_level_ids = array( $level );
	} else {
		$membership_level_ids = $level;
	}

	if ( ! is_array( $level_whitelists ) || ! is_array( $membership_level_ids ) ) {
		return $is_whitelisted;
	}

	foreach ( $level_whitelists as $membership_level_id => $whitelist ) {
		if ( in_array( $membership_level_id, $membership_level_ids ) ) {
			foreach ( $whitelist as $whitelist_address ) {
				if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelist_address ) ) {
					$is_whitelisted = true;
					break;
				}
			}
		}
	}

	return $is_whitelisted;

}


/**
 * Check if a given IP address is blacklisted
 *
 * @since       1.0.0
 * @return      bool $is_blacklisted True if blacklisted, false otherwise
 */
function rcp_ip_restriction_is_blacklisted() {
	$is_blacklisted = false;
	$ip_address     = rcp_get_ip();

	$settings         = get_option( 'rcp_ip_restriction_settings' );
	$global_blacklist = ( isset( $settings['blacklist'] ) ? $settings['blacklist'] : array() );

	foreach ( $global_blacklist as $blacklist_address ) {
		if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $blacklist_address ) ) {
			$is_blacklisted = true;
			break;
		}
	}

	return $is_blacklisted;
}

/**
 * Returns the subscription level ID for a given IP address if the IP is on a subscription
 * level whitelist. The user with this IP is effectively given this subscription level.
 *
 * @see rcp_ip_restriction_filter_subscription_id()
 *
 * @param string $ip_address IP address to get the subscription level of. Leave blank to use current user's IP.
 *
 * @return int|false ID of the spoofed subscription level, or false if none.
 */
function rcp_ip_restriction_get_subscription_level_for_ip( $ip_address = '' ) {

	$subscription_whitelists = get_option( 'rcp_ip_restriction_subscription_whitelists' );
	$subscription_id         = false;

	// No subscription level whitelists set - bail.
	if ( empty( $subscription_whitelists ) || ! is_array( $subscription_whitelists ) ) {
		return $subscription_id;
	}

	if ( empty( $ip_address ) ) {
		$ip_address = rcp_get_ip();
	}

	foreach ( $subscription_whitelists as $whitelist_sub_id => $sub_level_whitelist ) {
		foreach ( $sub_level_whitelist as $whitelisted_ip ) {
			if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelisted_ip ) ) {
				$subscription_id = $whitelist_sub_id;
				break;
			}
		}
	}

	return $subscription_id;

}