<?php
/**
 * IPCheck class helps validate an IP address against an IP or range.
 *
 * @package     IPCheck
 * @author      Daniel J Griffiths
 * @version     1.0.0
 */
class IPCheck {


	/**
	 * @var         bool $matches Whether or not the IP matches
	 * @since       1.0.0
	 */
	public $matches = false;


	/**
	 * Match an address against a range
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       string $ip The address to check
	 * @param       string $range The range to check against
	 * @return      bool $matches If the IP is in the specified range
	 */
	public function match( $ip = '', $range = '' ) {
		if ( $ip && $ip !== '' && $range && $range !== '' ) {
			if ( strstr( $range, '.' )  ) {
				// Run IPv4 checks
				$this->check_ipv4( $ip, $range );
			} elseif ( strstr( $range, ':' ) ) {
				// Run IPv6 checks
				$this->check_ipv6( $ip, $range );
			}
		}

		return $this->matches;
	}


	/**
	 * Checks an IPv4 address
	 *
	 * Supports the following formats:
	 * 1. Wildcard:  1.2.3.*
	 * 2. CIDR:      1.2.3/24 or 1.2.3.4/255.255.255.0
	 * 3. Start/End: 1.2.3.0-1.2.3.255
	 *
	 * @access      private
	 * @since       1.0.0
	 * @param       string $ip The address to check
	 * @param       string $range The range to check against
	 * @return      void
	 */
	private function check_ipv4( $ip, $range ) {
		$this->matches = false;

		if ( strpos( $range, '/' ) !== false ) {
			// Range is in IP/NETMASK format
			list( $range, $netmask ) = explode( '/', $range, 2 );

			if ( strpos( $netmask, '.' ) !== false ) {
				// Netmask is in decimal format
				$netmask     = str_replace( '*', '0', $netmask );
				$netmask_dec = ip2long( $netmask );

				$this->matches = ( ( ip2long( $ip ) & $netmask_dec ) == ( ip2long( $range ) & $netmask_dec ) );
			} else {
				// Netmask is a CIDR block
				$x = explode( '.', $range );

				while ( count( $x ) < 4 ) {
					$x[] = '0';
				}

				list( $a, $b, $c, $d ) = $x;

				$range        = sprintf( "%u.%u.%u.%u", empty( $a ) ? '0' : $a, empty( $b ) ? '0' : $b, empty( $c ) ? '0' : $c, empty( $d ) ? '0' : $d );
				$range_dec    = ip2long( $range );
				$ip_dec       = ip2long( $ip );
				$wildcard_dec = pow( 2, ( 32 - $netmask ) );
				$netmask_dec  = - $wildcard_dec;

				$this->matches = ( ( $ip_dec & $netmask_dec ) == ( $range_dec & $netmask_dec ) );
			}
		} else {
			// Check if we already have a range.
			if ( false !== strpos( $range, '-' ) ) {
				list( $lower, $upper ) = explode( '-', $range, 2 );

				// Fill up any missing octets with another asterisk.
				$lower = $this->fill_octets( $lower );
				$upper = $this->fill_octets( $upper );

				/**
				 * Handle wildcards within a range.
				 *
				 * @link https://github.com/restrictcontentpro/rcp-ip-restriction/issues/31
				 */
				$lower = str_replace( '*', '0', $lower );
				$upper = str_replace( '*', '255', $upper );
				$range = "$lower-$upper";
			}

			if ( strpos( $range, '*' ) !== false ) {
				// Range is in a.b.*.* format

				// Fill up any missing octets with another asterisk.
				$range = $this->fill_octets( $range );

				$lower = str_replace( '*', '0', $range );
				$upper = str_replace( '*', '255', $range );
				$range = "$lower-$upper";
			}

			if ( strpos( $range, '-' ) !== false ) {
				// Range is in A-B format
				list( $lower, $upper ) = explode( '-', $range, 2 );

				$lower_dec = (float) sprintf( "%u", ip2long( $lower ) );
				$upper_dec = (float) sprintf( "%u", ip2long( $upper ) );
				$ip_dec    = (float) sprintf( "%u", ip2long( $ip ) );

				$this->matches = ( ( $ip_dec >= $lower_dec ) && ( $ip_dec <= $upper_dec ) );
			}

			if ( $ip === $range ) {
				$this->matches = true;
			}
		}
	}

	/**
	 * Fill up an IP address with any missing octets. Missing octets are given a wildcard.
	 *
	 * Example: `127.0.*` becomes: `127.0.*.*`
	 *
	 * @param string $ip
	 *
	 * @since 1.2.7
	 * @return string
	 */
	private function fill_octets( $ip ) {
		$octets = explode( '.', $ip );
		while ( count( $octets ) < 4 ) {
			$octets[] = '*';
		}

		return implode( '.', $octets );
	}

	/**
	 * Checks an IPv6 address
	 *
	 * @access      private
	 * @since       1.0.0
	 *
	 * @param string $ip    The address to check
	 * @param string $range The range to check against
	 *
	 * @return      void
	 */
	private function check_ipv6( $ip, $range ) {
		$this->matches = false;

		$pieces      = explode( '/', $range, 2 );
		$left_piece  = $pieces[0];
		$right_piece = ( count( $pieces ) == 2 ? $pieces[1] : '' );

		// Extract out the main pieces
		$ip_pieces     = explode( '::', $left_piece, 2 );
		$main_ip_piece = $ip_pieces[0];
		$last_ip_piece = ( count( $ip_pieces ) == 2 ? $ip_pieces[1] : '' );

		// Pad out shorthand
		$main_ip_pieces = explode( ':', $main_ip_piece );

		foreach ( $main_ip_pieces as $key => $val ) {
			$main_ip_pieces[ $key ] = str_pad( $main_ip_pieces[ $key ], 4, '0', STR_PAD_LEFT );
		}

		// Create the pieces that denote the range
		$first = $main_ip_pieces;
		$last  = $main_ip_pieces;

		// Check to ensure the last block is set
		$last_piece = '';
		$size       = count( $main_ip_pieces );

		if ( trim( $last_ip_piece ) != '' ) {
			$last_piece = str_pad( $last_ip_piece, 4, '0', STR_PAD_LEFT );

			// Build the full form address
			for ( $i = $size; $i < 7; $i++ ) {
				$first[ $i ] = '0000';
				$last[ $i ]  = 'ffff';
			}

			$main_ip_pieces[7] = $last_piece;
		} else {
			// Build the full form address
			for ( $i = $size; $i < 8; $i++ ) {
				$first[ $i ] = '0000';
				$last[ $i ]  = 'ffff';
			}
		}

		// Rebuild the long form address
		$first = $this->ip2long6( implode( ':', $first ) );
		$last  = $this->ip2long6( implode( ':', $last ) );
		$ip    = $this->ip2long6( $ip );

		if ( $ip === $range ) {
			$this->matches = true;
		} else {
			$this->matches = ( $ip >= $first && $ip <= $last );
		}
	}


	/**
	 * Convert an IPv6 address to long
	 *
	 * @access      private
	 * @since       1.0.0
	 * @param       string $ip The address to convert
	 * @return      string The converted address
	 */
	private function ip2long6( $ip ) {
		if ( substr_count( $ip, '::' ) ) {
			$ip = str_replace( '::', str_repeat( ':0000', 8 - substr_count( $ip, ':' ) ) . ':', $ip );
		}

		$ip   = explode( ':', $ip );
		$r_ip = '';

		foreach ( $ip as $v ) {
			$r_ip .= str_pad( base_convert( $v, 16, 2 ), 16, 0, STR_PAD_LEFT );
		}

		return base_convert( $r_ip, 2, 10 );
	}
}
