<?php
/**
 * IP CIDR Blocks
 *
 * Test that we can match IPs against whitelists containing CIDR blocks.
 *
 * Example: `10.0.0.0/8` on whitelist, so `10.155.155.155` should match.
 *
 * @package   rcp-ip-restriction
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCP_IP_Restriction\Tests;

/**
 * Class Test_IP_CIDR
 *
 * @package RCP_IP_Restriction\Tests
 */
class Test_IP_CIDR extends UnitTestCase {

	/**
	 * Set up once before tests are run
	 */
	public static function setUpBeforeClass() {
		parent::setUpBeforeClass();

		// Update global whitelist.
		$settings = array(
			'whitelist' => array(
				'10.0.0.0/8', // CIDR
			)
		);

		update_option( 'rcp_ip_restriction_settings', $settings );
	}

	/**
	 * If an IP matches the start of a whitelisted CIDR block, it should be allowed.
	 *
	 * Whitelist: `10.0.0.0/8`
	 * IP: `10.0.0.0`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_start_of_cidr_is_whitelisted() {

		self::$ip = '10.0.0.0';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches a whitelisted CIDR block and ends in `.1`, it should be allowed.
	 *
	 * Whitelist: `10.0.0.0/8`
	 * IP: `10.0.0.1`
	 *
	 * @link https://github.com/restrictcontentpro/rcp-ip-restriction/issues/24
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_inside_cidr_ending_1_is_whitelisted() {

		self::$ip = '10.0.0.1';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches the end of a whitelisted CIDR block, it should be allowed.
	 *
	 * Whitelist: `10.0.0.0/8`
	 * IP: `10.255.255.255`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_end_of_cidr_is_whitelisted() {

		self::$ip = '10.255.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches the middle of a whitelisted CIDR block, it should be allowed.
	 *
	 * Whitelist: `10.0.0.0/8`
	 * IP: `10.123.1.150`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_middle_of_cidr_is_whitelisted() {

		self::$ip = '10.123.1.150';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is outside of a whitelisted CIDR block, it should not be allowed.
	 *
	 * Whitelist: `10.0.0.0/8`
	 * IP: `11.0.0.1`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_outside_of_cidr_is_not_whitelisted() {

		self::$ip = '11.0.0.1';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

}
