<?php
/**
 * IP Ranges
 *
 * Test that we can match IPs against whitelists containing ranges.
 *
 * Example: `127.0.0.0-130.255.255.255` on whitelist, so `127.255.255.255` should match.
 *
 * @package   rcp-ip-restriction
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCP_IP_Restriction\Tests;

/**
 * Class Test_IP_Ranges
 *
 * @package RCP_IP_Restriction\Tests
 */
class Test_IP_Ranges extends UnitTestCase {

	/**
	 * Set up once before tests are run
	 */
	public static function setUpBeforeClass() {
		parent::setUpBeforeClass();

		// Update global whitelist.
		$settings = array(
			'whitelist' => array(
				'127.0.1.1-127.0.255.255', // A-B range
				'123.123.*.*-125.*.*.*', // A-B range with wildcards
				'190.*-190.30.*', // A-B range with wildcards and less than four octets
			)
		);

		update_option( 'rcp_ip_restriction_settings', $settings );
	}

	/**
	 * If an IP matches the start of a whitelisted range, it should be allowed.
	 *
	 * Whitelist: `127.0.1.1-127.0.255.255`
	 * IP: `127.0.1.1`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_start_of_range_is_whitelisted() {

		self::$ip = '127.0.1.1';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches the start of a whitelisted range, it should be allowed.
	 *
	 * Whitelist: `127.0.1.1-127.0.255.255`
	 * IP: `127.0.255.255`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_end_of_range_is_whitelisted() {

		self::$ip = '127.0.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches the middle of a whitelisted range, it should be allowed.
	 *
	 * Whitelist: `127.0.1.1-127.0.255.255`
	 * IP: `127.0.185.20`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_middle_of_range_is_whitelisted() {

		self::$ip = '127.0.185.20';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is outside of a whitelisted range, it should not be allowed.
	 *
	 * Whitelist: `127.0.1.1-127.0.255.255`
	 * IP: `127.0.0.0`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_outside_of_range_is_not_whitelisted() {

		self::$ip = '127.0.0.0';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is within a whitelisted range that contains wildcards, should be allowed
	 *
	 * Whitelist: `123.123.*.*-125.*.*.*`
	 * IP: `123.123.0.0`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_start_of_range_with_wildcards_is_whitelisted() {

		self::$ip = '123.123.0.0';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is within a whitelisted range that contains wildcards, should be allowed
	 *
	 * Whitelist: `123.123.*.*-125.*.*.*`
	 * IP: `125.255.255.255`
	 *
	 * @link https://github.com/restrictcontentpro/rcp-ip-restriction/issues/31
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_end_of_range_with_wildcards_is_whitelisted() {

		self::$ip = '125.255.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is within a whitelisted range that contains wildcards, should be allowed
	 *
	 * Whitelist: `123.123.*.*-125.*.*.*`
	 * IP: `124.185.12.3`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_middle_of_range_with_wildcards_is_whitelisted() {

		self::$ip = '124.185.12.3';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is outside a whitelisted range that contains wildcards, should not be allowed
	 *
	 * Whitelist: `123.123.*.*-125.*.*.*`
	 * IP: `123.1.12.3`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_outside_of_range_with_wildcards_is_not_whitelisted() {

		self::$ip = '123.1.12.3';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is within a whitelisted range that contains wildcards and doesn't include all four octets,
	 * it should be allowed.
	 *
	 * Whitelist: `190.*-190.30.*`
	 * IP: `190.0.0.0'
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_start_of_range_with_wildcards_and_partial_octets_is_whitelisted() {

		self::$ip = '190.0.0.0';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is within a whitelisted range that contains wildcards and doesn't include all four octets,
	 * it should be allowed.
	 *
	 * Whitelist: `190.*-190.30.*`
	 * IP: `190.0.0.0'
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_end_of_range_with_wildcards_and_partial_octets_is_whitelisted() {

		self::$ip = '190.30.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is within a whitelisted range that contains wildcards and doesn't include all four octets,
	 * it should be allowed.
	 *
	 * Whitelist: `190.*-190.30.*`
	 * IP: `190.25.192.4'
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_at_middle_of_range_with_wildcards_and_partial_octets_is_whitelisted() {

		self::$ip = '190.25.192.4';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP is outside a whitelisted range that contains wildcards and doesn't include all four octets,
	 * it should not be allowed.
	 *
	 * Whitelist: `190.*-190.30.*`
	 * IP: `190.50.192.4'
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_outside_of_range_with_wildcards_and_partial_octets_is_not_whitelisted() {

		self::$ip = '190.50.192.4';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

}
