<?php
/**
 * IP Wildcards
 *
 * Test that we can match IPs against whitelists containing wildcards.
 *
 * Example: `127.0.*.*` on whitelist so `127.0.0.1` should match.
 *
 * @package   rcp-ip-restriction
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCP_IP_Restriction\Tests;

/**
 * Class Test_IP_Wildcards
 *
 * @package RCP_IP_Restriction\Tests
 */
class Test_IP_Wildcards extends UnitTestCase {

	/**
	 * Set up once before tests are run
	 */
	public static function setUpBeforeClass() {
		parent::setUpBeforeClass();

		// Update global whitelist.
		$settings = array(
			'whitelist' => array(
				'120.157.227.*', // Wildcard in fourth octet
				'127.0.*.*', // Wildcard in third and fourth octet
				'130.0.*', // Wildcard in third octet only
				'170.*', // Wildcard in second octet only
			)
		);

		update_option( 'rcp_ip_restriction_settings', $settings );
	}

	/**
	 * If an IP matches a wildcard within a single octet whitelist, it should be allowed.
	 *
	 * Whitelist: `120.157.227.*`
	 * IP: `120.157.227.1`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_matching_single_octet_wildcard_is_whitelisted() {

		self::$ip = '120.157.227.1';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP doesn't fully match the wildcard then it should not be allowed.
	 *
	 * Whitelist: `120.157.227.*`
	 * IP: `120.157.30.1`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_not_matching_single_octet_wildcard_not_whitelisted() {

		self::$ip = '120.157.30.1';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches a wildcard within a single octet whitelist, it should be allowed.
	 *
	 * Whitelist: `127.0.*.*`
	 * IP: `127.0.0.1`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_matching_two_octet_wildcard_is_whitelisted() {

		self::$ip = '127.0.0.1';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches a wildcard within a single octet whitelist, it should be allowed.
	 *
	 * Whitelist: `127.0.*.*`
	 * IP: `127.0.255.255`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_matching_two_octet_wildcard_upper_range_is_whitelisted() {

		self::$ip = '127.0.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP doesn't fully match the wildcard then it should not be allowed.
	 *
	 * Whitelist: `127.0.*.*`
	 * IP: `120.1.0.1`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_not_matching_two_octet_wildcard_not_whitelisted() {

		self::$ip = '120.1.0.1';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches an entry that is a wildcard with only three octets provided, it should be allowed.
	 *
	 * Whitelist: `130.0.*`
	 * IP: `130.0.0.1`
	 *
	 * @link https://github.com/restrictcontentpro/rcp-ip-restriction/issues/13
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_matching_wildcard_with_three_octets_is_whitelisted() {

		// Test IP 1
		self::$ip = '130.0.0.1';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

		// Test IP 2
		self::$ip = '130.0.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP doesn't match an entry that is a wildcard with only three octets provided, it should not be allowed.
	 *
	 * Whitelist: `130.0.*`
	 * IP: `130.1.0.0`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_not_matching_wildcard_with_three_octets_not_whitelisted() {

		self::$ip = '130.1.0.0';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP matches an entry that is a wildcard with only two octets provided, it should be allowed.
	 *
	 * Whitelist: `170.*`
	 * IP: `170.187.255.3`
	 *
	 * @link https://github.com/restrictcontentpro/rcp-ip-restriction/issues/13
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_matching_wildcard_with_two_octets_is_whitelisted() {

		// Test IP 1
		self::$ip = '170.187.255.3';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

		// Test IP 2
		self::$ip = '170.0.0.0';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

		// Test IP 3
		self::$ip = '170.255.255.255';

		$this->assertTrue( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

	/**
	 * If an IP doesn't match an entry that is a wildcard with only two octets provided, it should not be allowed.
	 *
	 * Whitelist: `170.*`
	 * IP: `169.187.53.78`
	 *
	 * @covers ::rcp_ip_restriction_is_whitelisted()
	 */
	public function test_ip_not_matching_wildcard_with_two_octets_not_whitelisted() {

		self::$ip = '169.187.53.78';

		$this->assertFalse( rcp_ip_restriction_is_whitelisted( self::$post->ID ) );

	}

}
