<?php
/**
 * MailChimp API
 *
 * @package   rcp-mailchimp-pro
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 * @since     1.5
 */

namespace RCP\MailChimp_Pro;

use RCP\MailChimp_Pro\Exceptions\FailedRequest;
use RCP\MailChimp_Pro\Exceptions\MissingAPI;
use WP_Error;

/**
 * Class API
 *
 * @package RCP\MailChimp_Pro
 */
class API {

	/**
	 * API key
	 *
	 * @var string
	 */
	protected $api_key;

	/**
	 * API URL
	 *
	 * Includes the parsed data center.
	 *
	 * @var string
	 */
	protected $api_url;

	/**
	 * Response of the last API request
	 *
	 * @var array|WP_Error
	 */
	protected $response;

	/**
	 * HTTP response code from the last API request
	 *
	 * @var int
	 */
	protected $response_code;

	/**
	 * Current error
	 *
	 * If an error occurs, it will be stored here so we can retrieve it.
	 *
	 * @var WP_Error
	 */
	protected $error;

	/**
	 * API constructor.
	 */
	public function __construct() {

		$settings      = get_option( 'rcp_mailchimp_pro_settings' );
		$this->api_key = ! empty( $settings['api_key'] ) ? trim( $settings['api_key'] ) : '';
		$this->setup_api_url();

	}

	/**
	 * Checks whether or not an API key has been saved
	 *
	 * @since 1.5
	 * @return bool
	 */
	public function has_api_key() {
		if ( empty( $this->api_key ) ) {
			$this->add_error( 'missing_api_key', __( 'Missing API key', 'rcp-mailchimp-pro' ) );

			return false;
		}

		return true;
	}

	/**
	 * Checks that we have a valid API key
	 *
	 * @since 1.5
	 * @throws MissingAPI
	 */
	protected function check_api_key() {
		if ( ! $this->has_api_key() ) {
			throw new MissingAPI();
		}
	}

	/**
	 * Sets up the base API URL, including the data center
	 *
	 * @since 1.5
	 * @return void
	 */
	protected function setup_api_url() {

		if ( ! $this->has_api_key() ) {
			return;
		}

		$pieces        = explode( '-', $this->api_key );
		$region        = end( $pieces );
		$this->api_url = 'https://' . urlencode( $region ) . '.api.mailchimp.com/3.0/';

	}

	/**
	 * Retrieves the headers to use in API requests
	 *
	 * @since 1.5
	 * @return array
	 */
	protected function get_headers() {
		return array(
			'Authorization' => 'Basic ' . base64_encode( 'user:' . $this->api_key )
		);
	}

	/**
	 * Retrieves the saved error(s).
	 *
	 * @since 1.5
	 * @return WP_Error
	 */
	public function get_error() {
		return $this->error;
	}

	/**
	 * Determines whether or not we have any errors.
	 *
	 * @since 1.5
	 * @return bool
	 */
	public function has_error() {
		return $this->error instanceof WP_Error && $this->error->has_errors();
	}

	/**
	 * Adds a new error
	 *
	 * @param string $code
	 * @param string $message
	 *
	 * @since 1.5
	 * @return void
	 */
	public function add_error( $code, $message ) {
		if ( ! $this->error instanceof WP_Error ) {
			$this->error = new WP_Error();
		}

		$this->error->add( $code, $message );
	}

	/**
	 * Determines whether or not an API response was successful. If not, adds an error message.
	 *
	 * @param array|WP_Error $response Response from the API request.
	 *
	 * @since 1.5
	 * @return bool
	 */
	protected function is_successful( $response ) {

		if ( is_wp_error( $response ) ) {
			/**
			 * @var WP_Error $response
			 */
			$this->add_error( $response->get_error_code(), $response->get_error_message() );

			return false;
		}

		if ( 200 != wp_remote_retrieve_response_code( $response ) ) {
			// Try to get the error.
			$body    = json_decode( wp_remote_retrieve_body( $response ), true );
			$message = __( 'Invalid response', 'rcp-mailchimp-pro' );

			if ( is_array( $body ) && array_key_exists( 'detail', $body ) ) {
				$message = $body['detail'];
			}

			$this->add_error( 'invalid_response', $message );

			return false;
		}

		return true;

	}

	/**
	 * Makes an API request
	 *
	 * @param string $request_url_piece Endpoint to append to the base API URL.
	 * @param array  $request_args      Request arguments.
	 *
	 * @since 1.5
	 * @return object|mixed
	 * @throws FailedRequest
	 * @throws MissingAPI
	 */
	public function make_request( $request_url_piece, $request_args = array() ) {

		$this->check_api_key();

		$request_url  = $this->api_url . $request_url_piece;
		$request_args = wp_parse_args( $request_args, array(
			'method'  => 'GET',
			'headers' => $this->get_headers()
		) );

		$this->response      = wp_remote_request( $request_url, $request_args );
		$this->response_code = wp_remote_retrieve_response_code( $this->response );

		if ( ! $this->is_successful( $this->response ) ) {
			throw new FailedRequest( $this->get_error(), $this->response_code );
		}

		return json_decode( wp_remote_retrieve_body( $this->response ) );

	}

	/**
	 * Retrieves the response of the last API request.
	 *
	 * @since 1.5
	 * @return array|WP_Error
	 */
	public function get_response() {
		return $this->response;
	}

	/**
	 * Retrieves the HTTP response code from the last API request.
	 *
	 * @since 1.5
	 * @return int
	 */
	public function get_response_code() {
		return $this->response_code;
	}

	/**
	 * Retrieves the MailChimp lists on the account
	 *
	 * @param bool $cached Whether or not to fetch cached results where possible.
	 *
	 * @return array
	 * @throws FailedRequest
	 * @throws MissingAPI
	 */
	public function get_lists( $cached = true ) {

		$lists = get_transient( 'rcp_mailchimp_pro_lists' );

		// Return cached lists if it exists and we're allowing cached data.
		if ( $cached && ! empty( $lists ) ) {
			return $lists;
		}

		$response_body = $this->make_request( 'lists/?count=150' );

		$lists = array();

		if ( empty( $response_body->lists ) || ! is_array( $response_body->lists ) ) {
			return $lists;
		}

		foreach ( $response_body->lists as $list_info ) {
			$lists[ $list_info->id ] = sanitize_text_field( $list_info->name );
		}

		set_transient( 'rcp_mailchimp_pro_lists', $lists, DAY_IN_SECONDS );

		return $lists;

	}

	/**
	 * Retrieves the group categories associated with a list
	 *
	 * @param string $list_id ID of the MailChimp list to retrieve groups for.
	 * @param bool   $cached  Whether or not to fetch cached results where possible.
	 *
	 * @return object|false Response object on success.
	 * @throws MissingAPI
	 * @throws FailedRequest
	 */
	public function get_list_group_categories( $list_id, $cached = true ) {

		$groups = get_transient( 'rcp_mailchimp_pro_list_groups_' . sanitize_key( $list_id ) );

		// Return cached groups if it exists and we're allowing cached data.
		if ( $cached && ! empty( $groups ) ) {
			return $groups;
		}

		$request_endpoint = 'lists/' . urlencode( $list_id ) . '/interest-categories/?count=100';
		$response_body    = $this->make_request( $request_endpoint );

		if ( empty( $response_body ) || ! is_object( $response_body ) ) {
			return false;
		}

		set_transient( 'rcp_mailchimp_pro_list_groups_' . sanitize_key( $list_id ), $response_body, DAY_IN_SECONDS );

		return $response_body;

	}

	/**
	 * Retrieves the groups within a group category
	 *
	 * @param string $list_id           ID of the MailChimp list.
	 * @param string $group_category_id ID of the group category.
	 *
	 * @return array|false Array of groups (`id` => `name`) on success, false on failure.
	 * @throws MissingAPI
	 * @throws FailedRequest
	 */
	public function get_list_groups( $list_id, $group_category_id ) {

		$request_endpoint = 'lists/' . urlencode( $list_id ) . '/interest-categories/' . urlencode( $group_category_id ) . '/interests/?count=100';
		$response_body    = $this->make_request( $request_endpoint );

		$groups = array();

		if ( empty( $response_body->interests ) ) {
			return $groups;
		}

		foreach ( $response_body->interests as $group ) {
			$groups[ $group->id ] = $group->name;
		}

		return $groups;

	}

	/**
	 * Determines whether or not a user is subscribed to a given list
	 *
	 * @param string $email
	 * @param string $list_id
	 *
	 * @return bool
	 * @throws MissingAPI
	 */
	public function is_subscribed( $email, $list_id ) {

		try {
			$request_endpoint = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $email ) ) );
			$response_body    = $this->make_request( $request_endpoint );

			return ! empty( $response_body->status ) && 'subscribed' === $response_body->status;
		} catch ( MissingAPI $e ) {
			throw new MissingAPI( $e->getMessage(), $e->getCode() );
		} catch ( \Exception $e ) {
			return false;
		}

	}

	/**
	 * Creates the STATUS merge var in MailChimp if we haven't already
	 *
	 * @param string $list_id
	 *
	 * @throws FailedRequest
	 * @throws MissingAPI
	 */
	public function maybe_create_merge_var( $list_id ) {

		$set_vars = get_option( 'rcp_mailchimp_pro_set_merge_vars' );

		// We've already created the merge var - bail.
		if ( is_array( $set_vars ) && array_key_exists( $list_id, $set_vars ) ) {
			return;
		}

		/*
		 * First get all merge vars to see if ours already exists.
		 */
		$request_endpoint = 'lists/' . urlencode( $list_id ) . '/merge-fields/';
		$merge_vars       = $this->make_request( $request_endpoint );

		foreach ( $merge_vars->merge_fields as $merge_field ) {
			// Bail if the STATUS tag already exists.
			if ( 'STATUS' === $merge_field->tag ) {
				return;
			}
		}

		/*
		 * At this point we haven't found the STATUS merge var so we need to create it.
		 */
		$request_endpoint = 'lists/' . urlencode( $list_id ) . '/merge-fields/';
		$args             = array(
			'method' => 'POST',
			'body'   => json_encode( array(
				'tag'  => 'STATUS',
				'name' => __( 'RCP Status', 'rcp-mailchimp-pro' ),
				'type' => 'text'
			) )
		);

		$this->make_request( $request_endpoint, $args );

		// Success!
		$set_vars[ $list_id ] = true;

		update_option( 'rcp_mailchimp_pro_set_merge_vars', $set_vars );

	}

	/**
	 * Subscribes an email to a MailChimp list
	 *
	 * @param string $email   Email address.
	 * @param string $list_id ID of the list.
	 * @param array  $args    Body arguments.
	 *
	 * @return object MailChimp response object.
	 * @throws FailedRequest
	 * @throws MissingAPI
	 */
	public function subscribe( $email, $list_id, $args = array() ) {

		$this->check_api_key();

		$settings      = get_option( 'rcp_mailchimp_pro_settings' );
		$double_opt_in = empty( $settings['bypass_optin'] );

		$this->maybe_create_merge_var( $list_id );

		// First try to get the subscriber profile if we have one.
		try {
			$request_endpoint    = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $email ) ) );
			$existing_subscriber = $this->make_request( $request_endpoint );
		} catch ( \Exception $e ) {
			$existing_subscriber = false;
		}

		$body_args = wp_parse_args( $args, array(
			'email_address' => sanitize_email( $email ),
			'email_type'    => 'html',
			'status_if_new' => $double_opt_in ? 'pending' : 'subscribed',
			'status'        => ( ( ! empty( $existing_subscriber->status ) && 'subscribed' == $existing_subscriber->status ) || ! $double_opt_in ) ? 'subscribed' : 'pending',
		) );

		$request_endpoint = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $email ) ) ) . '?skip_merge_validation=true';

		return $this->make_request( $request_endpoint, array(
			'method' => 'PUT',
			'body'   => json_encode( $body_args )
		) );

	}

}