<?php
/**
 * MailChimp API Handler
 *
 * @package     RCP\MailChimpPro\API
 * @since       1.0.1
 */


// Exit if accessed directly
use RCP\MailChimp_Pro\Exceptions\FailedRequest;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * MailChimp API handler class
 *
 * @since       1.0.1
 */
class RCP_MailChimp_Pro_API {


	/**
	 * @var         array $settings MailChimp Pro settings
	 * @since       1.0.1
	 */
	public $settings;


	/**
	 * @var         array $lists Per-level list settings
	 * @since       1.0.1
	 * @deprecated  1.5 In favour of `RCP\MailChimp_Pro\get_membership_level_list()`
	 */
	public $lists;


	/**
	 * @var         array $groups Per-level group settings
	 * @since       1.2.3
	 * @deprecated  1.5 In favour of `RCP\MailChimp_Pro\get_membership_level_group()`
	 */
	public $groups;


	/**
	 * Get things started
	 *
	 * @access      public
	 * @since       1.0.1
	 * @return      void
	 */
	public function __construct() {
		$this->setup_api();

		// Maybe set a flag to sign up the user
		add_action( 'rcp_form_processing', array( $this, 'maybe_signup' ), 10, 8 );

		if ( function_exists( 'rcp_get_membership') ) {
			// Subscribe to list.
			add_action( 'rcp_transition_membership_status_active', array( $this, 'maybe_subscribe' ), 10, 2 );

			// Update STATUS merge var.
			add_action( 'rcp_transition_membership_status', array( $this, 'update_status_merge_var' ), 10, 3 );
		} else {
			add_action( 'rcp_set_status', array( $this, 'maybe_add_to_list' ), 10, 4 );
			add_action( 'rcp_set_status', array( $this, 'set_status' ), 10, 4 );
		}

		// E-Commerce 360 integration
		add_action( 'rcp_update_payment_status_complete', array( $this, 'insert_payment' ), 5 );

		// Update MailChimp when user changes email
		add_action( 'profile_update', array( $this, 'update_subscription' ), 10, 2 );
	}


	/**
	 * Setup the API object
	 *
	 * @access      private
	 * @since       1.0.1
	 * @return      void
	 */
	private function setup_api() {
		if( ! $this->settings ) {
			$this->settings = get_option( 'rcp_mailchimp_pro_settings' );
			$this->lists    = get_option( 'rcp_mailchimp_pro_subscription_lists' );
			$this->groups   = get_option( 'rcp_mailchimp_pro_subscription_groups' );
		}
	}


	/**
	 * Retrieve the list to signup for
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       int $level_id The ID of the subscription level to lookup
	 * @return      string $list The MailChimp list to signup for
	 */
	public function get_list( $level_id ) {
		$list = RCP\MailChimp_Pro\get_membership_level_list( $level_id );

		if ( empty( $list ) || 'inherit' === $list ) {
			$list = $this->settings['saved_list'];
		}

		return apply_filters( 'rcp_mailchimp_pro_get_list', $list, $level_id );
	}


	/**
	 * Retrieve the group to signup for
	 *
	 * @access      public
	 * @since       1.2.3
	 * @param        int|string $level_id The ID of the subscription level to lookup, or "default" for the default group in settings.
	 * @param string $mailchimp_list_id   ID of the MailChimp list. For backwards compatibility.
	 * @return      string $group The MailChimp group to signup for
	 */
	public function get_group( $level_id = 'default', $mailchimp_list_id = '' ) {
		$group = 0;

		if ( 'default' == $level_id ) {

			// Default group saved in settings.
			$group = array_key_exists( 'saved_group', $this->settings ) ? $this->settings['saved_group'] : 0;

		} else {

			// Get the group for an individual subscription level.
			$group = RCP\MailChimp_Pro\get_membership_level_group( $level_id );

			if ( empty( $group ) || 'inherit' === $group ) {
				$group = $this->settings['saved_group'];
			}

		}

		/*
		 * If the group ID includes a slash, it's the old method of saving and is no longer valid.
		 * We need to do another API request to try to get the interest ID from the group ID and
		 * interest name.
		 */
		if ( false !== strpos( $group, '/' ) ) {

			$group_pieces  = explode( '/', $group );
			$group_id      = $group_pieces[0];
			$interest_name = $group_pieces[1];

			$group = 0; // Set back to zero in case we can't find the interest ID.

			$api = new \RCP\MailChimp_Pro\API();

			try {
				$groups    = $api->get_list_groups( $mailchimp_list_id, $group_id );
				$group_key = array_search( $interest_name, $groups );

				if ( false !== $group_key ) {
					$group = $group_key;
				}

			} catch ( Exception $e ) {

			}

			// If we were able to get an interest ID, let's update the settings.
			if ( ! empty( $group ) ) {

				if ( 'default' == $level_id ) {

					// Update the group ID in the main settings.
					$this->settings['saved_group'] = sanitize_text_field( $group );
					update_option( 'rcp_mailchimp_pro_settings', $this->settings );

				} else {

					// Update the individual membership level.
					$levels_db = new RCP_Levels();
					$levels_db->update_meta( $level_id, 'mailchimp_group', sanitize_text_field( $group ) );

				}

			}
		}

		return apply_filters( 'rcp_mailchimp_pro_get_group', $group, $level_id );
	}

	/**
	 * Create the grouping args
	 *
	 * @since 1.2.3
	 * @param int|string $level_id          The ID of the subscription level to lookup, or "default" for the default group in settings.
	 * @param string     $mailchimp_list_id ID of the MailChimp list. For backwards compatibility.
	 *
	 * @return array|false
	 */
	public function get_grouping( $level_id = 'default', $mailchimp_list_id = '' ) {
		$groupings = false;

		if( ! $group = $this->get_group( $level_id, $mailchimp_list_id ) ) {
			return $groupings;
		}

		$groupings = array(
			$group => true
		);

		return $groupings;
	}


	/**
	 * Check if a given member is subscribed on MailChimp
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       string $email The email address to lookup
	 * @param       string $list The list to check
	 * @return      array|false $member_exists Array of subscriber data from MailChimp if subscribed, false otherwise.
	 */
	public function member_exists( $email, $list ) {

		$api = new RCP\MailChimp_Pro\API();

		try {

			$request_endpoint = 'lists/' . urlencode( $list ) . '/members/' . urlencode( md5( strtolower( $email ) ) );
			$api->make_request( $request_endpoint );

			return json_decode( wp_remote_retrieve_body( $api->get_response() ), true );

		} catch ( Exception $e ) {
			return false;
		}

	}


	/**
	 * Create our status merge var
	 *
	 * @deprecated 1.5
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       string $list The list ID to create for
	 * @return      void
	 */
	public function create_merge_var( $list ) {
		try {
			$api = new RCP\MailChimp_Pro\API();
			$api->maybe_create_merge_var( $list );
		} catch ( \Exception $e ) {

		}
	}


	/**
	 * Subscribe an email to MailChimp
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       string $email The email address to subscribe
	 * @param       int $user_id The ID of the user to subscribe
	 * @param       RCP_Membership|false $membership Associated membership object. Must be provided when working with
	 *                                               multiple memberships. If omitted, membership is automatically
	 *                                               determined.
	 * @return      bool True if added successfully, false otherwise
	 */
	public function subscribe( $email, $user_id, $membership = false ) {

		$member            = new RCP_Member( $user_id );
		$level_id          = ! empty( $membership ) ? $membership->get_object_id() : $member->get_subscription_id();
		$double_opt_in     = ( isset( $this->settings['bypass_optin'] ) ? false : true );
		$saved_list        = $this->settings['saved_list'];
		$list              = $this->get_list( $level_id );
		$level_groupings   = $this->get_grouping( $level_id, $list );
		$default_groupings = $this->get_grouping( 'default', $saved_list );
		$fname             = sanitize_text_field( $member->first_name );
		$lname             = sanitize_text_field( $member->last_name );
		$status            = ! empty( $membership ) ? $membership->get_status() : $member->get_status();
		$subscribed_lists  = get_user_meta( $user_id, 'rcp_mailchimp_pro_subscribed_lists', true );

		if ( ! is_array( $subscribed_lists ) ) {
			$subscribed_lists = array();
		}

		$api = new RCP\MailChimp_Pro\API();

		if ( ! $api->has_api_key() ) {
			return false;
		}

		try {
			$api->maybe_create_merge_var( $list );
		} catch ( FailedRequest $e ) {
			rcp_log( sprintf( 'MailChimp Pro: Failed to create merge var for list %s. Request failed. Mesasge: %s', $list, $e->getMessage() ), true );

			return false;
		} catch ( \RCP\MailChimp_Pro\Exceptions\MissingAPI $e ) {
			rcp_log( sprintf( 'MailChimp Pro: Failed to create merge var for list %s. Missing API key. Mesasge: %s', $list, $e->getMessage() ), true );

			return false;
		}

		// Set up which lists to add users to.
		$lists = array(
			$list => array(
				'id'    => $list,
				'group' => $level_groupings
			)
		);

		// Maybe add a second list.
		if ( ! empty( $this->settings['double_subscribe'] ) && $saved_list != $list ) {
			$lists[ $saved_list ] = array(
				'id'    => $saved_list,
				'group' => $default_groupings
			);
		}

		if ( ! empty( $membership ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Beginning list signup for membership #%d.', $membership->get_id() ) );
		}

		foreach ( $lists as $list_id => $list_info ) {
			$is_subscribed = $this->member_exists( $email, $list_id );

			if ( $is_subscribed && 'subscribed' == $is_subscribed['status'] ) {
				$has_all_interests = true;

				if ( ! empty( $list_info['group'] ) ) {
					foreach ( $list_info['group'] as $group_id => $group ) {
						if ( empty( $is_subscribed['interests'][ $group_id ] ) ) {
							$has_all_interests = false;
						}
					}
				}

				// If the member is already subscribed and has all the required interests, we can skip adding them.
				if ( $has_all_interests ) {
					rcp_log( sprintf( 'MailChimp Pro: Skipping adding user ID #%d, as they are already subscribed to list ID %s.', $user_id, $list_id ) );

					if ( ! in_array( $list_id, $subscribed_lists ) ) {
						$subscribed_lists[] = sanitize_text_field( $list_id );
					}

					continue;
				}
			}

			$request_url = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $email ) ) ) . '?skip_merge_validation=true';

			$request_body = array(
				'email_address' => sanitize_email( $email ),
				'email_type'    => 'html',
				'status_if_new' => $double_opt_in ? 'pending' : 'subscribed',
				'status'        => ( ( $is_subscribed && 'subscribed' == $is_subscribed['status'] ) || ! $double_opt_in ) ? 'subscribed' : 'pending',
				'merge_fields'  => array(
					'FNAME'  => $fname,
					'LNAME'  => $lname,
					'STATUS' => $status
				)
			);

			if ( ! empty( $list_info['group'] ) ) {
				$request_body['interests'] = $list_info['group'];
			}

			/**
			 * Filters the arguments for subscribing to a MailChimp list.
			 *
			 * @param array      $request_body Arguments for the API call.
			 * @param RCP_Member $member       Member object.
			 * @param string     $list_id      ID of the list the user is being subscribed to.
			 *
			 * @since 1.4.3
			 */
			$request_body = apply_filters( 'rcp_mailchimp_pro_subscribe_args', $request_body, $member, $list_id );

			$args = array(
				'method' => 'PUT',
				'body'   => json_encode( $request_body )
			);

			try {
				$api->make_request( $request_url, $args );
			} catch ( FailedRequest $e ) {
				rcp_log( sprintf( 'Failed to subscribe user to MailChimp. Response code: %d. Response: %s', $e->get_http_code(), $e->getMessage() ) );

				/**
				 * Triggers when subscribing a user to a list fails.
				 *
				 * @param string         $list_id      ID of the list the user was being subscribed to.
				 * @param int            $user_id      ID of the user we were trying to subscribe.
				 * @param WP_Error|array $response     API request response.
				 * @param array          $request_body Body we sent in the API request.
				 *
				 * @since 1.4.6
				 */
				do_action( 'rcp_mailchimp_pro_subscribe_failed', $list_id, $user_id, $api->get_response(), $request_body );

				return false;
			} catch ( \Exception $e ) {
				return false;
			}

			if ( ! in_array( $list_id, $subscribed_lists ) ) {
				$subscribed_lists[] = sanitize_text_field( $list_id );
			}

			rcp_log( sprintf( 'MailChimp Pro: Successfully added user #%d to list %s.', $user_id, $list_id ) );

			/**
			 * Triggers when subscribing a user to a list is successful.
			 *
			 * @param string $list_id ID of the list the user was subscribed to.
			 * @param int    $user_id ID of the user who was subscribed.
			 *
			 * @since 1.4.6
			 */
			do_action( 'rcp_mailchimp_pro_subscribe_success', $list_id, $user_id );
		}

		// Update lists user is subscribed to.
		update_user_meta( $user_id, 'rcp_mailchimp_pro_subscribed_lists', $subscribed_lists );

		return true;

	}


	/**
	 * Checks whether a user should be signed up for the MailChimp list, and if so, sets a flag.
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       array                $posted              The fields posted by the submission form
	 * @param       int                  $user_id             The ID of this user
	 * @param       float                $price               Price of the membership.
	 * @param       RCP_Customer|false   $customer            Customer object.
	 * @param       int                  $membership_id       ID of the membership for this registration.
	 * @param       RCP_Membership|false $previous_membership Previous membership, if this is an upgrade/downgrade.
	 * @param       string               $registration_type   Type of registration (new, upgrade, downgrade, renewal).
	 * @return      void
	 */
	public function maybe_signup( $posted, $user_id, $price = 0.00, $payment_id = 0, $customer = false, $membership_id = 0, $previous_membership = false, $registration_type = 'new' ) {
		if ( function_exists( 'rcp_add_membership_meta' ) && ! empty( $membership_id ) ) {
			if ( isset( $posted['rcp_mailchimp_pro_signup'] ) ) {
				rcp_add_membership_meta( $membership_id, 'mailchimp_pro_pending_signup', true );
			} else {
				rcp_delete_membership_meta( $membership_id, 'mailchimp_pro_pending_signup' );
			}
		} else {
			if ( isset( $posted['rcp_mailchimp_pro_signup'] ) ) {
				// Set a flag so we know to add them to the list after account activation.
				update_user_meta( $user_id, 'rcp_pending_mailchimp_signup', true );
			} else {
				delete_user_meta( $user_id, 'rcp_pending_mailchimp_signup' );
			}
		}
	}

	/**
	 * When a membership is activated, add the user to the MailChimp list if the flag has been set.
	 *
	 * @param string $old_status    Previous status before this change.
	 * @param int    $membership_id ID of the membership.
	 *
	 * @access public
	 * @since 1.4.5
	 * @return void
	 */
	public function maybe_subscribe( $old_status, $membership_id ) {

		if ( ! rcp_get_membership_meta( $membership_id, 'mailchimp_pro_pending_signup', true ) ) {
			return;
		}

		$membership = rcp_get_membership( $membership_id );

		if ( empty( $membership ) ) {
			return;
		}

		$user = get_userdata( $membership->get_customer()->get_user_id() );

		if ( empty( $user ) ) {
			return;
		}

		$subscribed = $this->subscribe( $user->user_email, $user->ID, $membership );

		if ( $subscribed ) {
			update_user_meta( $user->ID, 'rcp_subscribed_to_mailchimp', 'yes' );
			delete_user_meta( $user->ID, 'rcp_pending_mailchimp_signup' );
			rcp_delete_membership_meta( $membership_id, 'mailchimp_pro_pending_signup' );
		}

	}

	/**
	 * Add member to the MailChimp list when their account is activated.
	 *
	 * @deprecated 1.4.5 In favour of `maybe_subscribe()`
	 * @see RCP_MailChimp_Pro_API::maybe_subscribe()
	 *
	 * @param string     $status     New status.
	 * @param int        $user_id    ID of the user.
	 * @param string     $old_status Previous status.
	 * @param RCP_Member $member     Member object.
	 *
	 * @since  1.3.2
	 * @return void
	 */
	public function maybe_add_to_list( $status, $user_id, $old_status, $member ) {

		if ( ! in_array( $status, array( 'active', 'free' ) ) ) {
			return;
		}

		if ( ! get_user_meta( $user_id, 'rcp_pending_mailchimp_signup', true ) ) {
			return;
		}

		$subscribed = $this->subscribe( $member->user_email, $user_id );

		if ( $subscribed ) {
			update_user_meta( $user_id, 'rcp_subscribed_to_mailchimp', 'yes' );
			delete_user_meta( $user_id, 'rcp_pending_mailchimp_signup' );
		}

	}


	/**
	 * Update subscription when user changes their email
	 *
	 * @access      public
	 * @since       1.0.2
	 * @param       int $user_id The ID of the user
	 * @param       object $old_user_data The old data for the user
	 * @return      void
	 */
	public function update_subscription( $user_id, $old_user_data ) {

		$api = new RCP\MailChimp_Pro\API();

		// No API key - bail.
		if ( ! $api->has_api_key() ) {
			return;
		}

		$user_data = get_userdata( $user_id );

		$new_email = $user_data->user_email;
		$old_email = $old_user_data->user_email;

		// Email hasn't changed - bail.
		if ( $new_email == $old_email ) {
			return;
		}

		$lists = rcp_mailchimp_pro_get_lists();

		if ( ! is_array( $lists ) ) {
			return;
		}

		foreach( $lists as $list_id => $list_name ) {
			$subscribed = $this->member_exists( $old_email, $list_id );

			if ( ! $subscribed ) {
				continue;
			}

			$request_url = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $old_email ) ) ) . '?skip_merge_validation=true';

			$request_body = array(
				'email_address' => $new_email
			);

			/**
			 * Filters the arguments for updating the MailChimp subscription.
			 *
			 * @param array  $request_body  Arguments for the API call.
			 * @param int    $user_id       User ID number.
			 * @param object $old_user_data The old data for the user
			 * @param string $list_id       ID of the list the subscription is being updated on.
			 *
			 * @since 1.4.4
			 */
			$request_body = apply_filters( 'rcp_mailchimp_pro_update_subscription_args', $request_body, $user_id, $old_user_data, $list_id );

			$args = array(
				'method'  => 'PATCH',
				'body'    => json_encode( $request_body )
			);

			try {
				$api->make_request( $request_url, $args );
			} catch ( \Exception $e ) {
				rcp_log( sprintf( 'MailChimp Pro: Failed to update subscription record for user #%d. Message: %s', $user_id, $e->getMessage() ), true );
			}
		}

	}


	/**
	 * E-Commerce 360 integration
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       int $payment_id The payment ID
	 * @return      void
	 */
	public function insert_payment( $payment_id ) {

		$api = new RCP\MailChimp_Pro\API();

		if ( ! $api->has_api_key() ) {
			return;
		}

		/**
		 * @var RCP_Payments $rcp_payments_db
		 */
		global $rcp_payments_db;

		$payment = $rcp_payments_db->get_payment( $payment_id );
		
		$user         = get_userdata( $payment->user_id );
		$subscription = rcp_get_subscription_details( $payment->object_id );
		$list_id      = $this->get_list( $subscription->id );

		if ( empty( $user ) ) {
			return;
		}

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Beginning to process e-commerce order for payment ID #%d.', $payment_id ) );
		}

		// Get or create store.
		$store_id = $this->create_mailchimp_store( $list_id );

		if ( empty( $store_id ) ) {
			return;
		}

		// Maybe create product for the subscription level.
		$this->create_mailchimp_product( $subscription, $store_id );

		$request_url = 'ecommerce/stores/' . urlencode( $store_id ) . '/orders/';

		$customer_args = array(
			'id'            => (string) $user->ID,
			'email_address' => $user->user_email,
			'opt_in_status' => false,
			'first_name'    => $user->first_name,
			'last_name'     => $user->last_name,
			// 'orders_count'  => '', // @todo
			// 'total_spent'   => ''
		);

		try {
			/*
			 * If a customer with this ID already exists, then overwrite our customer args to only include the `id`.
			 *
			 * @link https://github.com/restrictcontentpro/rcp-mailchimp-pro/issues/76
			 */
			$customer = $api->make_request( 'ecommerce/stores/' . urlencode( $store_id ) . '/customers/' . urlencode( $customer_args['id'] ) );

			if ( ! empty( $customer->id ) ) {
				$customer_args = array(
					'id' => (string) $customer->id
				);
			}
		} catch ( \Exception $e ) {

		}

		$order_args = apply_filters( 'rcp_mailchimp_pro_ecommerce_data', array(
			'id'                   => (string) $payment_id,
			'customer'             => $customer_args,
			'financial_status'     => 'pending',
			'fulfillment_status'   => 'pending',
			'currency_code'        => rcp_get_currency(),
			'order_total'          => $payment->amount,
			'discount_total'       => $payment->discount_amount,
			'processed_at_foreign' => $payment->date,
			'lines'                => array(
				array(
					'id'                 => $payment_id . '_' . $payment->subscription_key, // @todo should this be something else?
					'product_id'         => (string) $subscription->id,
					'product_variant_id' => (string) $subscription->id,
					'quantity'           => 1,
					'price'              => $payment->amount,
					'discount'           => $payment->discount_amount
				)
			)
		), $payment_id, (array) $payment );

		$args = array(
			'method' => 'POST',
			'body'   => json_encode( $order_args )
		);

		try {
			$api->make_request( $request_url, $args );
		} catch ( \Exception $e ) {
			rcp_log( sprintf( 'MailChimp Pro: Error creating e-commerce order. Response code: %d. Response: %s.', $e->getCode(), $e->getMessage() ) );
		}

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Successfully created e-commerce order #%d.', $payment_id ) );
		}
	}

	/**
	 * Retrieve or create the e-commerce store for a MailChimp list.
	 *
	 * @param string $list_id ID of the MailChimp list to create the store for.
	 *
	 * @access public
	 * @since 1.3.3
	 * @return string|false ID of the store on success, false on failure.
	 */
	public function create_mailchimp_store( $list_id ) {

		$api = new RCP\MailChimp_Pro\API();

		// The store ID is a combination of the home URL hash and the list ID.
		$store_id = md5( home_url() ) . '-' . $list_id;

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Getting or creating MailChimp store ID %s.', $store_id ) );
		}

		// Check if the store ID already exists.
		$request_url = 'ecommerce/stores/' . urlencode( $store_id );

		try {
			$api->make_request( $request_url );

			// Yep, this store already exists!
			if ( 200 == $api->get_response_code() ) {
				if ( function_exists( 'rcp_log' ) ) {
					rcp_log( sprintf( 'MailChimp Pro: Store ID %s already exists.', $store_id ) );
				}

				return $store_id;
			}
		} catch ( \Exception $e ) {

		}

		// If the store doesn't exist, we need to create it.
		$request_url = 'ecommerce/stores/';

		$body = array(
			'id'             => $store_id,
			'list_id'        => $list_id,
			'name'           => get_bloginfo( 'name' ),
			'platform'       => __( 'Restrict Content Pro', 'rcp' ),
			'domain'         => home_url(),
			'is_syncing'     => false, // @todo enable syncing?
			'email_address'  => get_site_option( 'admin_email' ),
			'currency_code'  => rcp_get_currency(),
			'money_format'   => rcp_currency_filter( '' ),
			'primary_locale' => substr( get_locale(), 0, 2 ),
			'timezone'       => get_option( 'timezone_string' ) // @todo this may not be set?
		);

		$args = array(
			'method' => 'POST',
			'body' => json_encode( $body )
		);

		try {
			$api->make_request( $request_url, $args );
		} catch ( \Exception $e ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'MailChimp Pro: Failed to create new store ID %s. Response code: %d. Response: %s', $store_id, $e->getCode(), $e->getMessage() ) );
			}

			return false;
		}

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Successfully created new store ID %s.', $store_id ) );
		}

		return $store_id;

	}

	/**
	 * Create product in MailChimp e-commerce.
	 *
	 * @param object $subscription_level Subscription level object.
	 * @param string $store_id           ID of the store in MailChimp.
	 *
	 * @access public
	 * @since 1.3.3
	 * @return void
	 */
	public function create_mailchimp_product( $subscription_level, $store_id ) {

		$api = new RCP\MailChimp_Pro\API();

		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Checking for MailChimp product ID %d.', $subscription_level->id ) );
		}

		// Check if the store ID already exists.
		$request_url = 'ecommerce/stores/' . urlencode( $store_id ) . '/products/' . urlencode( $subscription_level->id );

		try {
			$api->make_request( $request_url );

			// Yep, this product already exists.
			if ( 200 == $api->get_response_code() ) {
				if ( function_exists( 'rcp_log' ) ) {
					rcp_log( sprintf( 'MailChimp Pro: Product ID %s already exists.', $subscription_level->id ) );
				}

				return;
			}
		} catch ( \Exception $e ) {
			// Proceed.
		}

		// If the store doesn't exist, we need to create it.
		$request_url = 'ecommerce/stores/' . urlencode( $store_id ) . '/products/';

		$product_args = array(
			'id'          => (string) $subscription_level->id,
			'title'       => $subscription_level->name,
			'description' => stripslashes( $subscription_level->description ),
			'vendor'      => get_bloginfo( 'name' ),
			'variants'    => array(
				array(
					'id'    => (string) $subscription_level->id,
					'title' => $subscription_level->name,
					'price' => $subscription_level->price
				)
			)
		);

		$args = array(
			'method' => 'POST',
			'body'   => json_encode( $product_args )
		);

		try {
			$api->make_request( $request_url, $args );
		} catch ( \Exception $e ) {
			if ( function_exists( 'rcp_log' ) ) {
				rcp_log( sprintf( 'MailChimp Pro: Failed to create new product ID %d. Response code: %d. Response: %s', $subscription_level->id, $e->getCode(), $e->getMessage() ) );
			}

			return;
		}

		// Successfully created.
		if ( function_exists( 'rcp_log' ) ) {
			rcp_log( sprintf( 'MailChimp Pro: Successfully created new product ID %s.', $subscription_level->id ) );
		}

	}

	/**
	 * When a membership's status changes, update the STATUS merge var for the MailChimp subscription(s).
	 *
	 * @param string $old_status    Previous status, before this update.
	 * @param string $new_status    New membership status.
	 * @param int    $membership_id ID of the membership.
	 *
	 * @since 1.4.5
	 * @return void
	 */
	public function update_status_merge_var( $old_status, $new_status, $membership_id ) {

		$api = new RCP\MailChimp_Pro\API();

		$membership = rcp_get_membership( $membership_id );

		if ( empty( $membership ) ) {
			return;
		}

		// If this membership is disabled, we're not going to track it.
		if ( $membership->is_disabled() ) {
			return;
		}

		$user_id = $membership->get_customer()->get_user_id();
		$user    = get_userdata( $user_id );

		if ( empty( $user ) ) {
			return;
		}

		$email    = $user->user_email;
		$level_id = $membership->get_object_id();
		$list     = $this->get_list( $level_id );
		$api_key  = trim( $this->settings['api_key'] );

		if ( empty( $level_id ) || empty( $api_key ) ) {
			return;
		}

		$subscribed_lists = get_user_meta( $user->ID, 'rcp_mailchimp_pro_subscribed_lists', true );

		if ( empty( $subscribed_lists ) || ! is_array( $subscribed_lists ) ) {
			$subscribed_lists = array( $list );
		}

		foreach ( $subscribed_lists as $list_id ) {
			try {
				// Maybe create our custom merge var
				$api->maybe_create_merge_var( $list_id );

				$request_url = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $email ) ) ) . '?skip_merge_validation=true';

				$request_body = array(
					'email_address' => sanitize_email( $email ),
					'email_type'    => 'html',
					'merge_fields'  => array(
						'FNAME'  => $user->first_name,
						'LNAME'  => $user->last_name,
						'STATUS' => $membership->get_status()
					)
				);

				/**
				 * Filters the arguments for updating the MailChimp subscription with the new membership status.
				 *
				 * @param array  $request_body Arguments for the API call.
				 * @param int    $user_id      User ID number.
				 * @param string $list_id      ID of the list the subscription is being updated on.
				 *
				 * @since 1.4.4
				 */
				$request_body = apply_filters( 'rcp_mailchimp_pro_change_membership_status_args', $request_body, $user->ID, $list_id );

				$args = array(
					'method' => 'PATCH',
					'body'   => json_encode( $request_body )
				);

				rcp_log( sprintf( 'Updating MailChimp STATUS merge var to %s on list ID %s for user ID #%d.', $membership->get_status(), $list_id, $user->ID ) );

				$api->make_request( $request_url, $args );

				rcp_log( sprintf( 'Successfully updated STATUS merge var on list ID %s for user ID #%d.', $list_id, $user->ID ) );
			} catch ( \Exception $e ) {
				rcp_log( sprintf( 'MailChimp Error: Failed to update user on status change for list ID %s: %s', $level_id, $e->getMessage() ) );
			}
		}

	}


	/**
	 * Set the status for a user with MailChimp
	 *
	 * @deprecated 1.4.5 In favour of `update_status_merge_var()`
	 * @see RCP_MailChimp_Pro_API::update_status_merge_var()
	 *
	 * @access      public
	 * @since       1.0.1
	 * @param       string     $status     The status to set
	 * @param       int        $id         The user ID
	 * @param       string     $old_status Previous status.
	 * @param       RCP_Member $member     Member object.
	 * @return      void
	 */
	public function set_status( $status, $id, $old_status, $member ) {

		$api = new RCP\MailChimp_Pro\API();

		$email     = $member->user_email;
		$level_id  = rcp_get_subscription_id( $id );

		// Bail if no level ID.
		if ( empty( $level_id ) ) {
			return;
		}

		$list = $this->get_list( $level_id );

		$subscribed_lists = get_user_meta( $member->ID, 'rcp_mailchimp_pro_subscribed_lists', true );

		if ( empty( $subscribed_lists ) || ! is_array( $subscribed_lists ) ) {
			$subscribed_lists = array( $list );
		}

		foreach ( $subscribed_lists as $list_id ) {
			try {
				// Maybe create our custom merge var
				$api->maybe_create_merge_var( $list_id );

				$request_url = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $email ) ) ) . '?skip_merge_validation=true';

				$request_body = array(
					'email_address' => sanitize_email( $email ),
					'email_type'    => 'html',
					'merge_fields'  => array(
						'FNAME'  => $member->first_name,
						'LNAME'  => $member->last_name,
						'STATUS' => $status
					)
				);

				/**
				 * Filters the arguments for updating the MailChimp subscription with the new membership status.
				 *
				 * @param array  $request_body Arguments for the API call.
				 * @param int    $id           User ID number.
				 * @param string $list_id      ID of the list the subscription is being updated on.
				 *
				 * @since 1.4.4
				 */
				$request_body = apply_filters( 'rcp_mailchimp_pro_change_membership_status_args', $request_body, $id, $list_id );

				$args = array(
					'method' => 'PATCH',
					'body'   => json_encode( $request_body )
				);

				rcp_log( sprintf( 'Updating MailChimp STATUS merge var to %s on list ID %s for user ID #%d.', $status, $list_id, $member->ID ) );

				$api->make_request( $request_url, $args );

				rcp_log( sprintf( 'Successfully updated STATUS merge var on list ID %s for user ID #%d.', $list_id, $member->ID ) );
			} catch ( \Exception $e ) {
				rcp_log( sprintf( 'MailChimp Error: Failed to update user on status change for list ID %s: %s', $level_id, $e->getMessage() ) );
			}
		}
	}

}
