<?php
/**
 * Helper functions
 *
 * @package     RCP\MailChimpPro\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Get MailChimp subscription lists
 *
 * @param bool $cache Whether or not to get cached results.
 *
 * @since       1.0.0
 * @return      array|false $lists array The list of available lists or false on failure.
 */
function rcp_mailchimp_pro_get_lists( $cache = true ) {

	try {
		$api = new RCP\MailChimp_Pro\API();

		return $api->get_lists( $cache );
	} catch ( \Exception $e ) {
		return false;
	}

}


/**
 * Check if we should show the subscription field
 *
 * @since       1.0.0
 * @param       int $level A specific level to check
 * @return      bool $show True to show, false otherwise
 */
function rcp_mailchimp_pro_show_checkbox() {
	$settings = get_option( 'rcp_mailchimp_pro_settings' );

	if ( empty( $settings['api_key'] ) ) {
		return false;
	}

	// Show if we have at least one list saved somewhere.
	if ( ! empty( $settings['saved_list'] ) ) {
		return true;
	}

	global $wpdb;
	if ( $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->levelmeta} WHERE meta_key = 'mailchimp_list'" ) > 1 ) {
		return true;
	}
	
	$deprecated_lists = get_option( 'rcp_mailchimp_pro_subscription_lists' );
	if ( ! empty( $deprecated_lists ) && is_array( $deprecated_lists ) ) {
		return true;
	}

	return false;
}


/**
 * Get interest groups for MailChimp lists
 *
 * @since       1.2.3
 * @param       string $list  ID of the list to get the groups for.
 * @param       bool   $cache Whether or not to fetch cached results.
 * @return      object|false Response object from MailChimp or false on failure.
 */
function rcp_mailchimp_pro_get_groups( $list, $cache = true ) {

	try {
		$api = new RCP\MailChimp_Pro\API();

		return $api->get_list_group_categories( $list, $cache );
	} catch ( \Exception $e ) {
		return false;
	}
}

/**
 * Get interests in a given interest category
 *
 * @param string $list_id              ID of the list to get interests from.
 * @param string $interest_category_id ID of the interest category to get interests from.
 *
 * @since 1.3.3
 * @return array|false Array of interests (`id` => `name`) or false on failure.
 */
function rcp_mailchimp_pro_get_interests( $list_id, $interest_category_id ) {

	try {
		$api = new RCP\MailChimp_Pro\API();

		return $api->get_list_groups( $list_id, $interest_category_id );
	} catch ( \Exception $e ) {
		return false;
	}

}

/**
 * Adds a note to the customer when subscribing them to a MailChimp list fails.
 *
 * @param string         $list_id      ID of the list the user was being subscribed to.
 * @param int            $user_id      ID of the user we were trying to subscribe.
 * @param WP_Error|array $response     API request response.
 * @param array          $request_body Body we sent in the API request.
 *
 * @since 1.4.6
 * @return void
 */
function rcp_mailchimp_pro_add_customer_note_on_failure( $list_id, $user_id, $response, $request_body ) {

	if ( ! function_exists( 'rcp_get_customer_by_user_id' ) ) {
		return;
	}

	$customer = rcp_get_customer_by_user_id( $user_id );

	if ( ! $customer instanceof RCP_Customer ) {
		return;
	}

	$customer->add_note( sprintf( __( 'MailChimp Pro: Failed to subscribe customer to list %s. Response code: %d; Response: %s', 'rcp-mailchimp-pro' ), $list_id, wp_remote_retrieve_response_code( $response ), wp_remote_retrieve_body( $response ) ) );

}

add_action( 'rcp_mailchimp_pro_subscribe_failed', 'rcp_mailchimp_pro_add_customer_note_on_failure', 10, 4 );

/**
 * Adds a note to the customer when they are successfully subscribed to a MailChimp list.
 *
 * @param string $list_id ID of the list the user was subscribed to.
 * @param int    $user_id ID of the user who was subscribed.
 *
 * @since 1.4.6
 * @return void
 */
function rcp_mailchimp_pro_add_customer_note_on_success( $list_id, $user_id ) {

	if ( ! function_exists( 'rcp_get_customer_by_user_id' ) ) {
		return;
	}

	$customer = rcp_get_customer_by_user_id( $user_id );

	if ( ! $customer instanceof RCP_Customer ) {
		return;
	}

	$customer->add_note( sprintf( __( 'MailChimp Pro: Successfully subscribed customer to list %s.', 'rcp-mailchimp-pro' ), $list_id ) );

}

add_action( 'rcp_mailchimp_pro_subscribe_success', 'rcp_mailchimp_pro_add_customer_note_on_success', 10, 2 );