<?php
/**
 * Membership Level Functions
 *
 * @package   rcp-mailchimp-pro
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 * @since     1.5
 */

namespace RCP\MailChimp_Pro;

/**
 * Retrieves the ID of the MailChimp list assigned to a membership level.
 *
 * @param int $level_id ID of the membership level to check.
 *
 * @since 1.5
 * @return string|false MailChimp list ID on success, false if not found.
 */
function get_membership_level_list( $level_id ) {

	/*
	 * First check membership level meta.
	 */
	if ( function_exists( 'rcp_get_membership_level_meta' ) ) {
		$list_id = rcp_get_membership_level_meta( $level_id, 'mailchimp_list', true );
	} else {
		$levels_db = new \RCP_Levels();
		$list_id   = $levels_db->get_meta( $level_id, 'mailchimp_list', true );
	}

	// If we have a value, bail.
	if ( ! empty( $list_id ) ) {
		return $list_id;
	}

	/*
	 * Otherwise, let's check our old `rcp_mailchimp_pro_subscription_lists` option.
	 */
	$all_lists = get_option( 'rcp_mailchimp_pro_subscription_lists', array() );

	// If we don't have a value, just bail now.
	if ( ! is_array( $all_lists ) || empty( $all_lists[ $level_id ] ) ) {
		return false;
	}

	$list_id = $all_lists[ $level_id ];

	// Otherwise, we need to copy this value to membership level meta for next time.
	if ( function_exists( 'rcp_update_membership_level_meta' ) ) {
		rcp_update_membership_level_meta( $level_id, 'mailchimp_list', sanitize_text_field( $list_id ) );
	} else {
		$levels_db = new \RCP_Levels();
		$levels_db->update_meta( $level_id, 'mailchimp_list', sanitize_text_field( $list_id ) );
	}

	if ( isset( $all_lists[ $level_id ] ) ) {
		unset( $all_lists[ $level_id ] );
	}

	if ( empty( $all_lists ) ) {
		delete_option( 'rcp_mailchimp_pro_subscription_lists' );
	} else {
		update_option( 'rcp_mailchimp_pro_subscription_lists', array_map( 'sanitize_text_field', $all_lists ) );
	}

	return $list_id;

}

/**
 * Retrieves the ID of the MailChimp group assigned to a membership level.
 *
 * @param int $level_id ID of the membership level to check.
 *
 * @since 1.5
 * @return string|false MailChimp group ID on success, false if not found.
 */
function get_membership_level_group( $level_id ) {

	/*
	 * First check membership level meta.
	 */
	if ( function_exists( 'rcp_get_membership_level_meta' ) ) {
		$group_id = rcp_get_membership_level_meta( $level_id, 'mailchimp_group', true );
	} else {
		$levels_db = new \RCP_Levels();
		$group_id   = $levels_db->get_meta( $level_id, 'mailchimp_group', true );
	}

	// If we have a value, bail.
	if ( ! empty( $group_id ) ) {
		return $group_id;
	}

	/*
	 * Otherwise, let's check our old `rcp_mailchimp_pro_subscription_groups` option.
	 */
	$all_groups = get_option( 'rcp_mailchimp_pro_subscription_groups', array() );

	// If we don't have a value, just bail now.
	if ( ! is_array( $all_groups ) || empty( $all_groups[ $level_id ] ) ) {
		return false;
	}

	$group_id = $all_groups[ $level_id ];

	// Otherwise, we need to copy this value to membership level meta for next time.
	if ( function_exists( 'rcp_update_membership_level_meta' ) ) {
		rcp_update_membership_level_meta( $level_id, 'mailchimp_group', sanitize_text_field( $group_id ) );
	} else {
		$levels_db = new \RCP_Levels();
		$levels_db->update_meta( $level_id, 'mailchimp_group', sanitize_text_field( $group_id ) );
	}

	if ( isset( $all_groups[ $level_id ] ) ) {
		unset( $all_groups[ $level_id ] );
	}

	if ( empty( $all_groups ) ) {
		delete_option( 'rcp_mailchimp_pro_subscription_groups' );
	} else {
		update_option( 'rcp_mailchimp_pro_subscription_groups', array_map( 'sanitize_text_field', $all_groups ) );
	}

	return $group_id;

}