<?php
/**
 * Profile Editor Actions
 *
 * @package   rcp-mailchimp-pro
 * @copyright Copyright (c) 2020, Sandhills Development, LLC
 * @license   GPL2+
 */

namespace RCP\MailChimp_Pro;


use RCP\MailChimp_Pro\Exceptions\FailedRequest;
use RCP_Membership;

/**
 * If enabled, adds opt in/out checkboxes for each relevant email list to the Edit Profile form.
 *
 * @param int $user_id ID of the user editing their profile.
 *
 * @since 1.5
 */
function maybe_show_opt_in_out_checkboxes( $user_id ) {

	$settings = get_option( 'rcp_mailchimp_pro_settings' );

	if ( empty( $settings['allow_profile_opt_in'] ) || ! class_exists( 'RCP_Customer' ) ) {
		return;
	}

	$user     = get_userdata( $user_id );
	$customer = rcp_get_customer_by_user_id( $user_id );

	if ( ! $user instanceof \WP_User || ! $customer instanceof \RCP_Customer ) {
		return;
	}

	// Build an array of MailChimp lists to display.
	$list_ids = array();

	foreach ( $customer->get_memberships() as $membership ) {
		$list_id = get_membership_level_list( $membership->get_object_id() );

		if ( empty( $list_id ) || 'inherit' === $list_id ) {
			$list_id = $settings['saved_list'];
		}

		if ( ! empty( $list_id ) ) {
			$list_ids[ $membership->get_id() ] = $list_id;
		}

		if ( ! empty( $settings['double_subscribe'] ) && ! empty( $settings['saved_list'] ) && ! in_array( $settings['saved_list'], $list_ids ) ) {
			$list_ids[ $membership->get_id() ] = $settings['saved_list'];
		}
	}

	// Unique lists, please!
	$list_ids = array_unique( array_map( 'trim', $list_ids ) );

	$api = new API();

	$list_settings = array_fill_keys( $list_ids, array( 'name' => '', 'status' => '', 'membership_id' => null ) );

	try {
		foreach ( $list_ids as $membership_id => $list_id ) {

			$list_settings[ $list_id ]['membership_id'] = $membership_id;

			// First get list information. We need the name!
			$list = $api->make_request( 'lists/' . urlencode( $list_id ) );

			if ( ! empty( $list->name ) ) {
				$list_settings[ $list_id ]['name'] = $list->name;
			} else {
				// Bail if we can't get a list name.
				continue;
			}

			// Now we check the user's status on this list.
			try {
				$subscriber = $api->make_request( 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $user->user_email ) ) ) );

				$list_settings[ $list_id ]['status'] = ! empty( $subscriber->status ) ? $subscriber->status : '';
			} catch ( FailedRequest $e ) {
				// Not subscribed.
			}

		}
	} catch ( \Exception $e ) {

	}

	// Bail if we have no lists with names.
	if ( ! count( wp_list_pluck( $list_settings, 'name' ) ) ) {
		return;
	}

	$text = '<p>' . _n( 'Newsletter Subscription', 'Newsletter Subscriptions', count( wp_list_pluck( $list_settings, 'name' ) ), 'rcp-mailchimp-pro' ) . '</p>';
	?>
	<div id="rcp-newsletter-opt-in-status">
		<?php
		/**
		 * Filters the description text.
		 *
		 * @param string $text
		 *
		 * @since 1.5
		 */
		echo apply_filters( 'rcp_mailchimp_pro_edit_profile_subscriptions_text', $text );
		?>
		<ul>
			<?php foreach ( $list_settings as $list_id => $list_info ) :
				if ( empty( $list_info['name'] ) ) {
					continue;
				}
				?>
				<li>
					<?php if ( ! empty( $list_info['membership_id'] ) ) : ?>
						<input type="hidden" name="rcp_newsletter_list[<?php echo esc_attr( $list_id ); ?>][membership_id]" value="<?php echo esc_attr( $list_info['membership_id'] ); ?>">
					<?php endif; ?>
					<input type="hidden" name="rcp_newsletter_list[<?php echo esc_attr( $list_id ); ?>][opt_in]" value="">
					<input type="checkbox" id="rcp-newsletter-list-<?php echo esc_attr( $list_id ); ?>" name="rcp_newsletter_list[<?php echo esc_attr( $list_id ); ?>][opt_in]" <?php checked( 'subscribed', $list_info['status'] ); ?>>
					<label for="rcp-newsletter-list-<?php echo esc_attr( $list_id ); ?>">
						<?php
						echo esc_html( $list_info['name'] );
						if ( 'pending' === $list_info['status'] ) {
							echo '&nbsp;' . __( '(Pending - please check your email for confirmation)', 'rcp-mailchimp-pro' );
						}
						?>
					</label>
				</li>
			<?php endforeach; ?>
		</ul>
	</div>
	<?php

}

add_action( 'rcp_profile_editor_after', __NAMESPACE__ . '\maybe_show_opt_in_out_checkboxes' );

/**
 * Updates subscription preferences when a user edits their profile.
 *
 * @param int      $user_id
 * @param array    $new_userdata
 * @param \WP_User $old_userdata
 *
 * @since 1.5
 */
function change_subscription_preferences( $user_id, $new_userdata, $old_userdata ) {

	$settings = get_option( 'rcp_mailchimp_pro_settings' );

	// Bail if not enabled.
	if ( empty( $settings['allow_profile_opt_in'] ) || empty( $_POST['rcp_newsletter_list'] ) || ! is_array( $_POST['rcp_newsletter_list'] ) ) {
		return;
	}

	// Bail if we don't have an email for some reason.
	if ( empty( $new_userdata['user_email'] ) ) {
		return;
	}

	rcp_log( sprintf( 'MailChimp Pro: Checking to see if list subscriptions have changed for user #%d.', $user_id ) );

	$customer = rcp_get_customer_by_user_id( $user_id );
	$api      = new API();

	$subscribed_lists = get_user_meta( $user_id, 'rcp_mailchimp_pro_subscribed_lists', true );

	if ( ! is_array( $subscribed_lists ) ) {
		$subscribed_lists = array();
	}

	foreach ( $_POST['rcp_newsletter_list'] as $list_id => $list_settings ) {

		if ( ! isset( $list_settings['opt_in'] ) ) {
			continue;
		}

		$membership     = ! empty( $list_settings['membership_id'] ) ? rcp_get_membership( absint( $list_settings['membership_id'] ) ) : false;
		$desired_status = $list_settings['opt_in'];

		try {
			$is_subscribed = $api->is_subscribed( $new_userdata['user_email'], $list_id );
		} catch ( \Exception $e ) {
			rcp_log( sprintf( 'MailChimp Pro: Failed to check subscription status for user #%d. Message: %s. Exiting.', $user_id, $e->getMessage() ) );

			return;
		}

		if ( empty( $desired_status ) ) {

			/**
			 * User is opting out
			 */

			// If the user isn't subscribed, we can bail.
			if ( ! $is_subscribed ) {
				rcp_log( sprintf( 'MailChimp Pro: No change in subscription status for user #%d on list #%s.', $user_id, $list_id ) );

				continue;
			}

			rcp_log( sprintf( 'MailChimp Pro: Detected change in subscription preferences for user #%d on list #%s. Opting user out.', $user_id, $list_id ) );

			try {
				$request_endpoint = 'lists/' . urlencode( $list_id ) . '/members/' . urlencode( md5( strtolower( $new_userdata['user_email'] ) ) ) . '?skip_merge_validation=true';
				$api->make_request( $request_endpoint, array(
					'method' => 'PATCH',
					'body'   => json_encode( array(
						'status' => 'unsubscribed'
					) )
				) );

				rcp_log( sprintf( 'MailChimp Pro: User #%d successfully unsubscribed from list %s.', $user_id, $list_id ) );
				$customer->add_note( sprintf( __( 'MailChimp Pro: Customer unsubscribed from list %s.', 'rcp-mailchimp-pro' ), $list_id ) );

				// Remove this list from their subscribed lists.
				$list_key = array_search( $list_id, $subscribed_lists, true );
				if ( false !== $list_key ) {
					unset( $subscribed_lists[ $list_key ] );

					update_user_meta( $user_id, 'rcp_mailchimp_pro_subscribed_lists', $subscribed_lists );
				}
			} catch ( \Exception $e ) {
				rcp_log( sprintf( 'MailChimp Pro: Failed to opt user #%d out of list %s. Message: %s; Code: %s', $user_id, $list_id, $e->getMessage(), $e->getCode() ), true );
				$customer->add_note( sprintf(
					__( 'MailChimp Pro: Failed to opt user #%d out of list %s. Message: %s; Code: %s', 'rcp-mailchimp-pro' ),
					$user_id, $list_id, $e->getMessage(), $e->getCode()
				) );
			}

		} else {

			/**
			 * User is opting in
			 */

			// If the user is already subscribed, we can bail.
			if ( $is_subscribed ) {
				rcp_log( sprintf( 'MailChimp Pro: No change in subscription status for user #%d on list #%s.', $user_id, $list_id ) );

				continue;
			}

			rcp_log( sprintf( 'MailChimp Pro: Detected change in subscription preferences for user #%d on list #%s. Opting user in.', $user_id, $list_id ) );

			try {

				$body_args = array(
					'merge_fields' => array(
						'FNAME' => $new_userdata['first_name'],
						'LNAME' => $new_userdata['last_name']
					)
				);

				if ( $membership instanceof RCP_Membership ) {
					$body_args['merge_fields']['STATUS'] = $membership->get_status();
				}

				// Gather up interests.
				if ( $membership instanceof RCP_Membership ) {
					$interest = get_membership_level_group( $membership->get_object_id() );

					if ( empty( $interest ) || 'inherit' === $interest ) {
						$interest = $settings['saved_group'];
					}

					if ( ! empty( $interest ) ) {
						$body_args['interests'][ $interest ] = true;
					}
				}

				$subscriber = $api->subscribe( $new_userdata['user_email'], $list_id, $body_args );

				$new_status = ! empty( $subscriber->status ) ? $subscriber->status : __( 'unknown', 'rcp-mailchimp-pro' );

				rcp_log( sprintf( 'MailChimp Pro: User #%d successfully subscribed to list %s. New status: %s', $user_id, $list_id, $new_status ) );
				$customer->add_note( sprintf( __( 'MailChimp Pro: Customer subscribed to list %s. New status: %s.', 'rcp-mailchimp-pro' ), $list_id, $new_status ) );

				// Add this list to their subscribed lists.
				if ( ! in_array( $list_id, $subscribed_lists ) ) {
					$subscribed_lists[] = $list_id;

					update_user_meta( $user_id, 'rcp_mailchimp_pro_subscribed_lists', $subscribed_lists );
				}

			} catch ( \Exception $e ) {
				rcp_log( sprintf( 'MailChimp Pro: Failed to subscriber user #%d to list %s. Message: %s; Code: %s', $user_id, $list_id, $e->getMessage(), $e->getCode() ), true );
				$customer->add_note( sprintf(
					__( 'MailChimp Pro: Failed to subscriber user #%d to list %s. Message: %s; Code: %s', 'rcp-mailchimp-pro' ),
					$user_id, $list_id, $e->getMessage(), $e->getCode()
				) );
			}

		}

	}

}

add_action( 'rcp_user_profile_updated', __NAMESPACE__ . '\change_subscription_preferences', 10, 3 );