<?php
/**
 * Visibility Filters
 *
 * Control access to restricted content.
 *
 * @package   rcp-restrict-past-content
 * @copyright Copyright (c) 2019, Restrict Content Pro team
 * @license   GPL2+
 * @since     1.0
 */

namespace RCP\Addon\RestrictPastContent;

use \RCP_Membership;
use \DateTime;
use \DateTimeZone;
use \Exception;

defined( 'WPINC' ) or die;

/**
 * Prevents a membership from granting access to the requested content if it was published prior to the
 * membership start date.
 *
 * @param bool           $can_access    Whether or not the membership grants access to the post.
 * @param int            $membership_id ID of the membership being checked.
 * @param int            $post_id       ID of the post being checked.
 * @param RCP_Membership $membership    Membership object.
 *
 * @throws Exception
 * @since 1.0.2
 * @return bool
 */
function membership_can_access( $can_access, $membership_id, $post_id, $membership ) {

	// If it already doesn't grant access, bail.
	if ( ! $can_access ) {
		return $can_access;
	}

	// Return early for editors to avoid additional queries.
	if ( user_can( $can_access, 'edit_others_posts' ) ) {
		return true;
	}

	// Bypass date restrictions.
	$exclude = get_post_meta( $post_id, 'rcp_restrict_past_content_exclude', true );
	if ( ! empty( $exclude ) ) {
		return $can_access;
	}

	// Bail if the content isn't even restricted.
	if ( ! rcp_is_restricted_content( $post_id ) ) {
		return $can_access;
	}

	$timezone = get_option( 'timezone_string' );
	$timezone = ! empty( $timezone ) ? $timezone : 'UTC';

	try {
		$post_date = new DateTime( get_post_field( 'post_date', $post_id ), new DateTimeZone( $timezone ) );
	} catch ( Exception $e ) {
		return $can_access;
	}

	/**
	 * @var \RCP_Levels $rcp_levels_db
	 */
	global $rcp_levels_db;

	// Bail if the membership doesn't restrict past content.
	if ( ! $rcp_levels_db->get_meta( $membership->get_object_id(), 'restrict_past_content', true ) ) {
		return $can_access;
	}

	try {
		$start_date = new DateTime( $membership->get_created_date( false ), new DateTimeZone( $timezone ) );
	} catch ( Exception $e ) {
		return $can_access;
	}

	$start_date->modify( 'midnight' );

	$offset = $rcp_levels_db->get_meta( $membership->get_object_id(), 'restrict_past_content_offset', true );

	if ( $offset ) {
		$offset = - $offset . ' days';
		$start_date->modify( $offset );
	}

	// Deny access if the post was published before the start date.
	if ( $post_date < $start_date ) {
		$can_access = false;
	}

	return $can_access;

}

add_filter( 'rcp_membership_can_access', __NAMESPACE__ . '\membership_can_access', 100, 4 );

/**
 * Determines if the member can access the requested content
 * based on the member's join date.
 * @since 1.0
 * @return bool True if the content is visible to the member, false if not.
 */
function can_access( $ret, $user_id, $post_id, $user ) {

	// Bail if we're on RCP 3.0+.
	if ( function_exists( 'rcp_get_membership' ) ) {
		return $ret;
	}

	if ( ! $ret ) {
		return false;
	}

	if ( ! rcp_is_restricted_content( $post_id ) ) {
		return $ret;
	}

	if ( ! is_user_logged_in() ) {
		return $ret;
	}

	// Return early for editors to avoid additional queries.
	if ( user_can( $user, 'edit_others_posts' ) ) {
		return true;
	}

	$sub_id = (int) rcp_get_subscription_id( $user_id );

	if ( ! $sub_id ) {
		return $ret;
	}

	global $rcp_levels_db;

	if ( ! $rcp_levels_db->get_meta( $sub_id, 'restrict_past_content', true ) ) {
		return $ret;
	}

	$member = new \RCP_Member( wp_get_current_user()->ID );

	$timezone = get_option( 'timezone_string' );

	$timezone = ! empty( $timezone ) ? $timezone : 'UTC';

	$join_date = new \DateTime( $member->get_joined_date( $sub_id ), new \DateTimeZone( $timezone ) );

	$join_date->modify( 'midnight' );

	$post_date = new \DateTime( get_post_field( 'post_date', $post_id ), new \DateTimeZone( $timezone ) );

	$offset = $rcp_levels_db->get_meta( $sub_id, 'restrict_past_content_offset', true );

	if ( $offset ) {
		$offset = - $offset . ' days';
		$join_date->modify( $offset );
	}

	if ( $post_date >= $join_date ) {
		$ret = true;
	} else {
		$ret = false;
	}

	$exclude = get_post_meta( $post_id, 'rcp_restrict_past_content_exclude', true );

	if ( ! $ret && ! empty( $exclude ) ) {
		$ret = true;
	}

	return $ret;
}