<?php
/**
 * Helper functions
 *
 * @package     RCP\Site_Creation\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Get AJAX URL
 *
 * @since       1.0.0
 * @return      string URL to the AJAX file to call during AJAX requests.
 */
function rcp_site_creation_get_ajax_url() {
	$scheme = defined( 'FORCE_SSL_ADMIN' ) && FORCE_SSL_ADMIN ? 'https' : 'admin';

	$current_url = rcp_site_creation_get_current_page_url();
	$ajax_url    = admin_url( 'admin-ajax.php', $scheme );

	if( preg_match( '/^https/', $current_url ) && ! preg_match( '/^https/', $ajax_url ) ) {
		$ajax_url = preg_replace( '/^http/', 'https', $ajax_url );
	}

	return apply_filters( 'rcp_site_creation_ajax_url', $ajax_url );
}


/**
 * Get the current page URL
 *
 * @since       1.0.0
 * @param       bool $nocache If we should bust cache on the returned URL
 * @return      string $page_url Current page URL
 */
function rcp_site_creation_get_current_page_url( $nocache = false ) {
	global $wp;

	if( get_option( 'permalink_structure' ) ) {
		$base = trailingslashit( home_url( $wp->request ) );
	} else {
		$base = add_query_arg( $wp->query_string, '', trailingslashit( home_url( $wp->request ) ) );
		$base = remove_query_arg( array( 'post_type', 'name' ), $base );
	}

	$scheme = is_ssl() ? 'https' : 'http';
	$uri    = set_url_scheme( $base, $scheme );

	if( is_front_page() ) {
		$uri = home_url( '/' );
	}

	$uri = apply_filters( 'rcp_site_creation_get_current_page_url', $uri );

	if ( $nocache ) {
		$uri = rcp_site_creation_add_cache_busting( $uri );
	}

	return $uri;
}


/**
 * Adds the 'nocache' parameter to the provided URL
 *
 * @since       1.0.0
 * @param       string $url The URL being requested
 * @return      string The URL with cache busting added or not
 */
function rcp_site_creation_add_cache_busting( $url = '' ) {
	if( rcp_site_creation_is_caching_plugin_active() ) {
		$url = add_query_arg( 'nocache', 'true', $url );
	}

	return $url;
}


/**
 * Checks if a caching plugin is active
 *
 * @since       1.0.0
 * @return      bool $caching True if caching plugin is enabled, false otherwise
 */
function rcp_site_creation_is_caching_plugin_active() {
	$caching = ( function_exists( 'wpsupercache_site_admin' ) || defined( 'W3TC' ) || function_exists( 'rocket_init' ) );
	return apply_filters( 'rcp_site_creation_is_caching_plugin_active', $caching );
}


/**
 * Check if a given level allows site creation
 *
 * @since       1.0.0
 * @param       int $level The level to check
 * @return      bool $allows_site_creation True if allowed, false otherwise
 */
function rcp_site_creation_allowed( $level ) {
	$allows_site_creation = false;
	$settings             = get_option( 'rcp_site_creation_subscription_allow_site_creation' );
	if( is_array( $settings ) && array_key_exists( $level, $settings ) ) {
		$allows_site_creation = true;
	}

	return $allows_site_creation;
}


/**
 * Sanitize a subsite URL
 *
 * @since       1.0.0
 * @param       string $url The URL to sanitize
 * @return      string $url The sanitized URL
 */
function rcp_site_creation_sanitize_url( $url ) {
	$settings = get_option( 'rcp_site_creation_settings' );
	$pattern  = ! empty( $settings['domain_mapping'] ) ? '/[^A-Za-z0-9. ]/' : '/[^A-Za-z0-9 ]/';

	$url = trim( preg_replace( '/ +/', ' ', preg_replace( $pattern, ' ', urldecode( html_entity_decode( strip_tags( $url ) ) ) ) ) );
	$url = str_replace( ' ', '-', $url );

	return $url;
}


/**
 * Get the format for URLs
 *
 * @since       1.0.0
 * @param       string $subsite Optional subsite url
 * @return      array Array containing domain and path
 */
function rcp_site_creation_get_format( $subsite = false ) {
	if( ! $subsite ) {
		return false;
	}

	$settings = get_option( 'rcp_site_creation_settings' );

	if ( ! empty( $settings['domain_mapping'] ) ) {

		// Custom, domain mapping URL.
		$new_domain = rcp_site_creation_sanitize_url( $subsite );
		$path       = '/';

	} else {

		// Subdomain or subdirectory URL.
		$subsite    = rcp_site_creation_sanitize_url( $subsite );
		$site_parts = parse_url( network_home_url() );
		$main_site  = $site_parts['host'];

		if ( array_key_exists( 'port', $site_parts ) ) {
			$main_site = $main_site . ( $main_site['port'] == '80' ? '' : ':' . $main_site['port'] );
		}

		if ( is_subdomain_install() ) {
			$main_site  = preg_replace( '|^www\.|', '', $main_site );
			$new_domain = "{$subsite}.$main_site";
			$path       = '/';
		} else {
			$new_domain = $main_site;
			$path       = "/{$subsite}/";
		}

	}

	return array( 'domain' => $new_domain, 'path' => $path );
}

/**
 * Get upload quota for a subscription level.
 *
 * @param int $level ID of the level to check.
 *
 * @since 1.1
 * @return int|false Quota in MB or false if none is set.
 */
function rcp_site_creation_get_upload_quota( $level ) {
	/**
	 * @var RCP_Levels $rcp_levels_db
	 */
	global $rcp_levels_db;

	$quota = $rcp_levels_db->get_meta( $level, 'site_creation_upload_quota', true );

	if ( empty( $quota ) ) {
		$quota = false;
	}

	return apply_filters( 'rcp_site_creation_upload_quota', $quota, $level );
}

/**
 * Set upload quota for a subscription level.
 *
 * @param int $level ID of the subscription level to set the quota for.
 * @param int $quota Quota in MB.
 *
 * @since 1.1
 * @return bool False for failure. True for success.
 */
function rcp_site_creation_set_upload_quota( $level, $quota ) {
	/**
	 * @var RCP_Levels $rcp_levels_db
	 */
	global $rcp_levels_db;

	return $rcp_levels_db->update_meta( $level, 'site_creation_upload_quota', absint( $quota ) );
}

/**
 * Delete upload quota for a subscription level.
 *
 * @param int $level ID of the subscription level.
 *
 * @since 1.1
 * @return bool False for failure. True for success.
 */
function rcp_site_creation_remove_upload_quota( $level ) {
	/**
	 * @var RCP_Levels $rcp_levels_db
	 */
	global $rcp_levels_db;

	return $rcp_levels_db->delete_meta( $level, 'site_creation_upload_quota' );
}

/**
 * Returns a blog ID given its slug or custom domain.
 *
 * @param string $slug Blog subdomain/directory slug or custom domain name.
 *
 * @since 1.2
 * @return int|null Blog ID or null if none.
 */
function rcp_site_creation_get_id_from_blogname( $slug ) {

	$settings = get_option( 'rcp_site_creation_settings' );

	if ( ! empty( $settings['domain_mapping'] ) && false !== strpos( $slug, '.' ) ) {

		$site_ids = get_sites( array(
			'number' => 1,
			'fields' => 'ids',
			'domain' => $slug
		) );

		if ( empty( $site_ids ) ) {
			return null;
		}

		return array_shift( $site_ids );

	} else {

		return get_id_from_blogname( $slug );

	}

}

/**
 * Create a new site on the network for a given membership.
 *
 * @param RCP_Membership|int $membership Membership object or ID number.
 * @param array              $args       Site arguments.
 *
 * @since 1.2.1
 * @return int|WP_Error ID of the new site if created, or WP_Error object on failure.
 */
function rcp_site_creation_create_site_for_membership( $membership, $args = array() ) {

	$settings = get_option( 'rcp_site_creation_settings' );

	if ( ! is_object( $membership ) ) {
		$membership = rcp_get_membership( $membership );
	}

	if ( ! is_a( $membership, 'RCP_Membership' ) ) {
		return new WP_Error( 'invalid_membership', __( 'Please supply a valid membership object or ID.', 'rcp-site-creation' ) );
	}

	$default_args = array(
		'url'         => '', // Just used for backwards compat.
		'domain'      => '',
		'path'        => '',
		'name'        => '',
		'description' => '',
		'scheme'      => 'http',
		'quota'       => rcp_site_creation_get_upload_quota( $membership->get_object_id() ),
		'meta'        => array(
			'public' => 1 // Site status
		)
	);

	$args = wp_parse_args( $args, $default_args );

	// URL is required.
	if ( empty( $args['domain'] ) || empty( $args['path'] ) ) {
		return new WP_Error( 'empty_domain_path', __( 'The site domain and path are required.', 'rcp-site-creation' ) );
	}

	$meta = $args['meta'];

	if ( ! empty( $args['description'] ) ) {
		$meta['blogdescription'] = sanitize_text_field( $args['description'] );
	}

	if ( ! empty( $args['quota'] ) ) {
		$meta['blog_upload_space'] = absint( $args['quota'] );
	}

	if ( ! empty( $settings['domain_mapping'] ) ) {
		// We need to overwrite the site/home URLs manually in order to change the scheme.
		$url = $args['scheme'] . '://' . $args['domain'];
		$meta['siteurl'] = sanitize_text_field( $url );
		$meta['home']    = sanitize_text_field( $url );
	}

	/**
	 * Filters the meta array used during site creation.
	 *
	 * @since 1.0.0
	 *
	 * @param array $meta Site meta array.
	 * @param array $args Site arguments.
	 */
	$meta = apply_filters( 'rcp_site_creation_meta', $meta, $args );

	// WP 5.1+ compatibility.
	if ( function_exists( 'wp_insert_site' ) ) {
		$site_id = wp_insert_site( array(
			'domain'  => $args['domain'],
			'path'    => $args['path'],
			'title'   => $args['name'],
			'user_id' => $membership->get_customer()->get_user_id(),
			'meta'    => $meta,
		) );
	} else {
		$site_id = wpmu_create_blog( $args['domain'], $args['path'], $args['name'], $membership->get_customer()->get_user_id(), $meta );
	}

	if ( is_wp_error( $site_id ) ) {
		return $site_id;
	}

	// Set theme.
	if ( isset( $settings['default_theme'] ) && ! empty( $settings['default_theme'] ) ) {
		switch_to_blog( $site_id );
		switch_theme( $settings['default_theme'] );
		restore_current_blog();
	}

	/**
	 * Fires immediately after a site has been created and the default theme set,
	 * but before the user meta flag is deleted.
	 *
	 * @since 1.2
	 *
	 * @param array $site_meta Array of meta for the newly-added site.
	 * @param int   $site_id   ID of the new site.
	 * @param int   $user_id   User ID for the admin associated with the new site.
	 */
	do_action( 'rcp_site_creation_site_created', $args, $site_id, $membership->get_customer()->get_user_id() );

	/**
	 * Add meta to this membership (and user) designating that this is their Site Creation site.
	 */
	rcp_update_membership_meta( $membership->get_id(), 'site_creation_site_id', absint( $site_id ) );

	return $site_id;

}

/**
 * Get the site ID that corresponds to a given membership.
 *
 * If the meta is not set for the specified membership, then the old user meta value is checked and that data is then
 * migrated to membership meta for future use.
 *
 * @param int $membership_id Membership ID.
 *
 * @since 1.2.1
 * @return int|false ID of the site or false if none.
 */
function rcp_site_creation_get_membership_site_id( $membership_id ) {
	if ( ! function_exists( 'rcp_get_membership_meta' ) ) {
		return false;
	}

	$site_id = rcp_get_membership_meta( $membership_id, 'site_creation_site_id', true );

	// If the membership meta is set, return that now.
	if ( ! empty( $site_id ) ) {
		return $site_id;
	}

	/*
	 * At this point we don't have membership meta, so we'll use the deprecated function to check the old user meta
	 * value. If that's set, then the function will also migrate that value to membership meta for next time.
	 */

	$membership = rcp_get_membership( $membership_id );

	if ( empty( $membership ) ) {
		return false;
	}

	$user_id = $membership->get_customer()->get_user_id();

	if ( empty( $user_id ) ) {
		return false;
	}

	$site_id = get_user_meta( $user_id, 'rcp_site_creation_blog_id', true );

	if ( empty( $site_id ) ) {
		return false;
	}

	rcp_add_membership_meta( $membership_id, 'site_creation_site_id', absint( $site_id ) );
	delete_user_meta( $user_id, 'rcp_site_creation_blog_id' );

	return absint( $site_id );
}