<?php
/**
 * Add Taxonomy Fields
 *
 * @package   RCP\Timelock\Admin\Settings\Taxonomies
 * @copyright Copyright (c) 2017, Restrict Content Pro
 * @license   GPL2+
 * @since     1.1
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Setup actions for taxonomy restricted fields.
 *
 * @since 1.1
 * @return void
 */
function rcp_timelock_setup_taxonomy_edit_fields() {

	$taxonomies = rcp_get_restricted_taxonomies();

	foreach ( $taxonomies as $taxonomy ) {
		add_action( "{$taxonomy}_add_form_fields", 'rcp_timelock_term_add_meta_fields' );
		add_action( "{$taxonomy}_edit_form_fields", 'rcp_timelock_term_edit_meta_fields' );
	}

}

add_action( 'admin_init', 'rcp_timelock_setup_taxonomy_edit_fields' );

/**
 * Add restriction timelock to the add term page.
 *
 * @param string $taxonomy Taxonomy slug.
 *
 * @since 1.1
 * @return void
 */
function rcp_timelock_term_add_meta_fields( $taxonomy ) {

	?>
	<div class="form-field">
		<h2><?php _e( 'Restriction Time', 'rcp-timelock' ); ?></h2>
		<label for="rcp_timelock">
			<input type="text" name="rcp_timelock" id="rcp_timelock">
			<span class="description"><?php printf( __( 'Choose when this %s should become restricted.', 'rcp-timelock' ), $taxonomy ); ?></span>
		</label>
	</div>
	<?php

}

/**
 * Add restriction timelock to the edit term page.
 *
 * @param WP_Term $term Term object.
 *
 * @since 1.1
 * @return void
 */
function rcp_timelock_term_edit_meta_fields( $term ) {

	$timelock     = rcp_timelock_get_term_lock( $term->term_id );
	$timelock_was = rcp_timelock_get_past_term_lock( $term->term_id );
	$taxonomy     = ( isset( $_GET['taxonomy'] ) ? $_GET['taxonomy'] : __( 'category', 'rcp-timelock' ) );
	?>
	<tr>
		<th scope="row"><?php _e( 'Restriction Time', 'rcp-timelock' ); ?></th>
		<td>
			<label for="rcp_timelock">
				<input type="text" name="rcp_timelock" id="rcp_timelock" value="<?php echo esc_attr( $timelock ); ?>"/>
				<?php printf( __( 'Choose when this %s should become restricted.', 'rcp-timelock' ), $taxonomy ); ?>
			</label>
		</td>
	</tr>
	<?php if ( $timelock_was ) : ?>
		<tr>
			<th scope="row"><?php _e( 'Restriction Time Was:', 'rcp-timelock' ); ?></th>
			<td>
				<span><?php echo esc_html( $timelock_was ); ?></span>
			</td>
		</tr>
	<?php endif; ?>
	<?php

}

/**
 * Save custom term meta.
 *
 * @param int    $term_id  ID of the term being saved.
 * @param int    $tt_id    Taxonomy term ID.
 * @param string $taxonomy Taxonomy slug.
 *
 * @since 1.1
 * @return void
 */
function rcp_timelock_save_term_meta( $term_id, $tt_id, $taxonomy ) {

	if ( empty( $_POST['rcp_edit_term'] ) || ! wp_verify_nonce( $_POST['rcp_edit_term'], 'rcp_edit_term' ) ) {
		return;
	}

	if ( ! function_exists( 'update_term_meta' ) ) {
		return;
	}

	$restricted_taxonomies = rcp_get_restricted_taxonomies();

	// Bail if this taxonomy isn't supported for restrictions.
	if ( ! in_array( $taxonomy, $restricted_taxonomies ) ) {
		return;
	}

	$timelock = ! empty( $_POST['rcp_timelock'] ) ? sanitize_text_field( $_POST['rcp_timelock'] ) : false;

	if ( ! empty( $timelock ) ) {
		update_term_meta( $term_id, 'rcp_timelock', $timelock );
	} else {
		delete_term_meta( $term_id, 'rcp_timelock' );
	}

}

add_action( 'edited_term', 'rcp_timelock_save_term_meta', 10, 3 );
add_action( 'created_term', 'rcp_timelock_save_term_meta', 10, 3 );