<?php

RCPBP_Components::get_instance();

class RCPBP_Components {

	/**
	 * @var
	 */
	protected static $_instance;

	/**
	 * Only make one instance of the RCPBP_Components
	 *
	 * @return RCPBP_Components
	 */
	public static function get_instance() {
		if ( ! self::$_instance instanceof RCPBP_Components ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Add Hooks and Actions
	 */
	protected function __construct() {
		$this->hooks();
	}

	protected function hooks() {
		add_filter( 'rcp_member_can_access', [ $this, 'can_access' ], 11, 4 );
		add_action( 'rcp_add_subscription_form', [ $this, 'meta_fields' ] );
		add_action( 'rcp_edit_subscription_form', [ $this, 'meta_fields' ] );
		add_action( 'rcp_edit_subscription_level', [ $this, 'meta_save' ], 10, 2 );
		add_action( 'rcp_add_subscription', [ $this, 'meta_save' ], 10, 2 );
		add_action( 'rcpbp_levels_meta_fields', [ $this, 'level_fields' ] );
		add_filter( 'bp_get_send_public_message_button', [ $this, 'public_message_button' ] );
		add_filter( 'bp_get_template_part', [ $this, 'filter_form_template' ], 10, 3 );
		add_action( 'template_redirect', [ $this, 'maybe_redirect' ] );
		add_action( 'bp_init', [ $this, 'message' ] );
	}

	public function message() {
		$action  = isset( $_GET['rcpbp'] ) ? $_GET['rcpbp'] : false;
		$message = [];

		if ( empty( $action ) ) {
			return;
		}

		global $rcp_options;

		switch ( $action ) {
			case 'restricted':
				$message['message'] = __( 'The content you requested is restricted.', 'rcpbp' );
				$message['type']    = 'warning';
		}

		$message = apply_filters( 'rcpbp_message', $message, $action );

		if ( isset( $message['message'] ) ) {
			bp_core_add_message( $message['message'], $message['type'] );
		}
	}

	/**
	 * Maybe redirect a user to their profile tab
	 * if the current profile tab requested is
	 * restricted via Component
	 */
	public function maybe_redirect() {

		// Leave admins alone.
		if ( current_user_can( 'manage_options' ) ) {
			return;
		}

		$current_component = bp_current_component();

		// Not a BuddyPress component page
		if ( ! $current_component ) {
			return;
		}

		$user_components = $this->get_user_components();
		$user_components = $this->profile_mapper( $user_components );

		$can_access     = apply_filters( 'rcpbp_redirect_can_access_component', in_array( $current_component, $user_components ) );
		$profile_access = apply_filters( 'rcpbp_redirect_can_access_profile', in_array( 'profile', $user_components ) );

		// If user has access to xprofile, is on a profile page, but cannot access it
		if ( $profile_access && bp_displayed_user_id() && ! $can_access ) {
			wp_redirect( bp_get_displayed_user_link() . 'profile/?rcpbp=restricted' );
			die;
		}
	}

	/**
	 * Determine whether users are allowed to access
	 * content based on the current page's component
	 *
	 * @param $can_access
	 * @param $member_id
	 * @param $post_id
	 * @param $member
	 *
	 * @return bool
	 * @author Landon Otis
	 */
	public function can_access( $can_access, $member_id, $post_id, $member ) {

		// Early return if component restriction isn't enabled
		if ( ! $this->components_enabled() ) {
			return $can_access;
		}

		// Leave admins access alone.
		if ( current_user_can( 'manage_options' ) ) {
			return $can_access;
		}

		$current_component = bp_current_component();

		// Not a BuddyPress component page
		if ( empty( $current_component ) ) {
			return $can_access;
		}

		// Can't access unless membership allows
		$can_access = false;

		$user_components = $this->get_user_components();
		$user_components = $this->profile_mapper( $user_components );

		if ( in_array( $current_component, $user_components ) ) {
			$can_access = true;
		}

		return apply_filters( 'rcpbp_member_can_access', $can_access, get_current_user_id(), $group = null, $post_id );
	}

	/**
	 * Need to change component name xprofile to profile since
	 * BuddyPress calls it xprofile, but stores it as profile
	 *
	 * @param $user_components
	 *
	 * @return array
	 */
	public function profile_mapper( $user_components ) {
		$xprofile = array_search( 'xprofile', $user_components );

		if ( false !== $xprofile ) {
			$user_components[ $xprofile ] = 'profile';
		}

		return $user_components;
	}

	/**
	 * Add meta fields to the RCP levels
	 *
	 * @param null $level
	 */
	public function meta_fields( $level = null ) {
		$active_components = apply_filters( 'bp_active_components', bp_get_option( 'bp-active-components' ) );
		$is_settings       = isset( $_GET['page'] ) && 'rcpbp-settings' === $_GET['page'];
		$action            = $is_settings ? 'rcpbp_settings_fields' : 'rcpbp_levels_meta_fields';

		if ( $is_settings ) {
			$components = (array) get_option( 'rcpbp_public_components' );
			$message    = __( 'Choose which components logged out users can access.', 'rcpbp' );
		} else {
			$components = empty( $level->id ) ? [] : (array) $this->meta_get( $level->id, 'rcpbp_components' );
			$message    = sprintf( __( 'Choose which components this membership level can access. Enable in global <a href="%s" target="_blank">settings</a>.', 'rcpbp' ), '/wp-admin/admin.php?page=rcpbp-settings&tab=restrict_options' );
		}

		?>

        <tr class="form-field" id="rcpbp-components">
            <th scope="row" valign="top">
                <label for="rcpbp-components"><?php _e( 'BuddyPress Components', 'rcpbp' ); ?></label>
            </th>
            <td>
				<?php foreach ( $active_components as $name => $active ) {
					$checked = in_array( $name, $components ) ? 'checked' : '';
					?>
                    <input type="checkbox" name="rcpbp-components[]" <?php echo $checked ?>
                           value="<?php echo $name ?>"><label><?php echo $name ?></label>
				<?php } ?>
                <p class="description"><?php echo $message ?></p>
            </td>
        </tr>
		<?php
		do_action( $action, $level );
	}

	public function level_fields( $level ) {
		$setting = empty( $level->id ) ? 0 : $this->meta_get( $level->id, 'rcpbp_public_message' );
		?>
        <tr class="form-field">
            <th scope="row" valign="top">
                <label for="rcpbp-public-message"><?php _e( 'BuddyPress Public Messages', 'rcpbp' ); ?></label>
            </th>
            <td>
                <input type="checkbox" name="rcpbp-public-message" <?php checked( $setting ) ?> value="1">
                <label><?php _e( 'Allow BuddyPress public messages.', 'rcpbp' ) ?></label>
            </td>
        </tr>
		<?php
	}

	/**
	 * Get subscription meta
	 *
	 * @param $level_id
	 * @param $key
	 *
	 * @return mixed
	 */
	public static function meta_get( $level_id, $key ) {
		global $rcp_levels_db;

		$meta = $rcp_levels_db->get_meta( $level_id, $key, true );

		return apply_filters( "rcpbp_meta_get_{$key}", $meta, $level_id );
	}

	/**
	 * Save meta for the subscription
	 *
	 * @param $level_id
	 * @param $args
	 */
	public function meta_save( $level_id, $args ) {
		global $rcp_levels_db;
		$components  = isset( $_POST['rcpbp-components'] ) ? (array) $_POST['rcpbp-components'] : [];
		$message     = isset( $_POST['rcpbp-public-message'] ) ? $_POST['rcpbp-public-message'] : 0;
		$group_limit = isset( $_POST['rcpbp-group-limit'] ) ? $_POST['rcpbp-group-limit'] : 0;

		$rcp_levels_db->update_meta( $level_id, 'rcpbp_components', array_map( 'esc_attr', $components ) );
		$rcp_levels_db->update_meta( $level_id, 'rcpbp_public_message', esc_attr( $message ) );
		$rcp_levels_db->update_meta( $level_id, 'rcpbp_group_limit', absint( $group_limit ) );
	}

	/**
	 * Get allowed components of current user whether they
	 * are logged-in, logged-out, or have a membership
	 *
	 * @return array user's allowed components
	 */
	public function get_user_components() {

		$components = [];

		/**
		 * Users without a membership (logged-in or logged-out)
		 * are permitted use of components on a global scale.
		 */
		if ( ! rcp_user_has_active_membership( get_current_user_id() ) ) {
			$components = apply_filters( 'rcpbp_public_components', (array) get_option( 'rcpbp_public_components' ) );
			$components = $this->profile_mapper( $components );
		}

		/* todo: decide whether we want public components
         * merged into the same components array as members.
         * This would mean all members have at least what is
         * available to the public. Is there any reason we'd
         * want to support members having less access than public?
         */

		/**
		 * Retrieve components for users with an active membership
		 */
		global $rcp_levels_db;
		$customer_levels = rcp_get_customer_membership_level_ids();

		foreach ( $customer_levels as $level_id ) {
			$membership_components = $rcp_levels_db->get_meta( $level_id, 'rcpbp_components', true );
			$components            = array_unique( array_merge( $membership_components, $components ) );
		}

		return apply_filters( 'rcpbp_allowed_user_components', $components, $customer_levels );
	}

	/**
	 *
	 * @return boolean
	 * @author Landon Otis
	 */
	public function components_enabled() {
		return apply_filters( 'rcpbp_components_enabled', (bool) get_option( 'rcpbp_components_toggle' ) );
	}

	/**
	 * Helper function that checks to see if
	 * a user is allowed to send public messages
	 * based on their RCP levels
	 *
	 * @return bool
	 *
	 * @todo do we want to disable public messaging unless a user has a membership that allows it?
	 */
	public function can_user_public_message() {

		$can_message     = false;
		$customer_levels = [];

		if ( rcp_user_has_active_membership() ) {
			global $rcp_levels_db;
			$customer_levels = rcp_get_customer_membership_level_ids();

			foreach ( $customer_levels as $level_id ) {
				$setting = $rcp_levels_db->get_meta( $level_id, 'rcpbp_public_message', true );

				if ( $setting ) {
					$can_message = true;
					break;
				}
			}
		}

		if ( current_user_can( 'manage_options' ) ) {
			$can_message = true;
		}

		return apply_filters( 'rcpbp_can_public_message', $can_message, $customer_levels );
	}

	/**
	 * Hide public message button if
	 * the user is not allowed
	 *
	 * @param $args
	 *
	 * @return bool
	 */
	public function public_message_button( $args ) {
		if ( ! $this->can_user_public_message() ) {
			return apply_filters( 'rcpbp_show_public_message_button', false );
		}

		return $args;
	}

	/**
	 * Filter bp_get_template_part to disable the
	 * activity post form if the user is not allowed
	 * to public message
	 *
	 * @param $templates
	 * @param $slug
	 * @param $name
	 *
	 * @return bool|array
	 */
	public function filter_form_template( $templates, $slug, $name ) {

		if ( $this->can_user_public_message() ) {
			return $templates;
		}

		$form_template = strpos( $slug, 'activity/post-form' );

		// If activity post form, don't load template
		if ( false !== $form_template ) {
			return apply_filters( 'rcpbp_load_post_form_template', false );
		}

		return $templates;
	}

}