<?php

RCPBP_Groups::get_instance();

class RCPBP_Groups {

	/**
	 * @var
	 */
	protected static $_instance;
	protected static $group_count;

	/**
	 * Only make one instance of the RCPBP_Groups
	 *
	 * @return RCPBP_Groups
	 */
	public static function get_instance() {
		if ( ! self::$_instance instanceof RCPBP_Groups ) {
			self::$_instance = new self();
		}

		return self::$_instance;
	}

	/**
	 * Add Hooks and Actions
	 */
	protected function __construct() {
		$this->hooks();
	}

	protected function hooks() {
		add_action( 'bp_groups_admin_meta_boxes', array( $this, 'add_meta_box' ) );
		add_action( 'bp_group_admin_edit_after', array( $this, 'save_meta_data' ) );
		add_action( 'rcp_member_can_access', array( $this, 'maybe_restrict_group' ), 13, 4 );
		add_action( 'rcpbp_metabox_group_fields_after', array( $this, 'bp_fields' ) );
		add_action( 'rcpbp_levels_meta_fields', [ $this, 'level_fields' ] );
		add_filter( 'bp_user_can_create_groups', [ $this, 'can_create_group' ], 10, 2 );
		add_action( 'bp_before_member_groups_content', [ $this, 'group_count_notice' ] );
	}

	/**
	 * Display number of created groups in user's BuddyPress profile
	 *
	 * @author Landon Otis
	 */
	public function group_count_notice() {

		if ( ! apply_filters( 'rcpbp_show_group_count', true ) ) {
			return;
		}

		// Don't allow others to see
		if ( ! current_user_can( 'manage_options' ) ) {
			if ( get_current_user_id() !== bp_displayed_user_id() || get_current_user_id() == 0 || current_user_can( 'manage_options' ) ) {
				return;
			}
		}

		$count = $this->count_user_groups_created( bp_displayed_user_id() );

		if ( empty( $count ) ) {
			return;
		}

		$limit  = $this->group_creation_limit( bp_displayed_user_id() );
		$notice = sprintf( __( 'Groups created: %s of %s', 'rcpbp' ), $count, $limit );

		printf( '<div id="rcpbp_group_count">%s</div>', apply_filters( 'rcpbp_group_count_notice', $notice, $count ) );
	}

	public function level_fields( $level ) {
		$setting = empty( $level->id ) ? 0 : (int) RCPBP_Components::meta_get( $level->id, 'rcpbp_group_limit' );
		?>
        <tr class="form-field">
            <th scope="row" valign="top">
                <label for="rcpbp-group-limit"><?php _e( 'BuddyPress Group Creation Limit', 'rcpbp' ); ?></label>
            </th>
            <td>
                <input type="number" style="width: 100px;" name="rcpbp-group-limit" value="<?php echo $setting ?>">
                <p class="description"><?php _e( "Set membership's BuddyPress group creation limit.", 'rcpbp' ) ?></p>
            </td>
        </tr>
		<?php
	}

	/**
	 * Check if the current user should be allowed to create a
	 * group based on membership limit and how many they've created
	 *
	 * @param $can_create
	 * @param $restricted
	 *
	 * @return bool
	 * @author Landon Otis
	 */
	public function can_create_group( $can_create = true, $restricted = false ) {
		$creation_limit   = $this->group_creation_limit();
		$user_group_count = $this->count_user_groups_created();

		if ( $user_group_count >= $creation_limit ) {
			$can_create = false;
		}

		// Allows last group creation steps to continue since first step creates the group
		if ( ! $can_create && bp_is_group_create() && ! bp_is_first_group_creation_step() ) {
			$can_create = true;
		}

		return apply_filters( 'rcpbp_can_create_group', $can_create, $creation_limit, $user_group_count );
	}

	/**
	 * Returns number of groups a user can create
	 *
	 * @param int $user_id
	 *
	 * @return mixed|void
	 * @author Landon Otis
	 */
	public static function group_creation_limit( $user_id = 0 ) {

		if ( empty( $user_id ) ) {
			$user_id = get_current_user_id();
		}

		$limits = [ 0 ];

		$customer = rcp_get_customer_by_user_id( $user_id );

		if ( ! empty( $customer ) ) {
			global $rcp_levels_db;
			$customer_levels = rcp_get_customer_membership_level_ids( $customer->get_id() );
			$limits          = [ 0 ];

			foreach ( $customer_levels as $level_id ) {
				$level_limit = (int) $rcp_levels_db->get_meta( $level_id, 'rcpbp_group_limit', true );
				$limits[]    = $level_limit;
			}
		}

		return apply_filters( 'rcpbp_user_group_creation_limit', array_sum( $limits ), $limits, $user_id );
	}

	/**
	 * Returns number of groups the current user has created
	 *
	 * @param int $user_id
	 *
	 * @return int|bool int of group count if user is the creator; false on empty user_id.
	 * @author Landon Otis
	 */
	public static function count_user_groups_created( $user_id = 0 ) {

		if ( empty( $user_id ) ) {
			$user_id = get_current_user_id();
		}

		if ( empty( $user_id ) ) {
			return false;
		}

		$count = self::$group_count;

		if ( empty( $count ) ) {
			global $wpdb;
			$bp                = buddypress();
			$count             = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(*) FROM {$bp->groups->table_name} WHERE creator_id = %d", $user_id ) );
			self::$group_count = $count;
		}

		return absint( $count );
	}

	public function add_meta_box() {
		$rcp_meta_box = rcp_get_metabox_fields();
		add_meta_box( $rcp_meta_box['id'], __( 'Membership restrictions for this group.', 'rcpbp' ), array(
			$this,
			'render_meta_box'
		), get_current_screen()->id, $rcp_meta_box['context'], $rcp_meta_box['priority'] );
	}

	public function render_meta_box( $group ) {
		wp_enqueue_style( 'rcpbp-admin-css' );
		wp_enqueue_script( 'rcpbp-admin-js' );

		// enqueue RCP assets
		wp_enqueue_script( 'bbq', RCP_PLUGIN_URL . 'includes/js/jquery.ba-bbq.min.js' );
		wp_enqueue_script( 'rcp-admin-scripts', RCP_PLUGIN_URL . 'includes/js/admin-scripts.js', array(
			'jquery',
			'jquery-ui-tooltip',
			'jquery-ui-sortable',
			'jquery-ui-datepicker'
		), RCP_PLUGIN_VERSION );
		wp_enqueue_style( 'rcp-admin', RCP_PLUGIN_URL . 'includes/css/admin-styles.css', array(), RCP_PLUGIN_VERSION );

		// Use nonce for verification
		echo '<input type="hidden" name="rcp_meta_box" value="' . wp_create_nonce( basename( __FILE__ ) ) . '" />';

		do_action( 'rcpbp_metabox_group_fields_before' );

		include RCPBP_PATH . 'includes/admin/metabox-view-group.php';

		do_action( 'rcpbp_metabox_group_fields_after' );
	}

	/**
	 * Handle extra restriction fields for member types and groups
	 */
	public function bp_fields() {

		if ( empty( $_GET['gid'] ) ) {
			return;
		}

		$group_id = absint( $_GET['gid'] );

		$member_types = apply_filters( 'rcpbp_restricted_member_types', bp_get_member_types( array(), 'objects' ) );

		wp_nonce_field( basename( __FILE__ ), 'rcpbp_meta_box' );

		if ( ! empty( $member_types ) ) {
			$this->member_type_field( $member_types, $group_id );
		}

		if ( ! bp_is_active( 'groups' ) ) {
			return;
		}

		$groups = apply_filters( 'rcpbp_restricted_groups', groups_get_groups( 'show_hidden=true' ) );

		if ( ! empty( $groups['groups'] ) ) {
			$this->groups_field( $groups['groups'], $group_id );
		}

	}

	/**
	 * Print out member type fields for restricted content section
	 *
	 * @param $member_types
	 */
	protected function member_type_field( $member_types, $group_id ) {
		$field_id = 'rcpbp_member_types';
		$selected = groups_get_groupmeta( $group_id, $field_id, true );
		?>
        <div id="rcp-metabox-field-member-types" class="rcp-metabox-field">
            <p><?php _e( 'Require member to have any one of the selected member types.', 'rcpbp' ); ?></p>
			<?php foreach ( $member_types as $name => $member_type ) : ?>
                <input type="checkbox"
                       value="<?php echo esc_attr( $name ); ?>" <?php checked( true, in_array( $name, (array) $selected ) ); ?>
                       name="<?php echo $field_id; ?>[]"
                       id="<?php echo $field_id; ?>_<?php echo esc_attr( $name ); ?>"/>&nbsp;
                <label for="<?php echo $field_id; ?>_<?php echo esc_attr( $name ); ?>"><?php echo esc_html( $member_type->labels['name'] ); ?></label>
                <br/>
			<?php endforeach; ?>
        </div>
		<?php
	}

	/**
	 * Print out group fields for the restricted content section
	 *
	 * @param $groups
	 */
	protected function groups_field( $groups, $group_id ) {
		$field_id = 'rcpbp_groups';
		$selected = groups_get_groupmeta( $group_id, $field_id, true );
		?>
        <div id="rcp-metabox-field-groups" class="rcp-metabox-field">
            <p><?php _e( 'Require member to be a part of any one of the selected groups.', 'rcpbp' ); ?></p>
			<?php foreach ( $groups as $group ) : ?>
                <input type="checkbox"
                       value="<?php echo absint( $group->id ); ?>" <?php checked( true, in_array( $group->id, (array) $selected ) ); ?>
                       name="<?php echo $field_id; ?>[]"
                       id="<?php echo $field_id; ?>_<?php echo absint( $group->id ); ?>"/>&nbsp;
                <label for="<?php echo $field_id; ?>_<?php echo absint( $group->id ); ?>"><?php echo esc_html( $group->name ); ?></label>
                <br/>
			<?php endforeach; ?>
        </div>
		<?php
	}


	/**
	 * Save meta value
	 *
	 * @param $group_id
	 */
	function save_meta_data( $group_id ) {

		// verify nonce
		if ( ! isset( $_POST['rcp_meta_box'] ) || ! wp_verify_nonce( $_POST['rcp_meta_box'], basename( __FILE__ ) ) ) {
			return;
		}

		// check autosave
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}

		if ( ! current_user_can( 'bp_moderate' ) ) {
			return;
		}

		$is_paid     = false;
		$restrict_by = sanitize_text_field( $_POST['rcp_restrict_by'] );

		switch ( $restrict_by ) {

			case 'unrestricted' :

				groups_delete_groupmeta( $group_id, 'rcp_access_level' );
				groups_delete_groupmeta( $group_id, 'rcp_subscription_level' );
				groups_delete_groupmeta( $group_id, 'rcp_user_level' );

				break;


			case 'subscription-level' :

				$level_set = sanitize_text_field( $_POST['rcp_subscription_level_any_set'] );

				switch ( $level_set ) {

					case 'any' :

						groups_update_groupmeta( $group_id, 'rcp_subscription_level', 'any' );

						break;

					case 'any-paid' :

						$is_paid = true;
						groups_update_groupmeta( $group_id, 'rcp_subscription_level', 'any-paid' );

						break;

					case 'specific' :

						$levels = array_map( 'absint', $_POST['rcp_subscription_level'] );

						foreach ( $levels as $level ) {

							$price = rcp_get_subscription_price( $level );
							if ( ! empty( $price ) ) {
								$is_paid = true;
								break;
							}

						}

						groups_update_groupmeta( $group_id, 'rcp_subscription_level', $levels );

						break;

				}

				// Remove unneeded fields
				groups_delete_groupmeta( $group_id, 'rcp_access_level' );

				break;


			case 'access-level' :

				groups_update_groupmeta( $group_id, 'rcp_access_level', absint( $_POST['rcp_access_level'] ) );

				$levels = rcp_get_subscription_levels();
				foreach ( $levels as $level ) {

					if ( ! empty( $level->price ) ) {
						$is_paid = true;
						break;
					}

				}

				// Remove unneeded fields
				groups_delete_groupmeta( $group_id, 'rcp_subscription_level' );

				break;

			case 'registered-users' :

				// Remove unneeded fields
				groups_delete_groupmeta( $group_id, 'rcp_access_level' );

				// Remove unneeded fields
				groups_delete_groupmeta( $group_id, 'rcp_subscription_level' );

				$levels = rcp_get_subscription_levels();
				foreach ( $levels as $level ) {

					if ( ! empty( $level->price ) ) {
						$is_paid = true;
						break;
					}

				}

				break;

		}

		// buddypress settings
		if ( 'unrestricted' == $restrict_by ) {
			groups_delete_groupmeta( $group_id, 'rcpbp_member_types' );
			groups_delete_groupmeta( $group_id, 'rcpbp_groups' );
		} else {
			if ( empty( $_POST['rcpbp_groups'] ) ) {
				groups_delete_groupmeta( $group_id, 'rcpbp_groups' );
			} else {
				groups_update_groupmeta( $group_id, 'rcpbp_groups', array_map( 'absint', $_POST['rcpbp_groups'] ) );
			}

			if ( empty( $_POST['rcpbp_member_types'] ) ) {
				groups_delete_groupmeta( $group_id, 'rcpbp_member_types' );
			} else {
				groups_update_groupmeta( $group_id, 'rcpbp_member_types', array_map( 'sanitize_text_field', $_POST['rcpbp_member_types'] ) );
			}
		}


		$show_excerpt = isset( $_POST['rcp_show_excerpt'] );
		$hide_in_feed = isset( $_POST['rcp_hide_from_feed'] );
		$user_role    = sanitize_text_field( $_POST['rcp_user_level'] );

		groups_update_groupmeta( $group_id, 'rcp_show_excerpt', $show_excerpt );
		groups_update_groupmeta( $group_id, 'rcp_hide_from_feed', $hide_in_feed );
		if ( 'unrestricted' !== $_POST['rcp_restrict_by'] ) {
			groups_update_groupmeta( $group_id, 'rcp_user_level', $user_role );
		}

		groups_update_groupmeta( $group_id, '_is_paid', $is_paid );

		do_action( 'rcpbp_group_meta_data_saved', $group_id );
	}

	/**
	 * Maybe restrict this group
	 *
	 * @param $can_access
	 * @param $member_id
	 * @param $post_id
	 * @param $member
	 *
	 * @return bool
	 * @author Landon Otis
	 */
	public function maybe_restrict_group( $can_access, $member_id, $post_id, $member ) {

		// Leave admins access alone.
		if ( current_user_can( 'manage_options' ) ) {
			return $can_access;
		}

		// Disallow group creation if limit is reached
		if ( ! $this->can_create_group() && bp_is_group_create() && bp_is_first_group_creation_step() ) {
			return false;
		}

		if ( ! bp_is_group() ) {
			return $can_access;
		}

		if ( ! $group = groups_get_current_group() ) {
			return $can_access;
		}

		if ( $group->is_member ) {
			return $can_access;
		}

		if ( $this->user_can_access( $group, $can_access ) ) {
			return $can_access;
		}

		return false;
	}

	public function user_can_access( $group = null, $can_access ) {

		if ( empty( $group ) ) {
			$group = groups_get_current_group();
		}

		$user_id = get_current_user_id();

		$subscription_levels = $this->get_subscription_levels( $group->id );
		$access_level        = groups_get_groupmeta( $group->id, 'rcp_access_level', true );
		$user_level          = groups_get_groupmeta( $group->id, 'rcp_user_level', true );

		$customer = rcp_get_customer_by_user_id( $user_id );

		/**
		 * Created default values depending on if a user is a customer
		 * allowing groups to be individually restricted
		 */
		if ( empty( $customer ) ) {
			$sub_ids               = [];
			$has_paid_membership   = false;
			$has_active_membership = false;
			$has_access_level      = false;
		} else {
			$sub_ids               = rcp_get_customer_membership_level_ids( $customer->get_id() );
			$has_paid_membership   = $customer->has_paid_membership();
			$has_active_membership = $customer->has_active_membership();
			$has_access_level      = $customer->has_access_level( $access_level );
		}

		if ( $this->is_paid_group( $group->id ) && ! $has_paid_membership ) {
			$can_access = false;
		}

		if ( ! empty( $subscription_levels ) ) {

			if ( is_string( $subscription_levels ) ) {

				switch ( $subscription_levels ) {
					case 'any' :
						$can_access = ! empty( $sub_id ) && ! $has_active_membership;
						break;
					case 'any-paid' :
						$can_access = $has_paid_membership;
						break;
				}

			} else {

				$needs_paid = false;

				if ( ! empty( array_intersect( $sub_ids, $subscription_levels ) ) ) {

					foreach ( $subscription_levels as $level ) {
						$paid_levels = rcp_get_paid_levels();

						if ( in_array( $level, $paid_levels ) ) {
							$needs_paid = true;
						}
					}

					if ( $needs_paid ) {
						$can_access = rcp_user_has_active_membership();
					} else {
						$can_access = true;
					}

				} else {
					$can_access = false;
				}
			}
		}

		// Customer doesn't have high enough access level
		if ( ! $has_access_level && $access_level > 0 ) {
			$can_access = false;
		}

		// User does not have valid user level
		if ( $can_access && ! empty( $user_level ) && 'All' != $user_level ) {
			if ( ! user_can( $user_id, strtolower( $user_level ) ) ) {
				$can_access = false;
			}
		}

		$member_types      = groups_get_groupmeta( $group->id, 'rcpbp_member_types', true );
		$restricted_groups = groups_get_groupmeta( $group->id, 'rcpbp_groups', true );

		// Check if the user has one of the given member types
		if ( $member_types ) {
			$has_type = false;
			foreach ( (array) $member_types as $type ) {
				if ( bp_has_member_type( $user_id, $type ) ) {
					$has_type = true;
					break;
				}
			}

			$can_access = apply_filters( 'rcpbp_member_can_access_member_types', $has_type, $user_id, $group->id, $can_access );
		}

		// Check if the user is a member of any of the given groups
		if ( $restricted_groups ) {
			$group_member = false;
			foreach ( (array) $restricted_groups as $g ) {
				if ( groups_is_user_member( $user_id, $g ) ) {
					$group_member = true;
					break;
				}
			}

			$can_access = apply_filters( 'rcpbp_member_can_access_groups', $group_member, $user_id, $group->id, $can_access );
		}

		return apply_filters( 'rcpbp_member_can_access', $can_access, $user_id, $group, $post_id = 0 );

	}

	/**
	 * Get group subscription levels
	 *
	 * @param int $group_id
	 *
	 * @return bool|mixed|void
	 */
	public function get_subscription_levels( $group_id = 0 ) {
		$levels = groups_get_groupmeta( $group_id, 'rcp_subscription_level', true );

		if ( 'all' == $levels ) {
			// This is for backwards compatibility from when RCP didn't allow content to be restricted to multiple levels
			return false;
		}

		if ( 'any' !== $levels && 'any-paid' !== $levels && ! empty( $levels ) && ! is_array( $levels ) ) {
			$levels = array( $levels );
		}

		return apply_filters( 'rcpbp_get_subscription_levels', $levels, $group_id );
	}

	/**
	 * Determine if this group is a paid group
	 *
	 * @param $group_id
	 *
	 * @return bool
	 */
	public function is_paid_group( $group_id ) {
		$return = false;

		$is_paid = groups_get_groupmeta( $group_id, '_is_paid', true );
		if ( $is_paid ) {
			$return = true;
		}

		return (bool) apply_filters( 'rcpbp_is_paid_group', $return, $group_id );
	}

}